#!/usr/local/bin/perl -w

BEGIN {
#
    unshift(@INC,"/nfs/disk100/humpub/modules");
    unshift(@INC,"/nfs/disk100/humpub/modules/PerlModules");
    unshift(@INC,"/nfs/disk80/rbsk/perl");
    unshift(@INC,"/nfs/disk80/rbsk/perl/PerlModules");
    unshift(@INC,"/nfs/disk80/rbsk/perl/PerlModules/GFF");
    # unshift(@INC,"/nfs/disk100/humpub/modules.devel/PerlModules");
    # unshift(@INC,"/nfs/disk100/humpub/modules.devel/PerlModules/GFF");
    use CGI::Carp qw(fatalsToBrowser) ;
    $ENV{PATH} = "/bin:/usr/bin:/usr/local/bin" ;
}

use strict ;
use Getopt::Std;
use vars qw($opt_h $opt_H $opt_T $opt_o $opt_l $opt_v $opt_n);
getopts("hHT:o:l:v:n");

require "/nfs/WWW/cgi-bin/sangercgi.pl";

# package CWA ;

use CGI qw(:standard) ;
use GFF ;
use GFF::Graph qw(New_Plot Range_Score_Plot Scatter_Score_Plot Print_Plot);
use GFF::GifGFF;
use GD;

$| = 1 ;

if($opt_h){
    &help;
}elsif($opt_H){
    &help2;
}

sub help {
    print <<ENDHELP;
Checks and allows manual setting of state of pair list

  -h       for this help
  -H       for pod help
  -T file  run trace/log file (i.e. dumps GFF of genes being displayed)

  -l link  specify a particular link
  -o dir   specify output directory
  -v       vertical labels
  -n       new data

ENDHELP

    exit 0;
}

sub help2 {
    exec('perldoc', $0);
}

############################################################################
################ Data content for the page  ############################################
############################################################################

### some global switches ###

my $public        = 0 ; # set this flag on to remove Sanger internal code from execution
my $debug         = 1 ; # set this flag on to activate debug code from execution; value > 1 => verbose debug

my $scriptpath = $0 ; # I'll use script name for now, to guide my schizophrenic behaviour?
my ($scriptname) = ($scriptpath =~ m|/([^/]+)$|) ;

### Custom Data paths - to be modified for web server context!!! ###

my $HUMPUB    = "/nfs/disk80/rbsk/humpub" ;       # original humace sourced data
my $NEWHUMPUB = "/nfs/disk80/rbsk/humpub" ;       # original humace sourced data

my $HUMSEQ   = "http://webace.sanger.ac.uk/cgi-bin/ace/sequence/humall?name" ;
my $HUMLOCUS = "http://webace.sanger.ac.uk/cgi-bin/ace/simple/humall?class=locus&query" ;
my $HUMSEEK  = "http://webace.sanger.ac.uk/cgi-bin/ace/simple/humall?class=sequence&query" ;
my $OUTPATH;
my $HTMLOUT;

if($opt_o){
    $OUTPATH=$opt_o;
    open(HTMLOUT,">$OUTPATH/cwa.html") || die "cannot open $OUTPATH/cwa.html";
    $HTMLOUT=\*HTMLOUT;
}else{
    $OUTPATH="/nfs/disk80/rbsk/LocalWWW/tmp" ; # target location of GIF's
    $HTMLOUT=\*STDOUT;
}
my $OUTURL   = "http://intweb.sanger.ac.uk/LocalUsers/rbsk/tmp" ;

my $LOGFILE = 0 ;
if($opt_T || $debug>1) {
    my $LOGPATH=((defined($opt_T))?$opt_T:"$OUTPATH/cwa.log");
    open(LOGFILE,"+>$LOGPATH") || die "cannot open $LOGPATH";
    $LOGFILE=\*LOGFILE;
}

my %Databases ;
if($public) {
    %Databases  = (
		   "$HUMPUB" => 'Humace',
		   ) ; 
} else {
    %Databases  = (
		   "$HUMPUB"    => 'Humace',
		   "$NEWHUMPUB" => 'Humace (rebuild-in-progress)',
		   ) ; 
}
my $Database_List = join('|',(keys %Databases)) ;

# suppress gene labels except HUGO names
my $HUGOnly=1;

# force printing of indexed gene table
my $GeneTableOn=0;

# suppress caching of web pages
my $NoCache=1 ;    

### some global data ###

my $XORIGIN = 120 ;

my %Repeats = (
	       'alu'  => 'Alu',
	       'l1'   => 'Line1',
	       'l2'   => 'Line2',
	       'tan'  => 'STR',
	       'herv' => 'HERV',
	       'ltr'  => 'LTR',
	       'the'  => 'THE',
	       'mir'  => 'MIR',
	       'mer'  => 'MER',
	       'mlt'  => 'MLT',
	       'rp'   => 'Other',
	       'tan2'  => 'Dinucleotide',
	       'tan3'  => 'Trinucleotide',
	       'tan4'  => 'Quadranucleotide',
	       'tan5'  => 'Pentanucleotide',
	       'tan'   => 'Other Tandem',
) ;
my %Repeat_Analyses = (
    'alud'  => 'Alu Repeats',
    'l1d'   => 'Line1 Repeats',
    'tan2d' => 'Dinucleotide (Microsatellite) Repeats',
    'tan3d' => 'Trinucleotide Repeats',
    'tan4d' => 'Quadranucleotide Repeats',
    'tan5d' => 'Pentanucleotide Repeats',
    'tand'  => 'Other Tandem Repeats',
    'l2d'   => 'Line2 Repeats',
    'hervd' => 'HERV Repeats',
    'ltrd'  => 'LTR Repeats',
    'thed'  => 'THE Repeats',
    'mird'  => 'MIR Repeats',
    'merd'  => 'MER Repeats',
    'mltd'  => 'MLT Repeats',
    'rpd'   => 'Other Repeats',
) ;

# Matching colours given to Range_Colour_Plot (of repeats)...
my %repeat_colour = (
    'Alu Repeats'    => [255,0,0],   # Red
    'Line1 Repeats'  => [0,0,128],   # Dark blue
    'Tandem Repeats' => [0,128,128], # Dark cyan
);
       
my %repeat_colour2 = (
    'Alu Repeats'    => [255,0,0],   # Red
    'Line1 Repeats'  => [0,0,128],   # Dark blue
    'GC Content'     => [0,128,0],   # Dark green
);
       
#... with the gifgff colours of the same elements
my %repeat_colour_name = (
    'alu' => 'red',
    'l1'  => 'darkblue',
);
	
my %Ab_Initio_Analyses = (
    'gsd'  => 'Genscan Predicted Exons',
    'fgd'  => 'Fgenes  Predicted Exons',
    'fhd'  => 'Fgenesh Predicted Exons',
) ;

my %Similarity_Analyses = (
#    'estd' => 'EST Clusters', # EST's are too numerous to work with - Michele's SPAN data more informative?
    'spd'  => 'Spangle Predicted Exons',
    'bwd'  => 'Blastwise Predicted Exons',
    'hwd'  => 'Halfwise Predicted Exons',
    'vgd'  => 'Vertebrate mRNA Predicted Exons',
) ;

my %Annotation_Analyses = (
    'tgd'  => 'Annotated Transcript Clusters',
    'ted'  => 'Annotated Exons',
    'psgd' => 'Pseudogenes',
    'cpgd' => 'CpG Islands',
) ;

my %Total_Analyses = (
    'pgf'  => 'GC Fraction',
    %Repeat_Analyses,
    %Ab_Initio_Analyses,
    %Similarity_Analyses,
    %Annotation_Analyses,
) ;

# Repeat table features
my %Features = (
		'ted'  => 'Annotated Exons',
		'psg'  => 'Pseudogenes',
                'cpg'  => 'CpG Islands',
		%Repeats, 
		) ;

my $Analysis_Types = join('|', keys %Total_Analyses) ;

#######################################
# Sequence Feature Map Data Structure #
#######################################
my %MapData ;

############################################################################
################ The header generation code  #################################
############################################################################

my $cgititle       = 'Chromosome Wide Analysis' ;
my $cgiicon        = 'humana.gif' ;
my $cgiauthor      = 'Richard Bruskiewich' ;
my $cgidescription = 'Chromosome Wide Analysis of Human Chromosomes' ;
my $cgikeywords    = 'human chromosome analysis' ;

&output_header2($cgititle,$cgiicon);

if($debug>1) {
    print $HTMLOUT p("Script path: $scriptpath") ;
    print $HTMLOUT p("Script name: $scriptname") ;
}

############################################################################
################ Text content for the pages  ############################################
############################################################################

my %paragraph = (
    'Intro' => q|
This web page provides online access to <A HREF="cwa_help.pl?help=General">chromosome wide statistics</A>
computed for human chromosomes in the Sanger Centre human high throughput genomic sequencing project.
Select the chromosome to be analyzed by picking the corresponding radio button. Select the desired
width of graph plots you would like (this should reflect your available display screen resolution).
	|,
    'Contig' => q|
This is a currently defined 'superlink' list of linked contigs/sequences on this chromosome
(Note:the superlink is <B>not</B> guaranteed to be properly ordered except for published
chromosomes; links/sequences without buttons are currently unavailable for analysis). Select the contig 
(link object) to be analyzed by picking the corresponding labelled button.
All sequence start, end and length values are in base pairs.
	|,
    'Run Parameters' => q|
Specify the start and end point (in basepairs) plus the window size and scan offset (in kilobases) 
for density calculations. One or more of analyses may be run simultaneously and plotted on 
the same graph, if more than one check box is selected.</P>
	|,
    'Analysis' => q|
An explanation of the analysis is available <A HREF="cwa_help.pl?help=Analysis">here</A>.
Additional help is available by clicking the other hyperlinks on this page (including
the row labels of the GC content and feature map).
	|,
) ;

### A small macro function ###
sub para {
    my $id = shift ;
    print $HTMLOUT p($paragraph{$id}),"\n" ;
}

############################################################################
################ The main page/form generator code  #################################
############################################################################

sub by_start {
    my($nameA,$fnameA,$startA,$endA) = split ',', $a ;
    my($nameB,$fnameB,$startB,$endB) = split ',', $b ;
    $startA+0 <=> $startB+0 ;
}

########################################
# Publication Superlink Mode #####################################################################################################################
########################################
if($scriptname =~ /^(\d+)cwa.pl$/ || $opt_o) {

    my $chr = $1 ;

    ### Display Control ###

    # Pixel width of plots
    my $PLOT_WIDTH = param("PLOT_WIDTH") ;
    if( defined($PLOT_WIDTH) and $PLOT_WIDTH =~ /^(\d+)$/ ) { # untaint
	$PLOT_WIDTH = $1 ;
    } else {
	$PLOT_WIDTH = 2000 ; # pixels
    }

    # amount of genomic sequence (in basepairs) to display on one page
    my $PAGE_SIZE = param("PAGE_SIZE") ;
    if( defined($PAGE_SIZE) and $PAGE_SIZE =~ /^(\d+)$/ ) { # untaint
	$PAGE_SIZE = $1 ;
    } else {
	$PAGE_SIZE = 2000000 ; # basepairs
    }

    # Tick marks on genomic sequence 'x' coordinates
    my $TICK_STEP = param("TICK_STEP") ;
    if( defined($TICK_STEP) and $TICK_STEP =~ /^(\d+)$/ ) { # untaint
	$TICK_STEP = $1 ;
    } else {
	$TICK_STEP = 100 ; # kilobases per tick mark
    }

    my $MAX_PAGE = param("MAX_PAGE") ;
    if( defined($MAX_PAGE) and $MAX_PAGE =~ /^(\d+)$/ ) { # untaint
	$MAX_PAGE = $1 ;
    } else {
	$MAX_PAGE = 18 ; # 18 pages @ 2,000,000 bp/page ~ 35 MB Chromosome 22 genome
    }

    my $database = param('DATABASE') ;
    if( defined($database) and $database =~ m#^($Database_List)#) { # untaint
        $database = $1 ;
    } elsif($opt_n) {
	$database = $NEWHUMPUB;
    } else {
	$database = $HUMPUB;
    }

    print $HTMLOUT p("Database: $database") if ($debug>1) ;

    ### Analysis Content ###

    my $superlink = "SUPERLINK_C$chr" ;
    my $chrp      = "$database/Chr_$chr" ;
    my $chrf      = "$chrp/Chr_$chr" ;
    my $window    = 100 ; # kilobases
    my $offset    = 50 ;  # kilobases

    my $infix     = "$window\_$offset" ;
    $infix        =~ s/\s+//g ; # make sure no whitespace creeps in...

    my %Repeats = ( 'alu'   => 'Alu', 
		    'l1'    => 'Line1',
		    'tan2'  => 'Dinucleotide',
		    'tan3'  => 'Trinucleotide',
		    'tan4'  => 'Quadranucleotide',
		    'tan5'  => 'Pentanucleotide',
		    'tan'   => 'Other Tandem',
		    ) ; # limit public version to these repeats

    my %RepeatsD = ( 'alu'  => 'Alu', 
		     'l1'   => 'Line1',
		     ) ; # limit public version to these repeats
    my $page ;

    if( defined($page = param('GotoPage')) and
	$page =~ /^(\d+)$/) {
	$page = $1 ;
    } else {
	$page = 1 ;
    }

    $page = 1        if ($page < 1) ;
    $page = $MAX_PAGE if ($page > $MAX_PAGE) ;

    my $start = 1 + ( ($page-1) * $PAGE_SIZE ) ;
    my $end   = $start - 1 + $PAGE_SIZE ;
    my $GIFilename = "$superlink-$infix-$start\_$end-$PLOT_WIDTH" ;

    print $HTMLOUT p("GIFilename: $GIFilename") if $debug>1 ;

    print $HTMLOUT h1({-align=>'center'},"Long Range Analysis of Human Chromosome $chr<BR>(Chromosome Position: $start..$end)"),"\n" ;
    
    ######################
    # Print page header  #
    # w/ index to graphs #
    ######################
    print $HTMLOUT p({-align=>'center'},"<FONT SIZE=5>Analysis from basepair $start to $end,<br>",
	    " with window of size $window kb, incrementally displaced by $offset kilobases.</FONT>"),"\n" ;
    print $HTMLOUT '<UL>' ;
    print $HTMLOUT "<LI><A HREF=\"#Map\">GC Content Plot and Sequence Feature Map</A></LI>\n"  ;
    print $HTMLOUT "<LI><A HREF=\"#Transcript_Table\">Table of Annotated Verified Transcripts</A></LI>\n" ;
    print $HTMLOUT "<LI><A HREF=\"#Pseudogene_Table\">Table of Pseudogenes</A></LI>\n" ;
    print $HTMLOUT "</UL>\n" ;

    para('Analysis') ;

    ###########################################
    # Check if image data is cached already...
    ###########################################
    my $DataIsCached = 0 ;

    # opt_o specifies overwriting also
    if( !$NoCache && -e "$OUTPATH/$GIFilename.map" && !$opt_o) {
        if( !$GeneTableOn || -e "$OUTPATH/$GIFilename\_GeneTable.html") { # check for 'Gene Table' if required

	    if( ( -e "$OUTPATH/$GIFilename.gif") ) { # check that the GIF is available too?
		if(open(GIFMAP,"<$OUTPATH/$GIFilename.map")
		   ) {
		    # ...then just serve it out...
		    print $HTMLOUT p("$OUTPATH/$GIFilename (gif/map) is cached!"), "\n" if $debug>1 ;
		    while(defined(my $line = <GIFMAP>)) { print $HTMLOUT $line ; }
		    $DataIsCached = 1 ;
		    close(GIFMAP) ;
		} else {
		    print $HTMLOUT p("$OUTPATH/$GIFilename.map could not be opened: $!?"), "\n" if $debug>1 ;
		}
	    } else {
		print $HTMLOUT p("$OUTPATH/$GIFilename.gif does not exist?"), "\n" if $debug>1 ;
	    } 
	} else {
	    print $HTMLOUT p("$OUTPATH/$GIFilename\_GeneTable.html does not exist?"), "\n" if $debug>1 ;
	} 
    } else {
	    print $HTMLOUT p("$OUTPATH/$GIFilename.map does not exist?"), "\n" if $debug>1 ;
    } 

    my $GFFtall = 0 ;
    my $GFFpall  = 0 ;
    if($GeneTableOn) {
	$GFFtall = new GFF::GeneFeatureSet(2) ;
	$GFFpall = new GFF::GeneFeatureSet(2) ; 
    }

    if(!$DataIsCached) { # ...regenerate the data!

        print $HTMLOUT p("DataIsCached: $DataIsCached!"), "\n" if $debug>1 ;
	
	##########################
	# Input Superlink GFF file
	##########################
	if(open(SUPERLINK,"<$chrf.links")) {
	    sub superlink_range_filter {
		my $left  = shift ;
		my $right = shift ;
		return sub { my $self = shift ;
			 if( ($self->start() < $right && $self->end() > $left)) 
			 {
			     return $self ;
			 } else {
			     return 0 ;
			 }
		       } ;
	    }
	    my $GFFsl = new GFF::GeneFeatureSet(2) ;
	    $GFFsl->read(\*SUPERLINK,superlink_range_filter($start,$end)) ;
	    close(SUPERLINK) ;

	    if($debug>1) {
		print $HTMLOUT p("GFF Dump of Superlink records spanning $start..$end:"),"\n"  ;
		print $HTMLOUT "<pre>" ;
		$GFFsl->dump() ;
		print $HTMLOUT "</pre>" ;
	    }

	    my $GeneIdx = 0 ; 
	    my %chrom_band ;  # collect bands for everyone...
	    my %RepDensity ;

            my $GFFgaps  = new GFF::GeneFeatureSet(2) ;
	    my $nGFFgaps = 0 ;

	    foreach my $gfl ($GFFsl->eachGeneFeature()) {
		my $lstart = $gfl->start() ;
		my $lend   = $gfl->end() ;

		if($gfl->feature() =~ /^Gap$/i) { # mark the gaps?
		    print $HTMLOUT p("A gap @ $lstart..$lend") if($debug>1);
		    # clip gap markers to viewing window
		    if($lstart > $start) {
			$nGFFgaps++ ;
			my $gfg = $gfl->copy() ;
			$gfg->seqname('bar') ;
			$gfg->feature('bar') ;
			$gfg->end($gfg->start()) ;
			$GFFgaps->addGeneFeature($gfg) ;
			print $HTMLOUT '<p>Bar @ left side:</p><pre>',$gfg->dump(),'</pre>'  if($debug>1);
		    }
		    if($lend < $end) {
			$nGFFgaps++ ;
			my $gfg = $gfl->copy() ;
			$gfg->seqname('bar') ;
			$gfg->feature('bar') ;
			$gfg->start($gfg->end()) ;
			$GFFgaps->addGeneFeature($gfg) ;
			print $HTMLOUT '<p>Bar @ right side:</p><pre>',$gfg->dump(),'</pre>' if($debug>1) ;
		    }

		} elsif($gfl->feature() =~ /^Link$/i) {
		    my $linkname = $gfl->group_value('Sequence') ;
		    my $linkfile = $gfl->group_value('Filename') ;
		    my $linkpath = "$chrp/$linkfile/$linkfile" ;

		    # clip to viewing window
		    $lstart   = ($lstart < $start ? $start : $lstart) ;
		    $lend     = ($lend   > $end   ? $end   : $lend) ;

		    # Get the relevant range of data from this link

		    ##############
		    # GC Content #
		    ##############
		    my ($GFFgc,$nGFFgc) = &GC_Content($lstart,$lend,"$linkpath-$infix") ;
		    $MapData{'GC_Content'}->{"$linkfile"}= [$GFFgc,$nGFFgc]  ;

		    ####################################
		    # Transcript map (w/o Pseudogenes) #
		    ####################################
		    my ($GFFt,$nGFFt) = &TranscriptData($linkname,$lstart,$lend,$linkpath,'tg',$GeneIdx,1,$HUGOnly) ;
		    print $HTMLOUT p("\$GFFt: $GFFt, \$nGFFt: $nGFFt") if $debug>1 ;
		    if( $nGFFt) {
			$GeneIdx += $nGFFt ;
			$MapData{'Genes'}->{"$linkfile"} = [$GFFt,$nGFFt]  ;
			$GFFtall->addGFF($GFFt) if($GeneTableOn) ;
		    }

		    ##################
		    # Pseudogene map #
		    ##################
		    my ($GFFp,$nGFFp) = &TranscriptData($linkname,$lstart,$lend,$linkpath,'psg',$GeneIdx,1,$HUGOnly) ;
		    print $HTMLOUT p("\$GFFp: $GFFp, \$nGFFp: $nGFFp") if $debug>1 ;
		    if( $nGFFp ) {
			 $GeneIdx += $nGFFp ;
			 $MapData{'Pseudogenes'}->{"$linkfile"} = [$GFFp,$nGFFp] ;
			 $GFFpall->addGFF($GFFp) if($GeneTableOn) ;
		    }

		    ##################
		    # CpG Island map #
		    ##################
		    my ($GFFc,$nGFFc) = &CpGIslandData($lstart,$lend,$linkpath) ;
		    $MapData{'CpG_Islands'}->{"$linkfile"} = [$GFFc,$nGFFc] if $nGFFc ;

		    #########################
		    # Link Contig sequences #
		    #########################
		    my (%subseq) ;
		    my ($GFFl,$nGFFl,$GFFcb,$nGFFcb) = &ContigData($linkname,\%subseq,$lstart,$lend,$linkpath,\%chrom_band,1) ;
		    print $HTMLOUT p("\$GFFcb: $GFFcb, \$nGFFcb: $nGFFcb") if $debug>1 ;
		    $MapData{'Contigs'}->{"$linkfile"}          = [$GFFl,$nGFFl]   if $nGFFl ;
		    $MapData{'Chromosome_Bands'}->{"$linkfile"} = [$GFFcb,$nGFFcb] if $nGFFcb ;

		    #######################
		    # Genethon marker map #
		    #######################
		    my ($GFFgm,$nGFFgm) = &MarkerData($lstart,$lend,$chrf,\%subseq) ;
		    print $HTMLOUT p("\$GFFgm: $GFFgm, \$nGFFgm: $nGFFgm") if $debug>1 ;
		    $MapData{'Genetic_Markers'}->{"$linkfile"} = [$GFFgm,$nGFFgm] if $nGFFgm ;

		    ################################
		    # Matrix attachment region map #
		    ################################
		    my ($GFFmar,$nGFFmar) = &MARData($lstart,$lend,$linkpath) ;
		    $MapData{'MARs'}->{"$linkfile"} = [$GFFmar,$nGFFmar] if $nGFFmar ;

		    ##############
		    # Repeat Map #
		    ##############
		    &RepeatData(\%Repeats,$lstart,$lend,$linkpath,\%MapData) ; # sets %MapData directly
		    $RepDensity{"$linkfile"} = &RepeatDensity(\%RepeatsD,$lstart,$lend,"$linkpath-$infix",$window) ; # percent coverage density
		} 
	    }

	    $MapData{'Contigs'}->{"Superlink"} = [$GFFgaps, $nGFFgaps] ;

	    ###########################################
	    # Tedious merge of all of the repeat data
	    ###########################################
	    my %ARD ;
	    foreach my $repeat (keys %RepeatsD) {
		$ARD{$Repeat_Analyses{"$repeat\d"}} = new GFF::GeneFeatureSet(2) ;
	    }
	    foreach my $linkname (keys %RepDensity) {
		my $repdata = $RepDensity{"$linkname"} ;
		next if( !defined($repdata) or 
			 ref($repdata) !~ /HASH/ or 
			 !scalar(%{$repdata})) ;
		foreach my $reptype (keys %{$repdata}) {
		    my $GFFrp = $repdata->{"$reptype"} ;
		    $ARD{$reptype}->addGFF($GFFrp) ;
		}
	    }

	    ###########################################
	    # Hashing for unique temporary file names #
	    ###########################################
	    use MD5;
	    my $md5 = new MD5;
	    my $time = time() ;

	    ############################
	    # The Heavy Workhorse Plot #
	    ############################
	    &Integrated_Plot($superlink,$start,$end,$window,
			     \%MapData,\%chrom_band,
			     \%Repeats, \%ARD, # density plot data...
			     1,$GIFilename,$PLOT_WIDTH, $TICK_STEP, 1) ;
    
    
=pod

            ####################################
	    # Some Feature Coverage Statistics #
	    ####################################
	    &FeatureCoverageTable($linkpath) ;

	    ##########################
	    # Detailed Marker Tables #
	    ##########################
	    &MarkerTable($GFFgm) ;


=cut

	    ########################
	    # Detailed Gene Table #
	    ########################
            if($GeneTableOn) {
                # cache generated gene table in the GENETABLE file...
		open(GENETABLE,"+>$OUTPATH/$GIFilename\_GeneTable.html") ;
		&GeneTables(\*GENETABLE,$GFFtall,$GFFpall,1)  ;

		# then echo to output.
		seek GENETABLE,0,0 ; 
		while(defined(my $line = <GENETABLE>)) { print $HTMLOUT $line; }
		close(GENETABLE) ;
	    }

        } else {
            print $HTMLOUT p("Could not open $chrf.links file: $!! Cannot proceed?") ;
        }
    } # if NOT cached data

    ###################
    # page navigation #
    ###################

    print $HTMLOUT '<HR>' ;

    sub by_number {
	$a+0 <=> $b+0 ;
    }

    my %range ;
    for(my $i=1 ; $i <= $MAX_PAGE; ++$i) {
	my $start = 1 + ( ($i-1) * $PAGE_SIZE ) ;
	my $end   = $start - 1 + $PAGE_SIZE ;
	$range{"$i"} = "$start..$end" ;
    }

    print $HTMLOUT start_form() ;
    print $HTMLOUT '<TABLE ALIGN="CENTER"><TR>' ;
    print $HTMLOUT '<TD>',submit('Goto Chromosome Range (basepairs):'),'</TD>' ;
    print $HTMLOUT '<TD>',popup_menu( -NAME   =>"GotoPage",
			     -VALUES => [sort by_number keys %range],
			     -LABELS => \%range,),'</TD>' ;
    print $HTMLOUT '</TR></TABLE>' ;
    print $HTMLOUT end_form() ;


########################################
# Full analysis mode, for links #####################################################################################################################
########################################
} else {


    ### Display Control ###

    my $PLOT_HEIGHT     = 500 ;
    my $CORR_PLOT_SIZE  = 800 ;
    my $HIGH_RESOLUTION  = 2000000 ;

    my @Plot_Widths ;
    if($public) {
	@Plot_Widths = ('600','800') ;
    } else {
	@Plot_Widths  = ('600','800','1000','1200','1500') ;
    }

    my $Plot_Width_List = join('|',@Plot_Widths) ;

    ### Analysis Content ###

    my @Chromosomes     = ('1', '6', '13', '22', 'X') ;
    my $Chromosome_List = join('|',@Chromosomes) ;
    my @WINDOW_SIZES = ( 50, 100 ) ; # in kilobases;  took 10 && 250 out to lessen the load

    #################
    # PAGE 1 ################
    #################

    my $chr ;


    if(!param()) {

	######################################
	# Get the chromosome number first
	######################################
	print h1({-align=>'center'},'Chromosome Wide Analysis'),"\n" ;
	para('Intro') ;

	print start_form() ;
	print p('Select chromosome:'),
	      p(radio_group( -NAME      => 'CHROMOSOME',
			     -VALUES    => \@Chromosomes,
			     -DEFAULT   => '22',
			   )
	       );
	print p('Target Database:'),
	      p(radio_group( -NAME      => 'DATABASE',
			     -VALUES    => [keys %Databases],
			     -LABELS    => \%Databases,
			   )
	       );
	print p('Select width of your plots (pixels):'),
	      p(radio_group( -NAME      => 'PLOT_WIDTH',
			     -VALUES    => \@Plot_Widths,
			     -DEFAULT   => '1000',
			       )
	       );
	print p(submit('Analyse!')) ;
	print end_form() ;

    #################
    # PAGE 2 ################
    #################
    } elsif($chr = param('CHROMOSOME')) {

	######################################
	# Then get the link object next
	######################################

	if($chr =~ /^($Chromosome_List)/) { # untaint
	    $chr = $1 ;
	} else {
	    die "Param CHROMOSOME($chr) is invalid!" ;
	}

	my $database = param('DATABASE') ;
	if($database =~ m#^($Database_List)#) { # untaint
	    $database = $1 ;
	} else {
	    die "Param DATABASE($database) is invalid!" ;
	}

	my $chrp     = "$database/Chr_$chr" ;
	my $chrf     = "$chrp/Chr_$chr" ;

	my $plot_width   = param("PLOT_WIDTH") ;
	if( $plot_width =~ /^($Plot_Width_List)$/ ) { # untaint
	    $plot_width = $1 ;
	} else {
	    print p("Param PLOT_WIDTH($plot_width) is invalid! Resetting to 800") ;
	    $plot_width = 800 ;
	}

	print h1({-align=>'center'},"Chromosome $chr Wide Analysis",(!$public?" on ".$Databases{$database}:'')),"\n" ;

	print p("Click <A HREF=\"$OUTURL/Chr_22.summary.html\">HERE</A> to go to a summary table of statistics about chromosome $chr genomic features.") ;

	my $maxlink = '' ; 
	my $maxlinksize = 0 ;
	my %SUPERLINK ;
	##########################
	# Input Superlink GFF file
	##########################
	if(open(SUPERLINK,"<$chrf.links")) {
	    my $GFFsl = new GFF::GeneFeatureSet(2) ;
	    $GFFsl->read(\*SUPERLINK) ;
	    close(SUPERLINK) ;
	    my $nGap = 0 ;
	    foreach my $gfl ($GFFsl->eachGeneFeature()) {
		my $lstart   = $gfl->start() ;
		my $lend     = $gfl->end() ;
		if($gfl->feature() =~ /^Link$/i) {
		    my $linkname = $gfl->group_value('Sequence') ;
		    my $filename = $gfl->group_value('Filename') ;

		    $SUPERLINK{"$linkname"} = "$linkname,$filename,$lstart,$lend" ;

		    my $length = $lend-$lstart+1 ;
		    if($maxlinksize < $length) {
			$maxlink     = $linkname ;
			$maxlinksize = $length ;
		    }
		} else { # is_a Gap?
		    $nGap++ ;
		    $SUPERLINK{"Gap$nGap"} = "Gap $nGap,undef,$lstart,$lend" ;
		}
	    }
	} else {
	    print p("Could not open $chrf.links?") ;
	}

	if(open(LINKMAP,"<$chrf.map") ) {
	    while(defined(my $line = <LINKMAP>)) {
		print $line ; # assuming that this is HTML...?
	    }
	    close(LINKMAP) ;
	} else {
	    print h2("Superlink List of Linked Contigs/Sequences:"),"\n" ;
	    para('Contig') ;
	    print start_form() ;
	    print p( "Select the largest link:",
		     submit( -NAME  => $maxlink,
			     -VALUE => $maxlink,
		     ),"($maxlinksize base pairs)"
		   ),"\n" ;
	    print p("<STRONG>OR</STRONG> from the table:"),"\n" ;
	    print '<TABLE BORDER="1" CELLPADDING="3" WIDTH="60%">' ;
	    print Tr(th ['<FONT SIZE=3>Link Contig/Clone Name</FONT>', '<FONT SIZE=3>Start</FONT>',
			 '<FONT SIZE=3>End</FONT>','<FONT SIZE=3>Length</FONT>' ]),"\n" ;
	    foreach my $data (sort by_start values %SUPERLINK) {
		my ($linkname,$filename,$lstart,$lend) = split ',', $data ;
		my $length = $lend-$lstart+1 ;
		if(-d "$chrp/$filename") {
		    print Tr(th(
			     [ submit( -NAME  => $linkname,
				       -VALUE => $linkname,
				      ),
			       ($lstart,$lend,$length)
			     ])
			     ), "\n"  ;
		} else {
		    print Tr(td({-align=>'center'},[ $linkname,$lstart,$lend,$length ])), "\n"  ;
		}
	    }

	    # Just so I don't have to reread in the list of link sizes...
	    print "</TABLE>\n</P>\n" ;
	    print hidden( -NAME    => "LINKLIST",
			  -DEFAULT => [ %SUPERLINK ],
		    ),"\n" ;
	    print hidden( -NAME    => "CHR",
			  -DEFAULT => $chr,
		    ),"\n" ;
	    print hidden( -NAME    => "DATABASE",
			  -DEFAULT => $database,
		    ),"\n" ;
	    print hidden( -NAME    => "PLOT_WIDTH",
			  -DEFAULT => $plot_width,
		    ),"\n" ;
	    print end_form() ;
	}

    #################
    # PAGE 3 ################
    #################
    } elsif(my %SUPERLINK = param("LINKLIST")) {

	#############################
	# Get the run parameters
	#############################
	$chr = param("CHR") ;
	if($chr =~ /^($Chromosome_List)/) { # untaint
	    $chr = $1 ;
	} else {
	    die "Param CHR($chr) has invalid characters in it!" ;
	}

	my $database = param('DATABASE') ;
	if($database =~ m#^($Database_List)#) { # untaint
	    $database = $1 ;
	} else {
	    die "Param DATABASE($database) is invalid!" ;
	}

	my $plot_width   = param("PLOT_WIDTH") ;
	if($plot_width =~ /^($Plot_Width_List)$/ ) { # untaint
	    $plot_width = $1 ;

	} else {
	    print p("Param PLOT_WIDTH($plot_width) is invalid! Resetting to 800") ;
	    $plot_width = 800 ;
	}

	print h1({-align=>'center'},"Chromosome $chr Wide Analysis",(!$public?" on ".$Databases{$database}:'')),"\n" ;
	para('Run Parameters') ;

	my ($link,$linksize,$linkname,$filename,$linkstart,$linkend) ;
	foreach my $data (sort by_start values %SUPERLINK) {
	    ($linkname,$filename,$linkstart,$linkend) = split ',', $data ;

	    # if form SUBMIT name defined for specific link?
	    if(defined($link = param($linkname))) { 
		$linksize = $linkend-$linkstart+1 ;
		last ;
	    }
	}

	print p("Link: $link\($filename\)\[$linkstart,$linkend\] selected?"),"\n" if $debug>1 ;

	##############################################################
	# Adjust default window size if necessary, for smaller clones
	##############################################################
	my $defwinsize = 10 ; 
	my $lkbsize = $linksize/1000 ; #  kilobases
	foreach my $size (@WINDOW_SIZES) {
	    last if($lkbsize < $size) ;
	    $defwinsize = $size ;
	}

	if(defined($link)) {

	    print start_form() ;
	    print "<TABLE>" ;
	    print Tr(th ['Contig', 'Range', ' ', ' ',      'Window', 'Scan']) ;
	    print Tr(th ['Name',   'Start', ' ', 'Finish', 'Size',   'Offset']) ;
	    print Tr(th [ "<B>$link</B>",
			  textfield( -NAME      => "START", # in base pairs
				     -DEFAULT   => $linkstart,
				     -SIZE      => 10 ),
			  "-",
			  textfield( -NAME      => "END", # in base pairs
				     -DEFAULT   => $linkend,
				     -SIZE      => 10 ),

			  popup_menu( -NAME      => "WINDOW", # in kilobases
				      -VALUES    => \@WINDOW_SIZES,
				      -DEFAULT   => $defwinsize ),

			  popup_menu( -NAME      => "OFFSET", # in kilobases
				      -VALUES    =>  \@WINDOW_SIZES,
				      -DEFAULT   => $defwinsize ),

			 ]
		     ) ;
	    print "</TABLE>", p() ;

	    print "<TABLE BORDER=1>" ;
	    print Tr(th (checkbox( -NAME    => "CHROM_BANDS",
				   -VALUE   => "yes",
				   -CHECKED => "on",
				   -LABEL   => " Chromosome Banding Data"
				   )
			 )
		     ) ;
	    print Tr(th (checkbox( -NAME    => "REPEAT_PLOT",
				   -VALUE   => "yes",
				   -CHECKED => "on",
				   -LABEL   => " Repeat Density Analyses:"
				   )
			 )
		     ) ;
	    print Tr( th [ checkbox_group( -NAME      => "RP_ANALTYPE",
					   -VALUES    => [sort keys %Repeat_Analyses],
					   -DEFAULT   => ['alud', 'l1d', 'tand'],
					   -LABELS    => \%Repeat_Analyses,
					   -LINEBREAK => 1,
					   -COLUMNS   => 4,
				  ),
			 ]
		    ) ;
	    print Tr(th (checkbox( -NAME    => "AB_INITIO_PLOT",
				   -VALUE   => "yes",
				   -CHECKED => "on",
				   -LABEL   => " Ab Initio Predictions Density Analyses:"
				   )
			 )
		     ) ;
	    print Tr( th [ checkbox_group( -NAME      => "AB_ANALTYPE",
					   -VALUES    => [sort keys %Ab_Initio_Analyses],
					   -DEFAULT   => ['fhd'],
					   -LABELS    => \%Ab_Initio_Analyses,
					   -LINEBREAK => 1,
					   -COLUMNS   => 4,
				  ),
			 ]
		    ) ;
	    print Tr(th (checkbox( -NAME    => "SIMILARITY_PLOT",
				   -VALUE   => "yes",
				   -CHECKED => "on",
				   -LABEL   => " Similarity Predictions Density Analyses:"
				   )
			 )
		     ) ;
	    print Tr( th [ checkbox_group( -NAME      => "SIM_ANALTYPE",
					   -VALUES    => [sort keys %Similarity_Analyses],
					   -DEFAULT   => ['spd'],
					   -LABELS    => \%Similarity_Analyses,
					   -LINEBREAK => 1,
					   -COLUMNS   => 4,
				  ),
			 ]
		    ) ;
	    print Tr(th (checkbox( -NAME    => "ANNOTATION_PLOT",
				   -VALUE   => "yes",
				   -CHECKED => "on",
				   -LABEL   => " Transcript and Related Annotation Density Analyses:"
				   )
			 )
		     ) ;
	    print Tr( th [ checkbox_group( -NAME      => "ANN_ANALTYPE",
					   -VALUES    => [sort keys %Annotation_Analyses],
					   -DEFAULT   => ['tgd','cpgd'],
					   -LABELS    => \%Annotation_Analyses,
					   -LINEBREAK => 1,
					   -COLUMNS   => 4,
				  ),
			 ]
		    ) ;
	    print Tr(th (checkbox( -NAME    => "CORRELATION_PLOT",
				   -VALUE   => "yes",
				   -CHECKED => "on",
				   -LABEL   => " Correlation Plot:"
				   ),
			 checkbox( -NAME    => "SUPPRESS_ZEROS",
				   -VALUE   => "yes",
				   -CHECKED => "on",
				   -LABEL   => " Ignore X Zero Values?"
				   )
			 )
		     ) ;
	    print Tr( th ('<B> X: </B>',
			  popup_menu( -NAME    => 'CORR_X',
				      -VALUES  => [sort keys %Total_Analyses],
				      -LABELS  => \%Total_Analyses,
				      -DEFAULT => 'tgd',
				     ),
			  '<B>Against Y: </B>',
			  popup_menu( -NAME    => 'CORR_Y',
				      -VALUES  => [sort keys %Total_Analyses],
				      -LABELS  => \%Total_Analyses,
				      -DEFAULT => 'pgf',
				    ),
			 ),
		    ) ;
	    print "</TABLE>", p() ;

	    print hidden( -NAME    => "LINK2",
			  -DEFAULT => $link,
			) ;
	    print hidden( -NAME    => "LINKSIZE",
			  -DEFAULT => $linksize,
		    ) ;
	    print hidden( -NAME    => "LINKSTART",
			  -DEFAULT => $linkstart,
		    ) ;
	    print hidden( -NAME    => "LINKEND",
			  -DEFAULT => $linkend,
		    ) ;
	    print hidden( -NAME    => "PLOT_WIDTH",
			  -DEFAULT => $plot_width,
		    ),"\n" ;
	    print p(submit(),reset("Reset Run Parameters"),
		    checkbox( -NAME   => "TABLE_OUT",
			      -VALUE  => "yes",
			      -LABEL  =>  " Print list of annotated transcripts?",
			      -CHECKED => "on",
			    ),
		     ($debug?
			checkbox( -NAME   => "VERBOSE",
				  -VALUE  => "yes",
				  -LABEL  => " Verbose Debugging Output?",
			     )
			:''),
		    ) ;
	    print hidden( -NAME    => "CHR",
			  -DEFAULT => $chr,
		    ) ;
	    print hidden( -NAME    => "DATABASE",
			  -DEFAULT => $database,
		    ),"\n" ;
	    print end_form() ;
	} else { print p("<B>No valid link object specified?</B>") ;}

    ################################
    # PAGE 4 - Do the analysis ################
    ################################
    } elsif(my $linkname = param("LINK2")) {

	###################################
	# Untaint and reap the parameters ################
	###################################
	if($linkname =~ /^((:|\w)+)$/ ) { # untaint $linkname
	    $linkname = $1 ;
	} else {
	    die "Link name $linkname has invalid characters in it!" ;
	}

	$chr = param("CHR") ;
	if($chr =~ /^($Chromosome_List)/ ) { # untaint
	    $chr = $1 ;
	} else {
	    die "Param CHR($chr) has invalid characters in it!" ;
	}

	my $database = param('DATABASE') ;
	if($database =~ m#^($Database_List)#) { # untaint
	    $database = $1 ;
	} else {
	    die "Param DATABASE($database) is invalid!" ;
	}

	my $chrp = "$database/Chr_$chr" ;
	my $chrf = "$chrp/Chr_$chr" ;

	print h1({-align=>'center'},"Chromosome $chr Wide Analysis",(!$public?" on ".$Databases{$database}:'')),"\n" ;

	my $window = param("WINDOW") ; # in kilobases
	if($window =~ /^(\d+)$/ ) { # untaint
	    $window = $1 ;
	} else {
	    $window = 100 ;
	}

	my $offset = param("OFFSET") ; # in kilobases
	if($offset =~  /^(\d+)$/ ) { # untaint
	    $offset = $1 ;
	} else {
	    $offset = 10 ;
	}

	my $verbose = param("VERBOSE")? ' -T ':"" ;

	if($offset <= $window) {

	    my (@Analyses, %Analyses, $analtype) ;
	    push(@Analyses, 'pgf') ;
	    push(@Analyses, param("RP_ANALTYPE"))  if param("REPEAT_PLOT") ;
	    push(@Analyses, param("AB_ANALTYPE"))  if param("ABINITIO_PLOT");
	    push(@Analyses, param("SIM_ANALTYPE")) if param("SIMILARITY_PLOT");
	    push(@Analyses, param("ANN_ANALTYPE")) if param("ANNOTATION_PLOT");

	    my $i ;
	    for( $i = 0; $i<= $#Analyses ; $i++) {
		if( $Analyses[$i] =~ /^($Analysis_Types)$/) { # untaint
		    $analtype = $1 ;
		    if(!exists $Total_Analyses{$analtype}) {
			 die "Unknown analysis type $analtype" ;
		    }
		    $Analyses{$analtype}= 1 ; # add to required analyses
		 } else {
		    die "Param analtype($analtype) has invalid characters in it!" ;
		 }
	    }
	    my ($CX, $CY) ;
	    if( param("CORRELATION_PLOT") ) {
		$CX = param("CORR_X") ;
		if( defined($CX) &&
		    $CX =~ /^($Analysis_Types)$/) { # untaint
		    $CX = $1 ;
		    if(!exists($Total_Analyses{$CX})) {
			die "Unknown analysis type $CX" ;
		    }
		    $Analyses{$CX}= 1 ; # add to required analyses
		} else {
		    die "Param CORR_X($CX) has invalid characters in it!" ;
		}

		$CY = param("CORR_Y") ;
		if( defined($CY) &&
		    $CY =~ /^($Analysis_Types)$/) { # untaint
		    $CY = $1 ;
		    if(!exists($Total_Analyses{$CY})) {
			die "Unknown analysis type $CY" ;
		    }
		    $Analyses{$CY}= 1 ; # add to required analyses
		} else {
		    die "Param CORR_Y($CY) has invalid characters in it!" ;
		}
	    }

	    my $linksize = param("LINKSIZE") ;
	    if($linksize =~ /^(\d+)$/) { # untaint
		$linksize = $1 ;
	    } else {
		die "Param LINKSIZE($linksize) has invalid characters in it!" ;
	    }

	    my $linkstart = param("LINKSTART") ;
	    if($linkstart =~ /^(\d+)$/) { # untaint
		$linkstart = $1 ;
	    } else {
		die "Param LINKSTART($linkstart) has invalid characters in it!" ;
	    }

	    my $linkend = param("LINKEND") ;
	    if($linkend =~ /^(\d+)$/) { # untaint
		$linkend = $1 ;
	    } else {
		die "Param LINKEND($linkend) has invalid characters in it!" ;
	    }

	    my $start = param("START") ;
	    if($start =~ /^(\d+)$/ and 
	       $start >= $linkstart and 
	       $start < $linkend ) { # untaint
		 $start = $1 ;
	    } else {
		print p("Param START($start) is invalid. Resetting to link start ($linkstart)") ;
		$start = $linkstart ;
	    }

	    my $end = param("END") ;
	    if($end =~  /^(\d+)$/ and 
	       $end > $linkstart  and 
	       $end <= $linkend     ) { # untaint
		 $end = $1 ;
	    } else {
		print p("Param END($end) is invalid. Resetting to link end ($linkend)") ;
		$end = $linkend ;
	    }

	    if($start>$end) {
		print p("Start>End? Resetting interval to valid link boundaries!"),"\n" ;
		$start = $linkstart ;
		$end   = $linkend ;
	    }

	    ##########################################################################
	    # Adjust window/offset bounds if necessary, for smaller clones/intervals 
	    ##########################################################################
	    my $interval = ($end-$start+1)/1000 ; # in kilobases
	    my $smallersize = 10 ; #  kilobases
	    foreach my $size (@WINDOW_SIZES) {
		if($interval <= $size and $window > $smallersize) {
		    print p("Your analysis interval from base $start to $end ($interval kilobases) ",
			    "is less than or equal to your analysis window ($window kilobases).",
			    " I will reset your analysis window (and maybe the offset too) to $smallersize kb.") ,"\n" ;
		    $window = $smallersize ; # reset to more realistic window...
		    if($offset > $smallersize ) {
			$offset = $smallersize ; # reset to more realistic offset...
		    }
		    last ; # finished with adjustment
		}
		$smallersize = $size ;
	    }

	    my $max_width   = param("PLOT_WIDTH") ;
	    if($max_width =~ /^($Plot_Width_List)$/ ) { # untaint
	       $max_width = $1 ;
	    } else {
	       print p("Param PLOT_WIDTH($max_width) is invalid! Resetting to 800") ;
	       $max_width = 800 ;
	    }
	    my $plot_width = $max_width - $XORIGIN ; # need to account for offset? 

    	    my $analysis ;

	    #################################
	    # Gather the density data ##################
	    #################################
	    my ( %repeat_results,     $rp_y_min,  $rp_y_max,
		 %abinitio_results,   $ab_y_min,  $ab_y_max,
		 %similarity_results, $sim_y_min, $sim_y_max,
		 %annotation_results, $ann_y_min, $ann_y_max,
	       ) ;
	    $rp_y_min = $ab_y_min = $sim_y_min = $ann_y_min = 0 ;
	    $rp_y_max = $ab_y_max = $sim_y_max = $ann_y_max = 0 ;

	    my $infix = "$window\_$offset" ; 
	    $infix =~ s/\s+//g ; # make sure no whitespace creeps in...
	    my $linkfile = $linkname ; 
	    $linkfile =~ s/em://i ; # I *hate* em: prefixes!
	    my $linkpath = "$chrp/$linkfile/$linkfile" ;

	    my $y_min = 1000 ; my $y_max = 0 ;
	    my ($CX_GFF, $CX_title, $CX_min, $CX_max, 
		$CY_GFF, $CY_title, $CY_min, $CY_max, ) ;

	    foreach $analtype (keys %Analyses) {

		# Merge the output files within defined range
		print p("File: $linkpath-$infix.$analtype") if $verbose ;

		if( open(RESULTS,"<$linkpath-$infix.$analtype") ) {
		    my $GFFi = new GFF::GeneFeatureSet ;

		    if( $analtype eq 'pgf' ) {
			$GFFi->read(\*RESULTS,&range_filter($start-5000,$end+5000) ) ;
		    } else {
			$GFFi->read(\*RESULTS,&range_filter($start-$window*500,$end+$window*500) ) ;
		    }
		    close(RESULTS) ;

		    if($debug>1) {
			print p("$analtype dump (range: $start to $end):"),"\n" ;
			print "<pre>" ;
			$GFFi->dump() ;
			print "</pre>" ;
		    }

		    my $subymin = 1000 ;
		    my $subymax = 0 ;
		    foreach my $gf ($GFFi->eachGeneFeature()) {
			if($gf->feature() ne 'Total') {
			    my $score = $gf->score() ;
			    $subymin = $score if $subymin > $score ;
			    $subymax = $score if $subymax < $score ;
			}
		    }
		    $subymax = $subymin + 1 if $subymin >= $subymax ; # might arise sometimes...

		    print "<P>$linkfile\-$infix\.$analtype\(Min Score: $subymin, Max Score: $subymax)</P>\n" if $verbose ;

		    #######################################
		    # Range score plots...
		    #######################################
		    if( defined $GFFi and $GFFi->count() ) {
			if( $analtype eq 'pgf' ) {
			    my $nGFFgc = $GFFi->count() ;
			    $MapData{'GC_Content'}->{"$linkfile"} = [$GFFi,$nGFFgc] ;

			} elsif(exists $Repeat_Analyses{$analtype}) { # segregate repeats in their own plot
			    $rp_y_min = $subymin if $rp_y_min > $subymin ;
			    $rp_y_max = $subymax if $rp_y_max < $subymax ;
			    $repeat_results{$Repeat_Analyses{$analtype}} = $GFFi ;

			} elsif(exists $Ab_Initio_Analyses{$analtype})  { # segregate 'ab initio' predictions in their own plot
			    $ab_y_min = $subymin if $ab_y_min > $subymin ;
			    $ab_y_max = $subymax if $ab_y_max < $subymax ;
			    $abinitio_results{$Ab_Initio_Analyses{$analtype}} =  $GFFi ;

			} elsif(exists $Similarity_Analyses{$analtype}) { # segregate homology predictions in their own plot
			    $sim_y_min = $subymin if $sim_y_min > $subymin ;
			    $sim_y_max = $subymax if $sim_y_max < $subymax ;
			    $similarity_results{$Similarity_Analyses{$analtype}} =  $GFFi  ;

			} else { # everything else - e.g. CpG, EST's, true transcripts etc.
			    $ann_y_min = $subymin if $ann_y_min > $subymin ;
			    $ann_y_max = $subymax if $ann_y_max < $subymax ;
			    $annotation_results{$Annotation_Analyses{$analtype}} = $GFFi ;
			}

			#################################################
			# Scatter score plots...save GFF if seen #################
			#################################################
			if(defined $CX && $CX eq $analtype ) {
			    my $no_zeros = sub {
				my $self = shift ;
				return $self->score() ;  # use score as a boolean
			    } ;
			    if(param('SUPPRESS_ZEROS')) {
				$CX_GFF = $GFFi->filter($no_zeros) ;
			    } else {
				$CX_GFF = $GFFi ;
			    }
			    $CX_title = $Total_Analyses{$analtype} ;
			    $CX_min  = $subymin ;
			    $CX_max  = $subymax ;
			    print p("CX: $CX_title(Min:$CX_min,Max:$CX_max)") if $verbose ;
			} elsif(defined $CY && $CY eq $analtype ) {
			    $CY_GFF = $GFFi ;
			    $CY_title = $Total_Analyses{$analtype} ;
			    $CY_min  = $subymin ;
			    $CY_max  = $subymax ;
			    print p("CY: $CY_title(Min:$CY_min,Max:$CY_max)") if $verbose ;
			}

		    } else {
			print p("Empty GFF RESULTS file?") if $verbose ;
		    }
		} else {
		    print p("Data for ",$Total_Analyses{$analtype}," unavailable for plotting?\n") ; 
		    next ;
		}
	    }

	    ##############################
	    # Print page header and ##################
	    # index to graphs       ##################
	    ##############################
	    print "<P>Analysis of $linkname (basepairs $start to $end), with window of size $window kb, incrementally displaced by $offset kilobases:</P>\n" ;
	    print '<UL>' ;
	    if( scalar(%MapData) ) {
		print "<LI><A HREF=\"#Map\">GC Content Plot and Sequence Feature Map</A></LI>\n"  ;
	    }
	    if( scalar(%repeat_results) ) {
		print "<LI><A HREF=\"#Repetitive\">Repeat Densities</A></LI>\n" ;
		print '<UL>' ;
		foreach my $analname (keys %repeat_results) {
		    print "<LI>$analname</LI>\n" ;
		}
		print '</UL>' ;
	    } 
	    if( scalar(%abinitio_results) ) {
		print "<LI><A HREF=\"#Ab_Initio\"><EM>Ab Initio</EM> Prediction Density</A></LI>\n" ;
		print '<UL>' ;
		foreach my $analname (keys %abinitio_results) {
		    print "<LI>$analname</LI>\n" ;
		}
		print '</UL>' ;
	    } 
	     if( scalar(%similarity_results) ) {
		 print "<LI><A HREF=\"#Similarity\">Similarity Prediction Density</A></LI>\n" ;
		print '<UL>' ;
		foreach my $analname (keys %similarity_results) {
		    print "<LI>$analname</LI>\n" ;
		}
		print '</UL>' ;
	    }
	    if( scalar(%annotation_results) ) {
		print "<LI><A HREF=\"#Annotated\">Annotated Feature Density</A></LI>\n" ;
		print '<UL>' ;
		foreach my $analname (keys %annotation_results) {
		    print "<LI>$analname</LI>\n" ;
		}
		print '</UL>' ;
	    }

	    my $Corrtitle = "Correlation Plot of $CX_title against $CY_title (in $window kb windows offset by $offset kb)" ;
	    if( defined($CX_GFF) and defined($CY_GFF) ) {
		print "<LI><A HREF=\"#Correlation_Plot\">$Corrtitle</A></LI>\n" ;
	    }

	    if(param('TABLE_OUT')) {
		print "<LI><A HREF=\"#Genetic_Marker_Table\">Table of Reference Genetic Map Markers</A></LI>\n" ;
		print "<LI><A HREF=\"#Transcript_Table\">Table of Annotated Verified Transcripts</A></LI>\n" ;
		print "<LI><A HREF=\"#Pseudogene_Table\">Table of Pseudogenes</A></LI>\n" ;
	    }
	    print "</UL>\n" ;

	    para('Analysis') ; # header text
	    print "<A NAME=\"Map\">\n" ;
	    print h3("<A HREF=\"cwa_help.pl?help=Feature_Map\">GC Content and Sequence Feature Map</A>:"),"\n" ;

	    my $segment = "$start\_$end" ;
	    $segment =~ s/\s+//g ; # no white space!

	    my $range   = $end - $start ;

	    my $x_step = ($range/500000 <= 5)? 
			   (($range/100000 <= 5)? 
			       (($range/50000 <= 5)?
				   (($range/10000 <= 5)? 1 :10)
				   :20)
			       :50) 
			   : 500 ;

	    ####################################
	    # Transcript map (w/o Pseudogenes) #
	    ####################################
            my ($GFFt,$nGFFt) = &TranscriptData($linkname,$start,$end,$linkpath,'tg',0,($range <= $HIGH_RESOLUTION),$HUGOnly) ;
            $MapData{'Genes'}->{"$linkfile"} = [$GFFt,$nGFFt] if $nGFFt ;

	    ##################
	    # Pseudogene map #
	    ##################
            my ($GFFp,$nGFFp) = &TranscriptData($linkname,$start,$end,$linkpath,'psg',$nGFFt,($range <= $HIGH_RESOLUTION),$HUGOnly) ;
            $MapData{'Pseudogenes'}->{"$linkfile"} = [$GFFp,$nGFFp] if $nGFFp ;

	    ##################
	    # CpG Island map #
	    ##################
            my ($GFFc,$nGFFc) = &CpGIslandData($start,$end,$linkpath) ;
            $MapData{'CpG_Islands'}->{"$linkfile"} = [$GFFc,$nGFFc] if $nGFFc ;

	    #########################
	    # Link Contig sequences #
	    #########################
            my (%subseq,%chrom_band) ;
            my ($GFFl,$nGFFl,$GFFcb,$nGFFcb) = &ContigData($linkname,\%subseq,$start,$end,$linkpath,\%chrom_band,($range <= $HIGH_RESOLUTION)) ;
            $MapData{'Contigs'}->{"$linkfile"}          = [$GFFl,$nGFFl]   if $nGFFl ;
            $MapData{'Chromosome_Bands'}->{"$linkfile"} = [$GFFcb,$nGFFcb] if $nGFFcb ;

	    #######################
	    # Genethon marker map #
	    #######################
            my ($GFFgm,$nGFFgm) = &MarkerData($start,$end,$chrf,\%subseq) ;
            $MapData{'Genetic_Markers'}->{"$linkfile"} = [$GFFgm,$nGFFgm] if $nGFFgm ;

	    ################################
	    # Matrix attachment region map #
	    ################################
            my ($GFFmar,$nGFFmar) = &MARData($start,$end,$linkpath) ;
            $MapData{'MARs'}->{"$linkfile"} = [$GFFmar,$nGFFmar] if $nGFFmar ;

            ##############
            # Repeat Map #
            ##############
            &RepeatData(\%Repeats,$start,$end,$linkpath,\%MapData) ; # sets %MapData directly

	    ###########################################
	    # Hashing for unique temporary file names #
	    ###########################################
	    use MD5;
	    my $md5 = new MD5;
	    my $time = time() ;

	    ############################
	    # The Heavy Workhorse Plot #
	    ############################
            my $GIFilename = $md5->hexhash("$linkfile$segment$infix\GCplusMap$time") ;
            &Integrated_Plot($linkname,$start,$end,$window,
			     \%MapData,\%chrom_band,
			     \%Repeats, \%repeat_results, # density plot data...
			     ($range <= $HIGH_RESOLUTION),
			     $GIFilename,$plot_width,$x_step) ;

	    ####################################
	    # Some Feature Coverage Statistics #
	    ####################################
            &FeatureCoverageTable($linkpath) ;
    
	    ############################
	    # Repeat Feature Densities #####
	    ############################
	    if( param("REPEAT_PLOT") and scalar(%repeat_results) ) { 
		$GIFilename = $md5->hexhash("$linkfile.$segment$infix\repeat_features$time") ;
                &DensityPlot('Repetitive','Repetitive Feature',$GIFilename,
                             \%repeat_results,\%repeat_colour,
                             $plot_width,$PLOT_HEIGHT,
			     $start/1000,$end/1000,$x_step,$window,$rp_y_max) ;
            }

	    ##################################
	    # Ab Initio Prediction Densities #
	    ##################################
	    if( param("ABINITIO_PLOT") and scalar(%abinitio_results) ) {
		$GIFilename = $md5->hexhash("$linkfile.$segment$infix\abinitio_features$time") ;
                &DensityPlot('Ab_Initio','Ab Initio Predicted Exon',$GIFilename,
                             \%abinitio_results,undef,
                             $plot_width,$PLOT_HEIGHT,
			     $start/1000,$end/1000,$x_step,$window,$ab_y_max) ;
	     }

	    ################################
	    # Similarity Feature Densities #
	    ################################
	    if( param("SIMILARITY_PLOT") and scalar(%similarity_results) ) {
		$GIFilename = $md5->hexhash("$linkfile.$segment$infix\similarity_features$time") ;
                &DensityPlot('Similarity','Similarity Predicted Exon',$GIFilename,
                             \%similarity_results,undef,
                             $plot_width,$PLOT_HEIGHT,
			     $start/1000,$end/1000,$x_step,$window,$sim_y_max) ;
            }

    	    ######################
	    # Annotated Features #
	    ######################
	    if( param("ANNOTATION_PLOT") and scalar(%annotation_results) ) {
		$GIFilename = $md5->hexhash("$linkfile.$segment$infix\annotated_features$time") ;
                &DensityPlot('Annotated','Annotated Transcript Feature',$GIFilename,
                             \%annotation_results,undef,
                             $plot_width,$PLOT_HEIGHT,
			     $start/1000,$end/1000,$x_step,$window,$ann_y_max) ;
            }

	    ###################################
	    # Correlation plot... ##################
	    ###################################
	    if( param("CORRELATION_PLOT") and defined($CX_GFF) and defined($CY_GFF) ) {
		print "<A NAME=\"Correlation_Plot\">\n" ;
		$GIFilename = $md5->hexhash("$linkfile.$segment$infix\abinitio_features$time") ;
		my ($r,$Mx,$Sx,$My,$Sy,$Sxy) = 
		      Scatter_Score_Plot( gffX     => $CX_GFF, 
					  titleX   => $CX_title, 
					  gffY     => $CY_GFF, 
					  titleY   => $CY_title,
					  filename => "$OUTPATH/$GIFilename", 
					  width    => $CORR_PLOT_SIZE, 
					  height   => $CORR_PLOT_SIZE, 
					  x_min    => $CX_min, 
					  x_max    => $CX_max, 
					  y_factor => (($CX eq 'pgf')? 1 : $window), 
					  y_min    => $CY_min,
					  y_max    => $CY_max, 
					  y_factor => (($CY eq 'pgf')? 1 : $window), 
					  legend   => 'top',
					  debug    => $verbose) ;

		# then display the plots...

		if(-e "$OUTPATH/$GIFilename.gif") {
		    print "<A NAME=\"Correlation_Plot\">\n" ;
		    print h3({-align=>'center'},"$Corrtitle"),"\n" ;

		    print  '<STRONG><P ALIGN="CENTER">' ;
		    printf "$CX_title - Mean value: %.3f; Standard Deviation:  %.3f<br>", $Mx, $Sx ;
		    printf "<P ALIGN=\"CENTER\">$CY_title - Mean value: %.3f; Standard Deviation:  %.3f<br>", $My, $Sy ;
		    printf "<P ALIGN=\"CENTER\">$CX_title and $CY_title- Linear Correlation Coefficient: %.3f, Covariance:  %.3f\n", $r, $Sxy ;
		    print  '</P></STRONG>' ;

		    # The plot itself...
		    print "<img src=\"$OUTURL/$GIFilename.gif\" width=\"$CORR_PLOT_SIZE\" height=\"$CORR_PLOT_SIZE\">","\n"   ;
		} else {
		    print p("No correlation plot GIF found?"),"\n" ;
		}
	    }

	    ##################################################
	    # Optional Tables of Transcript, etc. ##################
	    ##################################################
	    if(param('TABLE_OUT')) {
		&MarkerTable($GFFgm) ;
                &GeneTables(\*STDOUT,$GFFt,$GFFp,$HUGOnly) ;
	    }

	    ################################################################
	    # Subversively embed some low level garbage collection here:####
	    # Delete old GIF's over 1 hour old #############################
	    ################################################################
	    my $SECONDS_PER_HOUR = 60 * 60 ;
	    foreach my $giffile (`ls -1 $OUTPATH/*.gif`) {
		next if $giffile =~ /SUPERLINK/ ;
		chomp $giffile ;
		my $writetime = (stat($giffile))[9] ;
		unlink($giffile) if $writetime < ($time - $SECONDS_PER_HOUR) ; 
	    }
	} else {
	   print p("*** Error: You must select a scan offset less or equal to your window size!") ;       
	}
    } else {
	print p("*** Error: You must select a sequence contig for analysis!") ;
    }
}

&output_footer; 

####################################################
#### Differential Run Modes, Common Subroutines ######### 
####################################################

######################################
# Transcript map (w/o Pseudogenes) ##################
######################################
sub TranscriptData {
    my $linkname  = shift ;
    my $start     = shift ;
    my $end       = shift ;
    my $linkfile  = shift ; 
    my $filext    = shift ;
    my $IdxOffset = shift ; # gene count index
    my $showlabel = shift ;
    my $HUGOnly   = shift ; # flag to suppress locus labels except for HUGO names

    my $GFFt  = 0 ;
    my $nGFFt = 0 ;
    if( open(TRANSCRIPTS,"<$linkfile.$filext") ) {

	# Filter out sequence records in a range...
	sub seq_range_filter {
	    my $left  = shift ;
	    my $right = shift ;
	    return sub {
		my $self = shift ;
		if( ($self->feature() eq 'sequence') &&
		    ($self->start() < $right && $self->end() > $left) ) 
		{
		    return $self ;
		} else {
		    return 0 ;
		}
	    };
	}

        $GFFt = new GFF::GeneFeatureSet ;
	$GFFt->read(\*TRANSCRIPTS,seq_range_filter($start,$end) ) ;
	close(TRANSCRIPTS) ;

        # labellocus is used to ensure locus name only appears once.
        my %labellocus;
	foreach my $gf ($GFFt->eachGeneFeature()) {
	    $nGFFt++ ;
	    my $GFFsid = $IdxOffset + $nGFFt ; 
	    my $tseq   = $gf->group_value('Sequence') ;
	    my $type   = $gf->group_value('Type') ;
	    my $predicted_by = $gf->group_value('Predicted_by') ;
	    my $tstart = $gf->start() ;
	    my $tend   = $gf->end() ;

	    # add a mouseover label
	    my $tseq_label="[$GFFsid] $type".(defined($predicted_by)?"\{$predicted_by\}":'').":$tseq($tstart,$tend)" ;
	    $gf->addMember($tseq_label,'label');
	    $gf->group_value('Local_Seq_Id',0,$GFFsid) ;

	    # clip the gene end sequences display coordinates as necessary
	    $gf->start($start) if $tstart < $start ; 
	    $gf->end($end)     if $tend   > $end ; 

	    # add a href to gene table, or directly to
	    # AceBrowser generated fmap page for transcript?
	    if(param('TABLE_OUT')) {
		$gf->addMember("#$tseq",'href') ;
	    } else {
		$gf->addMember("$HUMSEEK=$tseq*",'href') ;
	    }

	    # Only show gene labels explicitly at high resolution?
	    my $locus = $gf->group_value('Locus') ;
	    if($showlabel and 
	       ( !$HUGOnly or 
		 (defined($locus) and $locus !~ /\./ ) )) { # HUGO names don't have '.' in them...
#		print STDERR "Debug: hugo: $locus $tseq\n";
		if(!$HUGOnly || !$labellocus{$locus}){
		    # label the gene...if label not already seen
		    my $gf2 = new GFF::GeneFeature;
		    $gf2->seqname($linkname);
		    $gf2->source($gf->source()) ;
		    $gf2->feature("label$filext");
		    $gf2->strand($gf->strand());
		    $gf2->start($tstart);
		    $gf2->end($tstart);
		    if($tseq =~ /\./ && !$HUGOnly) { # seqname has a decimal qualifier?
			$gf2->addMember("$nGFFt",'label'); # just print locus ID?
		    } else {
			$gf2->addMember("$locus",'label');  # print HUGO name directly?
		    }
		    $gf2->group('Sequence',$tseq) ;
		    $gf2->group('Type',$type) if defined($type) ;
		    $gf2->group('Predicted_by',$predicted_by) if defined($predicted_by) ;
		    $GFFt->addGeneFeature($gf2);
		    $labellocus{$locus}=1;
		}
	    }
	}
	if($debug>1) {
	    print p("$linkfile.$filext GFF dump:") ;
	    print '<pre>' ;
	    $GFFt->dump(\*STDOUT) ;
	    print '</pre>' ;
	}
    }
    return ($GFFt,$nGFFt) ;
} # end TranscriptData

###############
# CpGIslands ##################
###############
sub CpGIslandData {
    my $start    = shift ;
    my $end      = shift ;
    my $linkfile = shift ;

    my $GFFc  = 0 ;
    my $nGFFc = 0 ;
    if( open(CPGS,"<$linkfile.cpg") ) {
	sub CpG_range_filter {
		my $left  = shift ;
		my $right = shift ;
		return sub { my $self = shift ;
			     if( ($self->source() eq 'Predicted_CpG_island') &&
				 ($self->feature() ne 'Total') &&
				 ($self->start() >= $left) && 
				 ($self->end() <= $right) ) 
			     {
				 return $self ;
			     } else {
				 return 0 ;
			     }
			 } ;
	}
	$GFFc = new GFF::GeneFeatureSet ;
	$GFFc->read(\*CPGS,CpG_range_filter($start,$end) ) ;
	close(CPGS) ;

	foreach my $gf ($GFFc->eachGeneFeature()) {
	    ++$nGFFc ;
	    my $mstart = $gf->start() ;
	    my $mend   = $gf->end() ;

	    # add a mouseover label
	    my $cpg_label="CpG($mstart,$mend)" ;
	    $gf->addMember($cpg_label,'label');
	}
	if($debug>1) {
	    print p("CpG Island GFF dump:") ;
	    print '<pre>' ;
	    $GFFc->dump(\*STDOUT) ;
	    print '</pre>' ;
	}
    }
    return ($GFFc,$nGFFc) ;
} # end CpGIslandData()


############################
# Link Contig sequences ##################
############################
sub ContigData {
    my $linkname    = shift ;
    my $subseq     = shift ; # isa reference to a hash of subsequences in the link
    my $start      = shift ;
    my $end        = shift ;
    my $linkfile   = shift ;
    my $chrom_band = shift ; # isa reference to a hash of chromosome bands 
                             # hybridized by the subsequences of the link
    my $showlabel  = shift ;

    croak "CWA::ContigData(): invalid subsequence hash reference?" 
	if(!defined($subseq) or ref($subseq) !~ /HASH/) ;
    
    croak "CWA::ContigData(): invalid chromosome band hash reference?" 
	if(!defined($chrom_band) or ref($chrom_band) !~ /HASH/) ;
    
    my $GFFl   = 0 ;
    my $nGFFl  = 0 ;
    my $GFFcb  = 0 ;
    my $nGFFcb = 0 ;
    if( open(CONTIG,"<$linkfile.ctg") ) {
	sub ctg_range_filter {
	    my $left  = shift ;
	    my $right = shift ;
	    return sub { my $self = shift ;
		     if( ($self->start() < $right && $self->end() > $left)) 
		     {
			 return $self ;
		     } else {
			 return 0 ;
		     }
		   } ;
	}
	$GFFl  = new GFF::GeneFeatureSet(2) ;
	$GFFcb = new GFF::GeneFeatureSet(2) ;

	$GFFl->read(\*CONTIG,ctg_range_filter($start,$end) ) ;
	close(CONTIG) ;

	my $toggle = 0 ;
	foreach my $gf ($GFFl->order_gf()) {
	    ++$nGFFl ;
	    my $sseq   = $gf->group_value('Sequence',0) ;
	    my $sstart = $gf->start() ;
	    my $ssend  = $gf->end() ;

	    my $seqname = uc($sseq) ; # because I'm using a hash ag'in
	    $subseq->{"$seqname"} = $sstart-1 ; # record subsequence with (super)link offset

	    # clip the contig end sequences display coordinates as necessary
	    $gf->start($start) if $sstart < $start ; 
	    $gf->end($end)     if $ssend  > $end ; 

	    $toggle = !$toggle ;
	    $gf->strand($toggle?'+':'-') ; # artificially alternate strands

	    # add a mouseover label
	    my $source = $gf->group_value('Sequenced_by') ; 
	    if( defined($source) ) { 
		# cosmetic expansion of names
		$source = "Sanger Centre" if $source =~ /^SC$/ ;
		$source = "WUSTL/GSC"     if $source =~ /^GSC$/ ;
	    }
	    my $embl = $gf->group_value('EMBL') ;
	    my $ssname ;
	    if(defined($embl) and $embl) {
		$ssname = $embl ; # rename to EMBL id if available
	    } else {
		$ssname = $sseq ;
	    }
	    $ssname =~ s/em://i ; # strip off 'Em:' from either source
	    
	    my $sseq_label="$ssname\{".(defined($source)?"$source:":'')."$sseq\}($sstart,$ssend)" ;
	    $gf->addMember($sseq_label,'label');

	    # add a href to AceBrowser generated fmap page for subsequence?
	    $gf->addMember("$HUMSEQ=$sseq",'href') ;

	    # Only show sequence labels explicitly at high resolution?
	    if( $showlabel ) {
		# label the gene...
		my $gf2 = new GFF::GeneFeature;
		$gf2->seqname($linkname);
		$gf2->source($gf->source());
		$gf2->feature('labell');
		$gf2->strand($gf->strand());
		$gf2->start($gf->start());
		$gf2->end($gf->start());
		$gf2->addMember($ssname,'label');
		$gf2->group_value('Sequenced_by',0,$source) if defined($source) ;
		$gf2->group_value('EMBL',0,$embl)           if defined($embl) ;
		$GFFl->addGeneFeature($gf2);
	    }
	    if(param('CHROM_BANDS')) {
		# Grab any chromosome band data if seen
		my $cblist = $gf->group_value_list('Chromosome_Band') ;
		if(defined($cblist) and ref($cblist) =~ /ARRAY/) {
		    foreach my $band (@{$cblist}) {
			$nGFFcb++ ;
			$chrom_band->{$band} = 1 ;
			# take a copy of the feature as a band fragment
			my $gfb = $gf->copy() ; 
			$gf->source('Chromosome_Band') ;
			$gf->feature($band) ;
			$GFFcb->addGeneFeature($gfb) ; 
		    }
		}
	    }
	}
	if($debug>1) {
	    print p("Link contig GFF dump:") ;
	    print '<pre>' ;
	    $GFFl->dump(\*STDOUT) ;
	    print '</pre>' ;
	}
    }
    return ($GFFl,$nGFFl,$GFFcb,$nGFFcb) ;
} # end ContigData()

######################################
# Genethon marker map ##################
######################################
sub MarkerData {
    my $start      = shift ;
    my $end        = shift ;
    my $markerfile = shift ;

    # isa reference to a hash of sequences in this region
    my $subseq   = shift ; 

    croak "CWA::MarkerData(): invalid subsequence hash reference?" 
	if(!defined($subseq) or ref($subseq) !~ /HASH/) ;
    
    my $GFFgm  = 0 ;
    my $nGFFgm = 0 ;
    if( open(GENETICMAP,"<$markerfile.gm") ) {
	sub gm_range_filter {
	    my $left   = shift ;
	    my $right  = shift ;
	    my $subseq = shift ;
	    return sub { 
		my $self    = shift ;
		my $seqname = $self->seqname() ;
		$seqname = uc($seqname) ; # because i'ma usi'n 'ashes
		if( exists($subseq->{"$seqname"}) ) {
		    # coordinate offset of sequence in Superlink
		    my $mstart   = $self->start() ;
		    my $mend     = $self->end() ;
		    my $sloffset = $subseq->{"$seqname"} ; 

		    print p("Marker: start($mstart), end($mend), Sequence($seqname) has offset($sloffset)"),"\n" if $debug>1 ;

		    # offset the marker coordinates
		    $mstart   += $sloffset ;
		    $mend     += $sloffset ;

		    # then check if fully within range	    
		    return $self if( $mstart >= $left && $mend <= $right ) ;
		}
		return 0 ;
	    } ;
	 }
	 $GFFgm = new GFF::GeneFeatureSet ;
	 $GFFgm->read(\*GENETICMAP,gm_range_filter($start,$end,$subseq) ) ;
	 close(GENETICMAP) ;

	 print p("Genetic map: found ",$GFFgm->count()," markers in this link?") , "\n" if $debug>1 ;

	 foreach my $gf ($GFFgm->eachGeneFeature()) {
	     ++$nGFFgm ;
	     my $seqname = $gf->seqname() ;
	     my $offset = $subseq->{uc($seqname)} ;
	     $gf->remap($offset) ;
	     my $locus  = $gf->group_value('Locus') ;

	     # mouse over label
	     $gf->addMember($locus,'label');

	     # add a href to EMBL locus entry?
	     $gf->addMember("$HUMLOCUS=$locus",'href') ;

	     my $gf2 = new GFF::GeneFeature(2);
	     $gf2->seqname($seqname);
	     $gf2->source($gf->source()) ;
	     $gf2->feature('labelgm');
	     $gf2->start($gf->start());
	     $gf2->end($gf->end());
	     $gf2->addMember($locus,'label');
	     # add a href to EMBL locus entry?
	     $gf2->addMember("$HUMLOCUS=$locus",'href') ;
	     $GFFgm->addGeneFeature($gf2);
	}
	if($debug>1) {
	    print p("Genetic Map GFF dump:") ;
	    print '<pre>' ;
	    $GFFgm->dump(\*STDOUT) ;
	    print '</pre>' ;
	}
    } else {
        print p("Could not open marker file $markerfile.gm: $!!"), "\n" if $debug>1 ;
    }
    return ($GFFgm,$nGFFgm) ;
} # end MarkerData()

######################################
# Matrix attachment region map #################
######################################
sub MARData {
    my $start    = shift ;
    my $end      = shift ;
    my $linkfile = shift ;

    return (0,0) ; # Disabled for now: underlying data and scripts are not solid

    my $GFFm  = 0 ;
    my $nGFFm = 0 ;
    if( open(MARS,"<$linkfile.mar") ) {
	sub MAR_range_filter {
		my $left  = shift ;
		my $right = shift ;
		return sub { my $self = shift ;
			     if( ($self->feature() eq 'MAR') &&
				 ($self->feature() ne 'Total') &&
				 ($self->start() >= $left) && 
				 ($self->end() <= $right) ) 
			     {
				 return $self ;
			     } else {
				 return 0 ;
			     }
			 } ;
	}
	$GFFm = new GFF::GeneFeatureSet ;
	$GFFm->read(\*MARS,MAR_range_filter($start,$end) ) ;
	close(MARS) ;

	foreach my $gf ($GFFm->eachGeneFeature()) {
	    ++$nGFFm ;
	    my $mstart = $gf->start() ;
	    my $mend   = $gf->end() ;

	    # add a mouseover label
	    my $mar_label="MAR($mstart,$mend)" ;
	    $gf->addMember($mar_label,'label');
	}
	if($debug>1) {
	    print p("MAR GFF dump:") ;
	    print '<pre>' ;
	    $GFFm->dump(\*STDOUT) ;
	    print '</pre>' ;
	}
    }
    return ($GFFm,$nGFFm) ;
}

############################
# Repeat Map  ##################
############################
sub RepeatData {
    # reference to a hash of (coded) names of repeats to plot
    # values of the hash are the corresponding labels
    my $replist   = shift ; 

    my $start     = shift ;
    my $end       = shift ;
    my $linkfile  = shift ; # (link) root filename to repeat data

    my $mapdata   = shift ; # keep each repeat GFF map class in a separate hash entry

    sub repeat_range_filter {
	my $left  = shift ;
	my $right = shift ;
	return sub {
	    my $self = shift ;
	    if( $self->feature() ne 'Total' &&
		$self->start() >= $left && 
		$self->end() <= $right
		) 
	    {
		return $self ;
	    } else {
		return 0 ;
	    }
	} ;
    }
    foreach my $reptype (sort keys %{$replist}) {
	my $replabel = $replist->{$reptype} ;
	print p("Repeat Type: $reptype") if $debug>1 ;
	if( open(RESULTS,"<$linkfile.$reptype") ) {
	    my $GFFrp  = new GFF::GeneFeatureSet ;
	    my $nGFFrp = 0 ;
	    $GFFrp->read(\*RESULTS,repeat_range_filter($start,$end) ) ;
	    close(RESULTS) ;
	    foreach my $gf ($GFFrp->eachGeneFeature()) {
		$nGFFrp++ ;
		$gf->feature($reptype) ;
		my $motif;
		if($reptype=~/tan/) {
		    my $note = $gf->group_value_list('Note') ;
		    if(defined($note) and ref($note) =~ /ARRAY/) {
			foreach my $repeat (grep /^\d+\s+copies\s+\d+\s+mer/, @{$note}) {
			    my ($copies,$size,$space,$consensus) = ($repeat  =~ /^(\d+)\s+copies\s+(\d+\s+mer)(\s+([atgc]+))?$/ ) ;
			    $motif .= "$copies@".$size.(defined($consensus)?"[$consensus]":'').',' ;
			}
			chop($motif) ; # last ',' character
		    } else {
		       $motif = "Unknown" ; 
		    }
		} else {
		    $motif = $gf->group_value('Target') ;
		    $motif =~ s/^Motif:// ;
		}
		$gf->addMember("$motif(".$gf->start().",".$gf->end().")",'label');
	    }
	    if($nGFFrp) {
		if($debug>1) {
		    print p("Repeat GFF dump of $reptype features($nGFFrp found):") ;
		    print '<pre>' ;
		    $GFFrp->dump(\*STDOUT) ;
		    print '</pre>' ;
		}
		$mapdata->{"$replabel"}->{"$linkfile"} = [$GFFrp,$nGFFrp] ;
	    } else {
		print p("No $reptype features found in $linkfile.$reptype?") if($debug>1) ;
	    }
	} else {
	    print p("Could not open file $linkfile.$reptype: $!!") ;
	}
    }
    # I don't return anything - it is directly stored in $mapdata...
} # end &RepeatData()

sub range_filter {
    my $left  = shift ;
    my $right = shift ;
    return sub { my $self = shift ;
		 if( $self->start() >= $left && $self->end() <= $right ) 
		 {
		     return $self ;
		 } else {
		     return 0 ;
		 }
	     } ;
}

############################
# GC Content data  ##################
############################
sub GC_Content {
    my $start     = shift ;
    my $end       = shift ;
    my $linkpath  = shift ; # full file path to repeat data

    my $GFFgc  = 0 ;
    my $nGFFgc = 0 ;
    if( open(RESULTS,"<$linkpath.pgf") ) { 
	$GFFgc  = new GFF::GeneFeatureSet ;
	$GFFgc->read(\*RESULTS,range_filter($start-5000,$end+5000) ) ;
	close(RESULTS) ;

	if( $GFFgc->count() ) {
	    if($debug>1) {
		print p("$linkpath.pgf dump (range: $start to $end):"),"\n" ;
		print "<pre>" ;
		$GFFgc->dump() ;
		print "</pre>" ;
	    }
	} else {
	    print p("$linkpath.pgf has no density records in the range from $start to $end?"),"\n" if ($debug>1);
	}
    } else {
	print p("GC content file $linkpath.pgf could not be opened: $!!"),"\n" if ($debug>1) ;
    }
    return ($GFFgc,$nGFFgc) ;
}

############################
# Repeat Density (percent 
# density) data  
############################
sub RepeatDensity {
    # reference to a hash of (coded) names of repeats to plot
    # values of the hash are the corresponding labels
    my $replist   = shift ; 

    my $start     = shift ;
    my $end       = shift ;
    my $linkpath  = shift ; # full file path to repeat data
    my $window    = shift ; # analysis window in kb

    my $repdata = {} ;
    my $gmax = 0 ; # global max score
    foreach my $reptype (sort keys %{$replist}) {
	my $replabel = $replist->{$reptype} ;
	if( open(RESULTS,"<$linkpath.$reptype\pd") ) { # density files have a 'pd' suffix character

	    my $GFFrp = new GFF::GeneFeatureSet ;
	    $GFFrp->read(\*RESULTS,range_filter($start-$window*500,$end+$window*500) ) ;
	    close(RESULTS) ;

	    if( $GFFrp->count() ) {
		if($debug>1) {
		    print p("$reptype\pd dump (range: $start to $end):"),"\n" ;
		    print "<pre>" ;
		    $GFFrp->dump() ;
		    print "</pre>" ;
		}
		my $lmax = 0 ; # local max score
		foreach my $gf ($GFFrp->eachGeneFeature()) {
		    if($gf->feature() ne 'Total') {
			my $score = $gf->score() ;
			$lmax = $score if $lmax < $score ;
		    }
		}
		$lmax = 1 if !$lmax ; 
		$gmax = $lmax if $gmax < $lmax ;
		$repdata->{$Repeat_Analyses{"$reptype\d"}} = $GFFrp ;
	    } else {
		print p("$reptype\pd has no density records in the range from $start to $end?"),"\n" if($debug) ;
	    }
	} else {
	    print p("Data file $linkpath.$reptype\d could not be opened: $!!"),"\n" if ($debug>1) ;
	}
    }
    return $repdata ;
}

#################################
# Integrated Plot of GC Content,#
# Sequence Feature map and      #
# Repeat Density                #
#################################
sub Integrated_Plot {
    my $linkname   = shift ; # host ((super)link) sequence name 
    my $start      = shift ;
    my $end        = shift ;
    my $window     = shift ;
    my $MapData    = shift ; # a reference to a hash of GFF data 2-tuples [ref(GFF),nGFF], indexed by row sets
    my $chrom_band = shift ; # a reference to a hash of band data
    my $replist    = shift ; # reference to a hash list of repetitive map elements (names) to plot (see &RepeatData())
    my $repeatdata = shift ; # repeat density data for plotting
    my $showlabels = shift ; # boolean flag to control display of high resolution details
    my $GIFilename = shift ; # output GIF (root) filename
    my $PLOT_WIDTH = shift ; # overall width of GIF plot
    my $x_step     = shift ; # sequence X axis coordinates major divisions ('ticks') in kilobases
    my $rpgc_merge = shift ; # boolean flag, when defined and set to non-null, merges repeat and GC content plots as one

    $rpgc_merge = 0 if !defined($rpgc_merge) ;

    #########################################
    # Merge the $MapData - a hash of hashes #
    #########################################
    my $GFFgc = new GFF::GeneFeatureSet(2) ;
    my $GFFi  = new GFF::GeneFeatureSet(2) ;
    foreach my $MapDataType (keys %{$MapData}) {

	my $linkdata = $MapData->{$MapDataType} ;
	croak "CWA::Integrate_Plot(): invalid primary map data entry?"
	    if(!defined($linkdata) or ref($linkdata) !~ /HASH/) ;

	foreach my $linkname (keys %{$linkdata}) {

	    my $data = $linkdata->{"$linkname"} ;
	    croak "CWA::Integrate_Plot(): invalid secondary map data entry?"
		if(!defined($data) or ref($data) !~ /ARRAY/) ;
	    print p("CWA::Integrate_Plot(): \$MapDataType =~ /$MapDataType/ data: $data->[0],$data->[1]") if $debug > 1;

	    unless(ref($data->[0])=~/GFF/){
		print p("WARN GFF object empty for $MapDataType, $linkname"),"\n";
		next;
	    }

	    if($MapDataType =~ /GC_Content/) {
		print p("CWA::Integrate_Plot(): \$MapDataType =~ /GC_Content/ data: $data->[0],$data->[1]") if $debug > 1 ;
		$GFFgc->addGFF($data->[0]) ;
	    } else {
		if($LOGFILE && $MapDataType =~ /(Genes|Pseudogenes)/) {
		    $data->[0]->dump($LOGFILE) ;
		}
		$GFFi->addGFF($data->[0]) ;
	    }
	}
    }

    # label function (specifies what is used for text written to gif)
    my $make_label = sub{
	my $self = shift;
	return $self->getMember('label');
    };

    my @layout ;
    my @gif_colour = (
		      'brown',
		      'red',
		      'darkblue',
		      'darkcyan',
		      'lightgreen',
		      'darkmagenta',
		      'blue',
		      'lightred',
		      'cyan',
		      'darkgreen',
		      'magenta',
		      'lightblue',
		      'darkred',
		      'green',
		      ) ;
    my $num_gif_colours = 14 ;

    my $lols = -50 ;
    my $rowlabels = {
	labelgm     => 'Genethon,blue,cwa_help.pl?help=Genethon_Markers',
	transcriptF => 'Transcript(+),darkgreen,cwa_help.pl?help=Transcripts',
	pseudogeneF => 'Pseudogene(+),mediumturquoise,cwa_help.pl?help=Pseudogenes',
	cpg_island  => 'CpG Islands,darkviolet,cwa_help.pl?help=CpG_Islands',
	transcriptR => 'Transcript(-),red,cwa_help.pl?help=Transcripts',
	pseudogeneR => 'Pseudogene(-),salmon,cwa_help.pl?help=Pseudogenes',
	contigF     => 'Contig,darkblue,cwa_help.pl?help=Contigs',
	MAR         => 'MAR,magenta,cwa_help.pl?help=MARs',
	%{$replist},
	tan3        => 'Trinucleotide,darkcyan',
    } ;

    # Cytogenetic banding data
    if(scalar(%{$chrom_band})) {
	foreach my $band (sort keys %{$chrom_band}) {
	    $rowlabels->{$band} = $band ; 
	    push(@layout,[$lols,20,$band,$gif_colour[$lols],$gif_colour[$lols],1]) ;
	    ++$lols ;
	}
	push(@layout,[$lols++,20]) ;
    }

    # colour palette to be passed to GFF::Graph::New_Plot()
    # and used inside the colour descriminant function below
    my $CWA_Palette = {
	  'dodgerblue'      => [0x10,0xE90,0xFF],
	  'darkviolet'      => [0x94,0x40,0xD3], 
	  'lime'            => [0x00,0xFF,0x00],
	  'mediumseagreen'  => [0x2C,0xB3,0x61],
	  'olivedrab'       => [0x4B,0x6E,0x13],
	  'mediumturquoise' => [0x40,0xC0,0xC0],
	  'salmon'          => [0xFF,0x40,0xC0],
	  'tomato'          => [0xFF,0x70,0x40],
	  'darkgray'        => [0x40,0x40,0x40],
	  'white' => [255,255,255],
	  'gray' => [192,192,192],
	  'black' => [0,0,0],
	  'red' => [255,0,0],
	  'lightred' => [255,128,128],
	  'darkred' => [128,0,0],
	  'green' => [0,255,0],
	  'lightgreen' => [64,192,64],
	  'darkgreen' => [0,128,0],
	  'blue' => [0,0,255],
	  'lightblue' => [170,216,255],
	  'darkblue' => [0,0,128],
	  'yellow' => [255,215,0],
	  'brown' => [218,165,32],
	  'magenta' => [255,0,255],
	  'darkmagenta' => [128,0,128],
	  'cyan' => [60,220,200],
	  'darkcyan' => [0,128,128],
    } ; 

    # differential contig colour function based 
    # upon 'Sequence_by' [group] value and strand
    my $ctg_colour = sub {
	my $coltab = shift ; # a reference to a hash table of allocated colours
	my $gf     = shift ;
	return $coltab->{'black'} if !(defined($gf) and $gf) ;
	my $source = $gf->group_value('Sequenced_by') ;
	if(!$source){
	    print STDERR "WARN: Sequenced_by missing for ".$gf->seqname."\n";
	}
	my $strand = $gf->strand ;
	if($source =~ /^(Sanger|SC)/) {
	    return $coltab->{'dodgerblue'} ;
	} elsif($source =~ /Oklahoma/) {
	    return $coltab->{'darkgreen'} ;
	} elsif($source =~ /(Keio|Shimizu)/) {
	    return $coltab->{'darkred'} ;
	} elsif($source =~ /(Washington|GSC)/) {
	    return $coltab->{'brown'} ;
	} elsif($source =~ /TIGR/) {
	    return $coltab->{'darkviolet'} ;
	} else {
	    return $coltab->{'darkgray'} ;
	}
};

    # differential transcript/pseudogene colour function based 
    # upon  Sequence Locus 'Type' and 'Predicted_by' [group] value and strand
    # note: a Perl 'closure' slight of hand here permits 'custom' colours:
    # since I allocate my own GD image, I can allocate some custom
    # $ColourPalette colours (allocated in New_Plot() below), and use them here?
    my $gene_colour = sub {
	my $coltab = shift ; # a reference to a hash table of allocated colours
	my $gf     = shift ;
	return $coltab->{'black'} if !(defined($gf) and $gf) ;

	my $source = $gf->source() ;
	my $type   = $gf->group_value('Type') ;
	my $strand = $gf->strand ;

	print p("Colouring Gene Feature { Source: ", $gf->source(), 
		", Feature: ",  $gf->feature(), 
		", Sequence: ", $gf->group_value('Sequence'),
		", Type: ",  (defined($type)?"'$type'":'?'),
		", Strand: ",(defined($strand)?"'$strand'":'?')," }" ),"\n" if $debug>1 ;

	if( $type   =~ /^Pseudogene$/i or
	    $source =~ /^Pseudogene$/i ) {

	    print "<UL><LI>Pseudogene: " if $debug>1 ;

	    if($strand eq '+') {
		print "+/mediumturquoise</LI></UL>\n" if $debug>1 ;
		return $coltab->{'mediumturquoise'} ;
	    } else {
		print "+/salmon</LI></UL>\n" if $debug>1 ;
		return $coltab->{'salmon'} ;
	    }
	} elsif($type =~ /^Predicted_gene$/i) {

	    print "<UL><LI>Predicted_gene: " if $debug>1 ;

	    my $Predicted_by = $gf->group_value('Predicted_by') ;
	    if($Predicted_by =~ /Blastx/i ) {
		print "<UL><LI>Predicted by Blastx: " if $debug>1 ;
		if($strand eq '+') {
		    print "+/mediumseagreen</LI></UL></LI></UL>\n" if $debug>1 ;
		    return $coltab->{'mediumseagreen'} ;
		} else {
		    print "+/tomato</LI></UL></LI></UL>\n" if $debug>1 ;
		    return $coltab->{'tomato'} ;
		}
		print "</LI></UL>\n" if $debug>1 ;
	    } elsif($Predicted_by =~ /^(Blastn|GD_Feature)$/i) {
		print "<UL><LI>Predicted by $Predicted_by: " if $debug>1 ;
		if($strand eq '+') {
		    print "+/olivedrab</LI></UL></LI></UL>\n" if $debug>1 ;
		    return $coltab->{'olivedrab'} ;
		} else {
		    print "-/orange</LI></UL></LI></UL>\n" if $debug>1 ;
		    return $coltab->{'darkred'} ;
		}
	    } else {
		print "<UL><LI>Predicted by method undefined! " if $debug>1 ;
		if($strand eq '+') {
		    print "+/black</LI></UL></LI></UL>\n" if $debug>1 ;
		    return $coltab->{'black'} ;
		} else {
		    print "-/gray</LI></UL></LI></UL>\n" if $debug>1 ;
		    return $coltab->{'gray'} ;
		}
	    }
	} elsif($type =~ /^Gene$/i) {

	    print "<UL><LI>Gene: " if $debug>1 ;

	    if($strand eq '+') {
		print "+/Lime</LI></UL>\n" if $debug>1 ;
		return $coltab->{'lime'} ;
	    } else {
		print "+/Red</LI></UL>\n" if $debug>1 ;
		return $coltab->{'red'} ;
	    }
	} else {
	    if($strand eq '+') {
		print "-/black</LI></UL></LI></UL>\n" if $debug>1 ;
		return $coltab->{'black'} ;
	    } else {
		print "-/gray</LI></UL></LI></UL>\n" if $debug>1 ;
		return $coltab->{'gray'} ;
	    }
	}
    };

    my $trimer_repeat_colour = sub {
	my $coltab = shift ; # a reference to a hash table of allocated colours
	my $gf     = shift ;
	return $coltab->{'black'} if !(defined($gf) and $gf) ;
	if($gf->CAG) {
	    return $coltab->{'magenta'} ;
	} else {
	    return $coltab->{'darkmagenta'} ;
	}
    } ;

    #####################################
    # Sequence and Contig Map  ##################
    #####################################
    if(exists($MapData->{'Genes'})) {
	if( $showlabels ) {
	    if($opt_v){
		push(@layout, [$lols++,50, 'labeltg', $gene_colour, 'VBF', $make_label]);
	    }else{
		push(@layout, [$lols++,8, 'labeltg', $gene_colour, 'F', $make_label]);
	    }
	    push(@layout, [$lols++,5]) ; 
	}
	push( @layout,[$lols++,20,'transcript',$gene_colour,'F', 1]) ;
	push( @layout,[$lols++,5]) ;
    }
    if(exists($MapData->{'Pseudogenes'})) {
	if( $showlabels ) {
	    if($opt_v){
		push(@layout, [$lols++,50, 'labelpsg', $gene_colour, 'VBF', $make_label]) ;
	    }else{
		push(@layout, [$lols++,8, 'labelpsg', $gene_colour, 'F', $make_label]) ;
	    }
	    push(@layout, [$lols++,5]) ;
	}
	push( @layout,[$lols++,20,'pseudogene',$gene_colour,'F', 1]) ;
	push( @layout,[$lols++,5]) ;
    }
    if(exists($MapData->{'CpG_Islands'})) {
	push( @layout,[$lols++,5]) ;
	push( @layout,[$lols++,20,'cpg_island','darkviolet','darkviolet',1]) ;
	push( @layout,[$lols++,10]) ;
    }
    if(exists($MapData->{'Genes'})) {
	if( $showlabels ) {
	    if($opt_v){
		push( @layout, [$lols++,50,'labeltg', $gene_colour, 'VBR', $make_label]) ;
	    }else{
		push( @layout, [$lols++,8,'labeltg', $gene_colour, 'R', $make_label]) ;
	    }
	    push( @layout, [$lols++,5]) ;
	}
	push( @layout,[$lols++,20,'transcript',$gene_colour,'R',1]) ;
	push( @layout,[$lols++,5]) ;
    }
    if(exists($MapData->{'Pseudogenes'})) {
	if( $showlabels ) {
	    if($opt_v){
		push(@layout, [$lols++,50,'labelpsg', $gene_colour, 'VBR', $make_label]) ;
	    }else{
		push(@layout, [$lols++,8,'labelpsg', $gene_colour, 'R', $make_label]) ;
	    }
	    push(@layout, [$lols++,5]) ;
	}
	push( @layout,[$lols++,20,'pseudogene',$gene_colour,'R',1]) ;
	push( @layout,[$lols++,50]) ;
    }
    if(exists($MapData->{'Contigs'})) {
	if( $showlabels ) {
	    if($opt_v){
		push(@layout, [$lols++,70,'labell', $ctg_colour,  'VBF', $make_label]) ;
	    }else{
		push(@layout, [$lols++,8,'labell', $ctg_colour,  'F', $make_label]) ;
	    }
	    push(@layout, [$lols++,5]) ; 
	}
	push( @layout,[$lols++,15,'contig',$ctg_colour,'F']) ;
	$lols=1;
	push( @layout,[$lols++,15,'contig',$ctg_colour,'R']) ;
	if( $showlabels ) {
	     push(@layout, [$lols++,5]) ;
	     if($opt_v){
		 push(@layout, [$lols++,70,'labell', $ctg_colour,  'VR', $make_label]) ;
	     }else{
		 push(@layout, [$lols++,8,'labell', $ctg_colour,  'R', $make_label]) ;
	     }
	}
	push( @layout,[$lols++,10]) ;
    }
    if(exists($MapData->{'Genetic_Markers'})) {
	push( @layout,[$lols++,10,'Genetic_Marker','blue','blue']) ;
	push( @layout,[$lols++,5]) ;
	if($opt_v){
	    push( @layout,[$lols++,50,'labelgm','blue','V', $make_label]) ;
	}else{
	    push( @layout,[$lols++,8,'labelgm','blue','blue', $make_label]) ;
	}
	push( @layout,[$lols++,10]) ;
    }

    if(exists($MapData->{'MARs'})) { 
	push( @layout,[$lols++,15,'MAR','magenta','magenta']) ;
    }

    # Repeat Map
    my $nrep = 0 ;
    foreach my $reptype (sort keys %{$replist}) {
	my $replabel = $replist->{$reptype} ;
	next if(!exists($MapData->{"$replabel"})) ; # skip if no data to plot?
        $nrep++ ;
	my $colour ;
	if($reptype eq 'tan3') { # treat trinucleotide repeats differently
	    $colour = $trimer_repeat_colour ;
	} elsif(exists($repeat_colour_name{$reptype})) {
	    $colour = $repeat_colour_name{$reptype} ;
	} else {
	    $colour = $gif_colour[$nrep % $num_gif_colours] ;
	}
	push(@layout,[$lols+2*$nrep-1,5]) ;
	push(@layout,[$lols+2*$nrep, 20,$reptype,$colour,$colour,1]) ;
	if($debug>1) {
	    print p("Repeat Type $reptype\($replabel\), Colour: $colour"), "\n" ;
	}
    }

    my $gf ;
    $gf = new GFF::GeneFeature;
    $gf->seqname($linkname);
    $gf->source('Link') ;
    $gf->feature('Subsequence'); # dummy subsequence
    $gf->start($start);
    $gf->end($start);
    $gf->addMember('Left Analysis End','label');
    $GFFi->addGeneFeature($gf);

    $gf = new GFF::GeneFeature;
    $gf->seqname($linkname);
    $gf->source('Link') ;
    $gf->feature('Subsequence');
    $gf->start($end);
    $gf->end($end);
    $gf->addMember('Right Analysis End','label');
    $GFFi->addGeneFeature($gf);

    ############################################
    # First, compute final height of GC plot and 
    # map (echo map layout too, if $debug>1)
    ############################################
    print p("Layout Array:"), "\n" if($debug>1) ;

    my $GC_Plot_Height;
    if($rpgc_merge) {
 	$GC_Plot_Height = 250 ;
    } else {
 	$GC_Plot_Height = 150 ;
    }
    my $Repeat_Plot_Height = 200 ;
    my $plot_height        = $GC_Plot_Height ;
    $plot_height += $Repeat_Plot_Height  if( !$rpgc_merge && scalar(%{$repeatdata}) ) ;
    foreach my $row (@layout) {
	$plot_height += $row->[1] ;
	print p(join ' - ',@{$row}),"\n" if $debug>1 ;
    }
    $plot_height += 320 ; # add some pixels for a couple of row bumping spaces...
    print p("GC Plot Height: $GC_Plot_Height, Repeat Plot Height: $Repeat_Plot_Height, Total: Plot_Height: $plot_height "),"\n" if($debug>1) ;

    ##########################   
    # Allocate the GD::Image
    ##########################
    my $image = New_Plot( width    => $PLOT_WIDTH,
			  height   => $plot_height,
			  palette  => $CWA_Palette, 
			 ) ;
    my $plot_width = $PLOT_WIDTH * 0.90 ; # cosmetic fudge factor

    # add legend
    my $startx=210;
    my $starty=1000;
    my $black=$CWA_Palette->{'black'};
    #      0   1   2   3   4   5   6
    my @x=(  0,100,200,400,500,600,700,900);
    my @y=(0,20,40,60,80,100);

    # labs
    &label_gif($image,$startx,$starty,$x[0],$y[0],'',
	       'Laboratories',$black,1);
    &label_gif($image,$startx,$starty,$x[0],$y[1],$CWA_Palette->{'dodgerblue'},
	       'Sanger',$black);
    &label_gif($image,$startx,$starty,$x[0],$y[2],$CWA_Palette->{'darkred'},
	       'Keio',$black);
    &label_gif($image,$startx,$starty,$x[1],$y[1],$CWA_Palette->{'darkgreen'},
	       'Oklahoma',$black);
    &label_gif($image,$startx,$starty,$x[1],$y[2],$CWA_Palette->{'brown'},
	       'GSC',$black);
    &label_gif($image,$startx,$starty,$x[2],$y[1],$CWA_Palette->{'darkgray'},
	       'TIGR',$black);

    # genes
    &label_gif($image,$startx,$starty,$x[3],$y[0],'',
	       'Genes',$black,1);
    &label_gif($image,$startx,$starty,$x[3],$y[1],'',
	       'Transcript(+)',$black);
    &label_gif($image,$startx,$starty,$x[3],$y[2],'',
	       'Transcript(-)',$black);
    &label_gif($image,$startx,$starty,$x[4],$y[0],'',
	       'Known gene',$black);
    &label_gif($image,$startx,$starty,$x[4],$y[1],$CWA_Palette->{'lime'},
	       '',$black);
    &label_gif($image,$startx,$starty,$x[4],$y[2],$CWA_Palette->{'red'},
	       '',$black);
    &label_gif($image,$startx,$starty,$x[5],$y[0],'',
	       'Like gene',$black);
    &label_gif($image,$startx,$starty,$x[5],$y[1],$CWA_Palette->{'mediumseagreen'},
	       '',$black);
    &label_gif($image,$startx,$starty,$x[5],$y[2],$CWA_Palette->{'tomato'},
	       '',$black);
    &label_gif($image,$startx,$starty,$x[6],$y[0],'',
	       'Predicted gene',$black);
    &label_gif($image,$startx,$starty,$x[6],$y[1],$CWA_Palette->{'olivedrab'},
	       '',$black);
    &label_gif($image,$startx,$starty,$x[6],$y[2],$CWA_Palette->{'darkred'},
	       '',$black);


#    &label_gif($image,$startx,$starty,$x[0],$y[0],$CWA_Palette->{''},
#	       '',$black);

    ###################   
    # GC Content Plot ##################
    ###################
    my $map_leftmargin  = 120 ; 
    my $map_rightmargin = 20 ;
    my $GFFgaps;
    my $flag_addgaps=1;
    if($flag_addgaps && $MapData{'Contigs'}->{"Superlink"}){
	$GFFgaps=$MapData{'Contigs'}->{"Superlink"}->[0];
	my $count=$GFFgaps->count;
	print "$count gap GFF entries\n" if($debug>1) ;
    }
    if(exists($MapData->{'GC_Content'}) or 
       ($rpgc_merge && scalar(%{$repeatdata})) 
       ) {
	my %density_data = ('GC Content' => $GFFgc) ;
 	%density_data = (%density_data, %{$repeatdata}) if($rpgc_merge) ;
 	print "<P>\%density_data:",(join "\t",%density_data),'</P></BODY></HTML>' if $debug>1 ;
	my($x,$y,$w,$h) = 
	    Range_Score_Plot( image     => $image,
			      xorigin   => $XORIGIN,
			      x_offset  => 120,
			      series    => \%density_data, 
			      colours   => \%repeat_colour2,
			      gaps      => $GFFgaps,
			      width     => $plot_width,
			      height    => $GC_Plot_Height, 
			      no_x_axis => ($rpgc_merge ? 0 : 1), # suppress X axis coordinates here?
			      x_min     => $start/1000, 
			      x_max     => $end/1000, 
			      x_step    => $x_step, # kilobases x axis major divisions
			      y_min     => 0.40,
			      y_max     => 0.60, 
			      y_step    => 0.05,
			      y_title   => "GC Fraction",
			      y2_axis   => ($rpgc_merge ? $repeatdata :undef),
			      y2_min    => 0.0,
			      y2_max    => 99,
			      y2_step   => 0.2,
			      y2_factor => 100.0, # convert fractional data to percentage
			      y2_title  => "Percentage Alu/Line1 Repeats",
			      legend    => ($rpgc_merge ? 'top,left':'top'),
			      legend2   => ($rpgc_merge ? 'top,right':undef),
			      no_totals => 1, # suppress totals
			      debug     => ($debug>1),
			      ) ;

	$map_leftmargin  = $x ;   
	$map_rightmargin = $plot_width -($x+$w) ; 
    } else {
	print p("No GC Fraction data?"),"\n" ;
    }

    # to map gff features->layout labels
    my $group_features = sub{
	my $self    = shift;
	my $type    = $self->group_value('Type') ;
	my $source  = $self->source() ;
	my $feature = $self->feature() ;
	if(!(defined($source) and defined($feature))) {
	    if($debug) {
		print "<pre>\CWA::\$group_features(): warning: suspicious feature: " ;
		$self->dump() ;
		print "\n</pre>" ;
	    }
            return 'Unknown' ; 
        }
	if(  defined($type) and 
	    ($type =~ /Pseudogene/i or
	     $source =~ /^Pseudogene$/i)) {
	    return 'pseudogene' ;
	} elsif( $source =~ /^Predicted_CpG_island$/i) {
	    return 'cpg_island' ;
	} elsif( $feature =~ /^sequence$/i) {
	    return 'transcript' ;		
	} elsif( $feature =~ /^Subsequence$/i) {
	    return 'contig' ;		
	} else {
	   return $feature ; # including all labels
	}
    };

    ###################   
    # Sequence Map ##################
    ###################
    my $map_topmargin = $GC_Plot_Height+20;

    # caching the map in $GIFilename.map
    if( !open(MAP,"+>$OUTPATH/$GIFilename.map") ) { 
	print p("CWA::Integrated_Plot: cannot open $GIFilename.map file for output: $!!") ;
	return ;
    } else {
	print MAP "<map name=\"sequence_map\">\n" ;
	my $scale = ($end-$start)/($plot_width-($map_leftmargin+$map_rightmargin)-10) ; # best approximation for width...
	my($x,$y)=&GFF::GifGFF::gff2gif( width       => $plot_width,
					 scale       => $scale,
					 gff         => $GFFi,
					 layout      => \@layout,
					 bump        => 0,  # turn bumping off for now - 'tis problematic?
					 filter      => $group_features,
					 image       => $image,
					 palette     => $CWA_Palette,
					 leftmargin  => $map_leftmargin,
					 topmargin   => $map_topmargin,
					 rightmargin => $map_rightmargin,
					 rowlabels   => $rowlabels,
					 barwidth    => 1, # 1 pixel gap lines?
					 mio         => \*MAP,
					 bumplabel   => 1,
					 resolution  => 100,
					 bumplabelseparation => 0.9,
					 );
	print MAP "</map>\n" ;
    }

    ###################################################
    # Embedded Repeat Feature Density plot ##################
    ###################################################
    if( !$rpgc_merge and scalar(%{$repeatdata}) ) { 
	Range_Score_Plot( image    => $image,
			  series   => $repeatdata,
			  colours  => \%repeat_colour,
			  width    => $plot_width, 
			  xorigin  => $XORIGIN,
			  x_offset => 120,
			  height   => $Repeat_Plot_Height, 
			  yorigin  => $plot_height-$Repeat_Plot_Height,
			  x_min    => $start/1000, 
			  x_max    => $end/1000, 
			  x_step   => $x_step, # kilobases x axis major divisions
			  y_min    => 0,
			  y_max    => 2,  # 2/kb - y_factor will multiply...
			  # y_step => 0.05, 
			  y_factor => $window, 
			  y_title  => "Repeats/$window kb",
			  legend   => 'bottom',
			  debug    => ($debug>1),
			  ) ;

    } else {
	print p("No repeat feature density plot data?"),"\n" if !$rpgc_merge ;
    }

    ##################################################
    # Plot the new version GC+repeat+sequence Map ##################
    ##################################################
    Print_Plot( image    => $image,
		filename => "$OUTPATH/$GIFilename.gif",
		) ;
    print MAP "<img src=\"$OUTURL/$GIFilename.gif\" usemap=\"#sequence_map\" width=$PLOT_WIDTH height=$plot_height border=0>\n" ;
    close(MAP) ;

    # Now, echo the map...
    if( !open(MAP,"<$OUTPATH/$GIFilename.map") ) {
	print p("CWA::Integrated_Plot: cannot open $GIFilename.map file for input: $!!") ;
	return ;
    } else {
	# echo <map> to stdout
	while(defined(my $line = <MAP>)) { print $HTMLOUT $line ; }
    }
    close(MAP) ;
} # end Integrated_Plot

sub DensityPlot {
    my $bookmark  = shift ;
    my $title     = shift ;
    my $filename  = shift ;
    my $results   = shift ; # a reference to a series hash of GFF sets
    my $colours   = shift ; # a reference to a series hash of $RGB colour arrays
    my $width     = shift ;
    my $height    = shift ;
    my $x_min     = shift ; # in kilobases
    my $x_max     = shift ; # in kilobases
    my $x_step    = shift ; # in kilobases
    my $window    = shift ; # sequence window (in kilobases)
    my $max_score = shift ;
    
    ######################################
    # Ab Initio Prediction Densities ##################
    ######################################
    Range_Score_Plot( filename => "$OUTPATH/$filename", 
		      series   => $results, 
		      colours  => $colours,
		      width    => $width, 
		      height   => $height, 
		      xorigin  => $XORIGIN,
		      x_offset => 120,
		      x_min    => $x_min, 
		      x_max    => $x_max, 
		      x_step   => $x_step, 
		      y_min    => 0,
		      y_max    => $max_score, 
		      y_factor => $window, 
		      y_title  => "$title Density",
		      legend   => 'top',
		      debug    => ($debug>1)) ;

    # then display the plots...

    if(-e "$OUTPATH/$filename.gif") {
	print "<A NAME=\"$bookmark\">\n" ;
	print h3("$title Density Plot:"),"\n" ;
	print "<img src=\"$OUTURL/$filename.gif\" width=\"$width\" height=\"$height\">","\n"   ;
    } else {
	print p("No $title density plot generated from the data?"),"\n" ;
    }
}
    
    
sub FeatureCoverageTable {
    
    my $filename = shift ;
    
    #####################################
    # Repeat Count/Coverage Table ##################
    #####################################
    print h3({-align=>'center'},"Feature Coverage Table"),"\n" ;

    my $total_filter = sub {
	my $self = shift ;
	if($self->feature() =~ /^Total$/i) 
	{
	    return $self ;
	} else {
	    return 0 ;
	}
    } ;
    my @Totals ;
    foreach my $featype (keys %Features) {
	my $feat_name = $Features{"$featype"} ;
	$feat_name .= ' Repeats' if exists $Repeats{"$featype"} ;
	my $GFFtotal = 0 ;
	if( open(FEATIN,"<$filename.$featype") ) {
	    print p("Reading 'Total' records from $filename.$featype?"),"\n" if $debug>1 ;
	    $GFFtotal = new GFF::GeneFeatureSet(2) ;
	    $GFFtotal->read(\*FEATIN, $total_filter) ;
	    close(FEATIN) ;

	    # should only find a single 'Total' record?
	    my $gftotal = $GFFtotal->nextGeneFeature() ;
	    if(defined($gftotal)) {
		push(@Totals,[$feat_name,$gftotal]) ;
	    } else {
		print p("<B>### No 'Total' record found in $filename.$featype? ###</B>"),"\n" if $debug>1 ;
		next ;
	    }
	} else {
	    print p("Could not open $filename.$featype for reading:: $!!"),"\n" if $debug>1 ;
	}
    }
    print "<TABLE BORDER=1 ALIGN=\"CENTER\">\n" ;
    print "<TR><TH>Feature<BR>Name</TH><TH>Total<BR>Number</TH>".
	  "<TH>Coverage<BR>(basepairs)</TH><TH>Percent<BR>Coverage</TH></TR>\n" ;
    sub by_descending_coverage {
	my $acov = $a->[1]->group_value('Coverage') ;
	$acov = 0 if !defined($acov) ;
	my $bcov = $b->[1]->group_value('Coverage') ;
	$bcov = 0 if !defined($bcov) ;
	$bcov<=>$acov  ;
    }
    foreach my $t (sort by_descending_coverage @Totals) {
	my $feat_name = $t->[0] ;
	my $gf        = $t->[1] ;
	my $count    = $gf->score() ;
	my $coverage = $gf->group_value('Coverage') ;
	my $pcc      = $gf->group_value('Percent_Coverage') ;
	    if(defined($coverage) and defined($pcc)) {
		print "<TR>" ;
		print "<TD ALIGN=\"CENTER\">$feat_name</TD>" ;
		print "<TD ALIGN=\"CENTER\">$count</TD>" ;
		print "<TD ALIGN=\"CENTER\">$coverage</TD>" ;
		print "<TD ALIGN=\"CENTER\">$pcc</TD>\n" ;
		print "</TR>\n" ;
	    }
    }
    print "</TABLE>\n" ;
}
    
####################################################
# Table of Genetic (Genethon) Markers
####################################################
sub MarkerTable {
    my $htmlfile   = shift ;
    my $GFFgm = shift ;

    if($GFFgm && $GFFgm->count()) {
	sub by_map_order {
	    $a->group_value('Map_Order')+0 <=> $b->group_value('Map_Order')+0
	} 
	print $htmlfile  "<A NAME=\"Genetic_Marker_Table\">\n" ;
	print $htmlfile  h3("Table of Reference Genetic Map Markers"),"\n" ;
	print $htmlfile  '<TABLE BORDER=1>' ;
	print $htmlfile  "<TR ALIGN=\"CENTER\">" ;
	print $htmlfile  "<TD><B>Map<br>Order</B></TD><TD><B>Locus</B></TD><TD><B>Sex Ave.<br>Distance<br>to next locus</B></TD>" ;
	print $htmlfile  "<TD><B>Genethon Id.</B></TD><TD><B>EMBL Id.</B></TD><TD><B>Host<br>Sequence</B></TD>" ;
	print $htmlfile  "<TD><B>Start</B></TD><TD><B>End</B></TD></TR>\n" ;
	foreach my $gf (sort by_map_order $GFFgm->eachGeneFeature()) {

	    my $feature = $gf->feature() ;
	    next if $feature =~ /Genetic_Marker/i ;

	    print $htmlfile  '<TR>' ;
	    print $htmlfile  '<TD ALIGN="CENTER">',$gf->group_value('Map_Order'),'</TD>' ;
	    print $htmlfile  '<TD ALIGN="CENTER">',$gf->group_value('Locus'),'</TD>' ;
	    print $htmlfile  '<TD ALIGN="CENTER">',$gf->score(),'</TD>' ;
	    print $htmlfile  '<TD ALIGN="CENTER">',$gf->group_value('Alias'),'</TD>' ;
	    print $htmlfile  '<TD ALIGN="CENTER">',$gf->group_value('EMBL_ID'),'</TD>' ;
	    print $htmlfile  '<TD ALIGN="CENTER">',$gf->seqname(),'</TD>' ;
	    print $htmlfile  '<TD ALIGN="CENTER">',$gf->start(),'</TD>' ;
	    print $htmlfile  '<TD ALIGN="CENTER">',$gf->end(),'</TD>' ;
	    print $htmlfile  "</TR>\n" ;
	}
	print $htmlfile  '</TABLE>' ;
    }
}
    
#############################################################
# Common code for GeneTables display of genes OR pseudogenes
#############################################################
sub SeqTabOut {
    my $tabfile   = shift ;
    my $GFFs       = shift ;
    my $WithSeqIds = shift ;
    $WithSeqIds = 0 if !defined($WithSeqIds) ;

    print $tabfile  '<TABLE BORDER=1>' ;
    print $tabfile  "<TR ALIGN=\"CENTER\">" ;

    print $tabfile  "<TD><B>No.</B></TD>" if $WithSeqIds ;

    print $tabfile  "<TD><B>Locus</B></TD><TD><B>Type</B></TD><TD><B>Predicted by</B></TD>";
    # print $tabfile  "<TD><B>Description</B></TD>" ;
    print $tabfile  "<TD><B>OMIM</B></TD><TD><B>Sequence</B></TD><TD><B>Start</B></TD><TD><B>End</B></TD><TD><B>Strand</B></TD></TR>\n" ;
    foreach my $gf ($GFFs->eachGeneFeature()) {
	next if($gf->feature() ne 'sequence') ;

	my $seqid        = $gf->group_value("Local_Seq_Id")  ;

	my $locus        = $gf->group_value('Locus') ;
	my $type         = $gf->group_value('Type') ;
	my $predicted_by = $gf->group_value('Predicted_by') ;
	my $desc         = $gf->group_value('Description') ;
	$desc = '?' if !(defined($desc) and $desc and $desc !~ /^[\s]*$/) ;
	my $OMIM         = $gf->group_value('OMIM') ;
	$OMIM = '?' if !(defined($OMIM) and $OMIM and $OMIM !~ /^[\s]*$/) ;
	my $seqname      = $gf->group_value('Sequence') ;

	print $tabfile  '<TR>' ;
	print $tabfile  "<TD ALIGN=\"CENTER\">$seqid</TD>" if $WithSeqIds ;
	print $tabfile  "<TD ALIGN=\"CENTER\"><A HREF=\"$HUMLOCUS=$locus\">$locus</A></TD>" ;
	print $tabfile  '<TD ALIGN="CENTER">',$type,'</TD>' ;

	print $tabfile  '<TD ALIGN="CENTER">' ;
	if(defined($predicted_by)) {
	    print $tabfile  $predicted_by ;
	} else {
	    print $tabfile  '-' ;
	}
	print $tabfile  '</TD>' ;

=pod

	if(defined($desc) and $desc and $desc !~ /^(\s+|\?)?$/ ) {
	    my $text = substr($desc,0,60) if length($desc) > 60 ; # clip if necessary
	    print $tabfile  '<TD>', $text ;
	} else {
	    print $tabfile  '<TD ALIGN="CENTER">?' ;
	}
	print $tabfile  '</TD>' ;

=cut

	print $tabfile  '<TD ALIGN="CENTER">' ;
	if(defined($OMIM) and $OMIM) {
	    print $tabfile  "<A HREF=\"http://www.ncbi.nlm.nih.gov/htbin-post/Omim/dispmim?$OMIM\">$OMIM</A><BR>" ;
	} else {
	    print $tabfile  '?' ;
	}
	print $tabfile  '</TD>' ;

	print $tabfile  "<TD ALIGN=\"CENTER\"><A NAME=\"$seqname\"><A HREF=\"$HUMSEEK=$seqname*\">$seqname</A></TD>" ;
	print $tabfile  '<TD ALIGN="CENTER">',$gf->start(),'</TD>' ;
	print $tabfile  '<TD ALIGN="CENTER">',$gf->end(),'</TD>' ;
	print $tabfile  '<TD ALIGN="CENTER">',$gf->strand(),'</TD>' ;
	print $tabfile  "</TR>\n" ;
    }
    print $tabfile  '</TABLE>' ;
}

###################################
# Draws HTML TABLE's of gene data
###################################
sub GeneTables {
    my $tabfile   = shift ;
    my $GFFt       = shift ; # genes
    my $GFFp       = shift ; # pseudogenes
    my $WithSeqIds = shift ; 
    
    if($GFFt && $GFFt->count()) {
	print $tabfile  "<A NAME=\"Transcript_Table\">\n" ;
	print $tabfile  h3("Table of Annotated and Verified Transcript Clusters ('Genes')"),"\n" ;
	SeqTabOut($tabfile,$GFFt,$WithSeqIds) ;
    } else {
	print $tabfile  p("No annotated, verified transcripts found in this region.") ;
    }
    if($GFFp && $GFFp->count()) {
	print $tabfile  "<A NAME=\"Pseudogene_Table\">\n" ;
	print $tabfile  h3("Table of Pseudogenes"),"\n" ;
	SeqTabOut($tabfile,$GFFp,$WithSeqIds) ;
    } else {
	print $tabfile  p("No pseudogenes found in this region.") ;
    }
}

sub label_gif{
    my($image,$sx,$sy,$xo,$yo,$col,$label,$labelcolour,$bold)=@_;
    my $x=$sx+$xo;
    my $y=$sy+$yo;
    my $size=10;
    my $offset=0;
    if($col){
	$image->filledRectangle($x,$y,$x+$size,$y+$size,$col);
	$image->rectangle($x,$y,$x+$size,$y+$size,$labelcolour);
	$offset=20;
    }
    if($bold){
	$image->string(gdMediumBoldFont,$x+$offset,$y,$label,$labelcolour);
    }else{
	$image->string(gdSmallFont,$x+$offset,$y,$label,$labelcolour);
    }
}
