
#
# Perl Module for EMBL::DE
#
# Cared for by Matthew Pocock <mrp@sanger.ac.uk>
#
# Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package EMBL::DE;

use vars qw($AUTOLOAD @ISA @EXPORT_OK %fields);

use Exporter;
use Carp;
use strict;

use EMBL::_LineGroup;
use EMBL::TextFunc;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();


BEGIN {
    @ISA = ( 'Exporter', 'EMBL::_LineGroup' );

    %fields = (
	       'description' => undef
    );
}

sub initialize {}

sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    my $self = new EMBL::_LineGroup;

    $self->rebless($class);
    return $self;
}

sub fromEMBLStream {
    my $class = shift;
    my $fh = shift;
    my $lineRef = shift;
    my $lineGroup = new EMBL::DE;
    my $line;
    my $text = undef;

    $line = $$lineRef;
    while(substr($line, 0, 2) eq 'DE') {
	chomp($line = substr $line, 5);                            # remove leading text
	defined($text) && ($text .= " ".$line) || ($text = $line); # add this line to previous text
	$line = <$fh>;
	if(not defined $line) { warn "DE: End of stream"; last; }
    }

    $lineGroup->description($text);

    $$lineRef = $line;
    return $lineGroup;
}

sub toEMBLStream {
    my $self = shift;
    my $fh = shift;
    my $line;

    foreach $line (EMBL::TextFunc->wrapText($self->description(), 75)) {
	print $fh "DE   $line\n";
    }
}

1;  # says use was ok
__END__

=head1 NAME

EMBL::DE

=head1 DESCRIPTION

B<EMBL::DE> represents the DE (description) lines of an embl entry, as specified in
http://www.ebi.ac.uk/ebi_docs/embl_db/usrman/de_line.html

B<EMBL::DE> inherits from B<EMBL::_LineGroup>

=head1 AUTHOR

B<Matthew Pocock> Email mrp@sanger.ac.uk

=head1 Member variables

=over

=item description

This contains a literal string representation of the description.
It contains all of the text of a single set of description lines.
On printing, this string is broken up into fragments that fit onto
embl width lines.

=back

=head1 Functions

=over

=item fromEMBLStream

This is a module function that accepts a \*filehandle and \$string as arguments. See
EMBL::_LineGroup for a description of what they are for. It is designed to be invoked
as 

    $lg = EMBL::DE->fromEMBLStream($fh, \$line);

within the parser. 

=item toEMBLStream

Writes the description to a stream in embl formatting.
It takes a \*filehandle as its only argument

    $deLine->toStream(\*STDOUT);
