
#
# Perl Module for EMBL::Image
#
# Cared for by Mathew Pocock <mrp@sanger.ac.uk>
#
# Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package EMBL::Image;

use vars qw($AUTOLOAD @ISA @EXPORT_OK %fields);

use Exporter;
use Carp;
use strict;

use EMBL::_LineGroup;

#
# @ISA has our inheritance.
#

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();


BEGIN {
    @ISA = ( 'Exporter', 'EMBL::_LineGroup' );

    %fields = (
	       #Insert field names here as field => undef,
	       text => undef
	       );
    EMBL::Image->check4Clashes();
}

sub initialize {}

sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    my $self = new EMBL::_LineGroup;

    $self->rebless($class);
    return $self;
}


sub fromEMBLStream {
    my $class = shift;
    my $fh = shift;
    my $lineRef = shift;
    my $line = new EMBL::Image;

    $line->text($$lineRef);

    $$lineRef = <$fh>;
    return $line;
}

sub toEMBLStream {
    my $self = shift;
    my $fh = shift;

    print $fh $self->text;
}


1;  # says use was ok
__END__

=head1 NAME

EMBL::Image

=head1 DESCRIPTION

B<EMBL::Image> is designed for when you compose your custom parser combinations!
It allows you to just stoor lines - zero queriability, but perfection on reading
in and printing out.  Usefull when you need to copy bits of an entry to the
output, but do not need to process them.

=head1 AUTHOR

B<Mathew Pocock> Email mrp@sanger.ac.uk

=head1 Member variables

=over

=item text

This contains the text that comprises the line.
It is stoored complete with tag and trailing new line.

=back

=head1 Functions

=item fromEMBLStream

Reads in a single line, and images it.  Invoked as

    $image = EMBL::Image->fromEMBLStream($fh, \$line);

=item toEMBLStream

Prints the image of the line back out to a stream.  It takes a \*filehandle as
its only argument.

    $image->toStream(\*STDOUT);


