
#
# Perl Module for EMBL::NI
#
# Cared for by Matthew Pocock <mrp@sanger.ac.uk>
#
# Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package EMBL::NI;

use vars qw($AUTOLOAD @ISA @EXPORT_OK %fields);

use Exporter;
use Carp;
use strict;

use EMBL::_LineGroup;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();


BEGIN {
    @ISA = ( 'Exporter', 'EMBL::_LineGroup' );

    %fields = (
	       #Insert field names here as field => undef,
	       'nucleotideID' => undef
    );

  EMBL::NI->check4Clashes();
}

sub initialize {};

sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    my $self = new EMBL::_LineGroup;

    $self->rebless($class);
    return $self;
}

sub fromEMBLStream {
    my $class = shift;
    my $fh = shift;
    my $lineRef = shift;
    my $lineGroup = new EMBL::NI;
    my $NI;

    $NI = substr $$lineRef, 5;
    chomp $NI;
    $lineGroup->nucleotideID($NI);

    $$lineRef = <$fh>;

    return $lineGroup;
}

sub toEMBLStream {
    my $self = shift;
    my $fh = shift;

    print $fh "NI   ", $self->nucleotideID, "\n";

    return $fh;
}

1;  # says use was ok
__END__

=head1 NAME

EMBL::NI

=head1 DESCRIPTION

B<EMBL::NI> represents the NI (nucleotide identifier) line of an embl
entry as specified in http://www.ebi.ac.uk/ebi_docs/embl_db/usrman/ni_line.html

EMBL::NI inherits from B<EMBL::_LineGroup>

=head1 AUTHOR

B<Matthew Pocock> Email mrp@sanger.ac.uk

=head1 Member variables

=over nucleotideID

The id number

=back

=head1 Functions

=over

=item fromEMBLStream

This is a module function that accepts a \*filehandle and \$string as arguments.
See EMBL::_LineGroup for a description of what they are for.  It is designed to be
invoked as

    $lg = EMBL::NI->fromEMBLStream($fh, \$line);

within the parser.

=item toEMBLStream

Writes the database reference to a stream in embl formatting.  It takes a \*filehandle as
its only argument.

    $niLine->toEMBLStream(\*STDOUT);
