
#
# Perl Module for EMBL::TextFunc
#
# Cared for by Matthew Pocock <mrp@sanger.ac.uk>
#
#Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package EMBL::TextFunc;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

sub listFromText {
    my $class = shift;
    my $text = shift;
    my $sep = shift;
    my $end = shift || $sep;

    chomp $text;          # remove \n from end of text
    $text =~ s/^$sep//;   # remove seperater at the beginning of the text
    $text =~ s/$end$//;   # remove seperator at the end of the text

    return split $sep, $text;
}

sub textFromList {
    my $class = shift;
    my $list = shift;
    my $sep = shift;
    my $end = shift;
    my $width = shift;
    my $text;

    $text = (join $sep, @{$list}) . $end;    # produce one funck off long string
    $width -= length $sep;                   # work out how many characters can be per line

    while($text =~ m/(.{1,$width}(($sep)|($)))/g) {
	push @_, $1;
    }
    
    for (@_) { # Remove trailing spaces from lines
	s/\s+$//;
    }
    
    return @_;
}

sub wrapText {
    my $class = shift;
    my $text = shift;
    my $width = shift;

    while($text =~ m/(.{1,$width})(\s+|$)/g) {
	push @_, $1;
    }
    
    for (@_) { # Remove trailing spaces from lines
	s/\s+$//;
    }
    
    return @_;
}

sub wrapOnCommas {
    my $class = shift;
    my $text = shift;
    my $width = (shift)-1;

    while($text =~ m/(.{1,$width}(,|$))/g) {
	push @_, $1;
    }

    return @_;
}

sub cleanLine {
    my $class = shift;
    my $text = shift;

    chomp($text);
    if(length($text) != 2) {
	$text = substr $text, 5;
    } else {
	$text = undef;
    }

    return $text;
}

1;  # says use was ok
__END__

=head1 NAME

EMBL::TextFunc

=head1 DESCRIPTION

Description for B<EMBL::TextFunc>

This module is designed to be a repository of functions that are
repeatedly used during parsing and formatting of embl lines.
If more than two line types need to do aproximately the same thing
then it is probably in here.

All functions expect to be called as package->function(param list)

=head1 AUTHOR

B<Matthew Pocock> Email mrp@sanger.ac.uk

=over

=item listFromText

Takes a piece of text, a seperator regex and a seperator that may appear at the end.
Returns an array of items that were seperated in the text by that seperator.  Takes care of
null items (looses them for you).

=item textFromList

Takes an array of items, a sepreator, a terminating string, and a line width.
Returns an array of strings, eachending in the seperator or the termiantor with
a width less than or equal to the width specified.

Seems to do the wrong thing for references - not sure why.  Don't use it for that.

=item wrapText;

Takes a string and a length.  Returns an array of strings which are shorter or equal
in length to length, spliting the string on white space.

=item wrapOnCommas

Takes a string and a length.  Returns an array of strings which are shorter or equal
in length to length, split on commas.
