
#
# Perl Module for EMBL::XX
#
# Cared for by Matthew Pocock <mrp@sanger.ac.uk>
#
# Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package EMBL::XX;

use vars qw($AUTOLOAD @ISA @EXPORT_OK %fields);

use Exporter;
use Carp;
use strict;

use EMBL::Discard;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

BEGIN {
    @ISA = ( 'Exporter', 'EMBL::Discard' );

    %fields = (
	       #Insert field names here as field => undef,
	       );

  EMBL::XX->check4Clashes;
}

sub initialize {
    my $self = shift;
}

sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    my $self = new EMBL::Discard;

    $self->rebless($class);

    return $self;
}

sub fromEMBLStream {
    my $self = EMBL::Discard::fromEMBLStream(@_);
    $self->rebless('EMBL::XX');
    return $self;
}

sub toEMBLStream {
    my $self = shift;
    my $fh = shift;

    print $fh "XX\n";

    return $fh;
}

1;  # says use was ok
__END__

=head1 NAME

EMBL::XX

=head1 DESCRIPTION

B<EMBL::XX> represent the XX lines that litter embl entries.
Inherit from B<EMBL::Discard>, and overrides the C<toEMBLStream> functionality.
Have a look at the C<fromEMBLStream> to see how I used the base class function.

=head1 AUTHOR

B<Matthew Pocock> Email mrp@sanger.ac.uk

=over

=item toEMBLStream

Prints an XX line to the stream.
Fits the API.
