
#
# Perl Module for FTUtils
#
# Cared for by Matthew Pocock <mrp@sanger.ac.uk>
#
# Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package FTUtils;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

sub DEandKW {
    my $class = shift;
    my $ft = shift;
    my (@kw, @de, @qual) = ();    
    my ($f, $com, $product, $i, $cds, $misc);
    my ($CpGs, $ESTs, $STSs);
    my ($first, $last, $conDes);

    $cds = $ft->filter(Filter::by_key('CDS'));

    # get all of the qualifiers into @qual
    # extract all the keywords, and the descriptions that go with them
    foreach $f ($ft->features()) {
	($product) = ($f->filter(Filter::by_name('product')))->qualifiers;
	if($product) {
	    my ($text, $gene, $kw);
	    ($text) = $product->value() =~ m"\((.*like\s*protein)";
	    if($f->filter(Filter::by_name('pseudo'))->qualifiers()) {
		$gene = "pseudo gene";
	    } else {
		$gene = "gene";
	    }
	    $kw = $text;
	    $kw =~ s/\s*protein\s*$//;
	    $kw =~ s/\s*like$//;
	    $kw =~ s/\s*putative\s*//;
	    $text =~ s/\s*protein$/ $gene/;
	    push @de, ", a ".$text;
	    push @kw, $kw;
	}
	push @qual, $f->qualifiers;
    }

    # make a single feature with all qualifiers from each feature
    $com = Feature->fromArray(@qual);

    #sort de & remove duplicates
    if(@de) {
	@de = sort @de;
	for($i = @de-1; $i; $i--) {
	    lc($de[$i]) eq lc($de[$i-1]) && (splice @de, $i, 1);
	    print "checking de for duplicates\n";
	}
    }

    # sort keywords nad throw out duplicates
    if(@kw) {
	@kw = sort @kw;
	for($i = @kw-1; $i; $i--) {
	    lc($kw[$i]) eq lc($kw[$i-1]) && (splice @kw, $i, 1);
	}
    }

    # work out if we have cpg islands
    $misc = $com->filter(Filter::by_nameValue('note', 'CpG island(?!.clone)'));
    $CpGs = ($misc->qualifiers());
    if($CpGs) {
	push @de, (($CpGs>1) ? ", CpG islands" : ", a CpG island"); 
	push @kw, "CpG island";
    }
    
    # work out if we have ESTs 
    $misc = $com->filter(Filter::by_nameValue('note', 'match:\s*EST'));
    $ESTs = ($misc->qualifiers());
    if($ESTs) {
	push @de, (($ESTs>1) ? ", ESTs" : ", an EST");
    }

    # work out if we ahve STSs
    $misc = $com->filter(Filter::by_nameValue('note', 'match:\s*STS'));
    $STSs = ($misc->qualifiers());
    if($STSs) {
	push @de, (($STSs>1) ? ", STSs" : ", an STS");
    }
    
    $first = shift @de;
    $last = pop @de;
    $first =~ s/, \s*//;
    $last =~ s/, / and /;
    $conDes = "Contains ".join ( "", $first, @de, $last);

    return ($conDes, @kw);
}

1;  # says use was ok
__END__

=head1 NAME

FTUtils

=head1 DESCRIPTION

Description for B<FTUtils>

=head1 AUTHOR

B<Matthew Pocock> Email mrp@sanger.ac.uk

=over

=item FTUtils::DEandKW

Takes a FeatureTable object.  Returns one string and an array.
The string is a description line.
The array is a list of keywords.

    ($de, @kw) = FTUtils->DEandKW($ft);



