#
# Perl Module for SDTable
#
# Cared for by Simon Dear <sd@sanger.ac.uk>
#
package SDTable;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
# use strict;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );



sub new {
    my($this) = shift;
    my $self = {
	"format" => "default",
	"delim" => "\t",
	"titles" => [],
	"titlescol" => [],
	"titleslabel" => [],
	"titleshref" => [],
	"title" => undef,
	"titlecol" => undef,
	"table" => [],
	"sum" => [],
	"count" => [],
	"named" => undef,
	"border" => 2,
	"tablecol" => [],
	"noBackgroundOnEmpty" => undef,
    };

    bless $self;
    $self->set(@_);

    $self;
}

sub set {
    my($self) = shift;
    local($key,$value);

    while($#_>=0) {
        ($key,$value) = (shift,shift);
        $self->{$key} = $value;
    }

    $self;
}

sub get {
    my($self,$key) = @_;

    $self->{$key};
}

sub rows {
    my($self) = @_;

    $#{$self->{"table"}}+1;
}


sub add {
    my($self) = shift;
    push(@{$self->{"table"}},[@_]);

    $self;
}

sub addcol {
    my($self) = shift;
    push(@{$self->{"tablecol"}},[@_]);

    $self;
}

sub format {
    my($self) = shift;
    my(%args) = @_;

    my($output);
    my(@width);
    my($i,$j);
    my($style);
    my($sep);
    my($len);
    my($cols); # number of columns
    my($left,$right);
    my(@sum) = ();
    my(@count) = ();
    my($htmltit);
    my($fhtmltit)=0;
    my($colstr)='';
    my($colstr2)='';

    $output = "";
    $style = (defined $self->{'format'}) ? $self->{'format'} : "gaudy";

    # Do summation
    if ($#{$self->{"sum"}}>-1) {
	for($i=0;$i<=$#{$self->{"table"}};$i++) {
	    foreach $j ( @{$self->{"sum"}} ) {
		$sum[$j] += $self->{"table"}->[$i]->[$j];
	    }
	}
	
    }

    # Do count
    if ($#{$self->{"count"}}>-1) {
	foreach $j ( @{$self->{"count"}} ) {
	    local(%unique_values) = ();
	    for($i=0;$i<=$#{$self->{"table"}};$i++) {
		$unique_values{$self->{"table"}->[$i]->[$j]}++;
	    }
	    $count[$j] = scalar(keys %unique_values);
	    
	}
	
    }

    # Determine max width of each column
    @width = ();
    if (defined $self->{'titles'}) {
	for($i=0;$i<=$#{$self->{"titles"}};$i++) {
	    if ($width[$i] < length($self->{"titles"}->[$i])) {
		$width[$i] = length($self->{"titles"}->[$i]);
	    }
	}
    }
    for($i=0;$i<=$#{$self->{"table"}};$i++) {
	for ($j=0;$j<=$#{$self->{"table"}->[$i]};$j++) {
	    if ($width[$j] < length($self->{"table"}->[$i]->[$j])) {
		$width[$j] = length($self->{"table"}->[$i]->[$j]);
	    }
	}
    }
    if ($#{$self->{"sum"}}>-1) {
	for($i=0;$i<=$#sum;$i++) {
	    if ($width[$i] < length($sum[$i])) {
		$width[$i] = length($sum[$i]);
	    }
	}
    }
    if ($#{$self->{"count"}}>-1) {
	for($i=0;$i<=$#count;$i++) {
	    if ($width[$i] < length($count[$i])) {
		$width[$i] = length($count[$i]);
	    }
	}
    }



    if ($style eq "htg") {
	$style = "gaudy";
	$sepstart = "#";
	$titlestart = "#";
    } elsif ($style eq "gaudy") {
	$sepstart = "+";
	$titlestart = "|";
    }

    if ($style eq "tabbed") {
	# work out how width to nearest tab stop
	for ($j=0;$j<=$#width;$j++) {
	    $width[$j] += 8; # one for a space, 7 to round up
	    $width[$j] -= $width[$j]%8; # truncate to nearest 8
	}
    }

    $cols = $#width + 1;

    $sep = "";
    if ($style eq "gaudy") {
	for ($j=0;$j<=$#width;$j++) {
	    $sep .= ($j ? "+" : $sepstart) . "-" . "-" x $width[$j] . "-";
	}
	$sep .= "+";
    }

    if($style eq "html"){
	$htmltit=qq{
<TABLE width=100% border=$self->{'border'} cellpadding=2>
};
    }

    # Print out table title
    if (defined $self->{"title"}) {
	if ($style eq "plain" || $style eq "tabbed") {
	    $output .= "$self->{'title'}\n\n";
	} elsif ($style eq "html") {
	    if(defined $self->{'titlecol'}){
		$colstr=qq{ bgcolor=$self->{'titlecol'}};
	    }
	    $fhtmltit=1;
	    $output .= $htmltit.qq{
<TR><TH colspan=$cols align=center$colstr>$self->{'title'}</TH></TR>
};
	} elsif ($style eq "gaudy") {
	    $len = length($sep) - 2;

	    if (length($self->{'title'}) > $len) {
		# Expand to fit title
		$width[$#width] += length($self->{'title'}) - $len + 2;
		$sep = "";
		for ($j=0;$j<=$#width;$j++) {
		    $sep .= ($j ? "+" : $sepstart) . "-" . "-" x $width[$j] . "-";
		}
		$sep .= "+";
		$len = length($sep) - 2;
	    }
	    $left = int(($len - length($self->{'title'})) / 2);
	    $right = $len - length($self->{'title'}) - $left;
	    $output .=  $sepstart . "-" x $len . "+\n";
	    $output .=  $titlestart . " "x$left . $self->{'title'} . " "x$right . "|\n";
	}
    }

    $output .=  $sep . "\n";

    # Print out column headings
    if (defined $self->{'titles'} && $#{$self->{'titles'}} >= 0) {
	if($style eq "html"){
	    unless($fhtmltit){
		$output .= $htmltit;
		$fhtmltit=1;
	    }
	    $colstr='';
	    $output .= "<TR align=center>";
	}
	for ($i=0;$i<=$#width;$i++) {

	    $output .=  ($i ? "| " : "$titlestart ") if $style eq "gaudy";

	    if ($style eq "delimited") {
		$output .= $self->{"titles"}->[$i];
		$output .= $self->{"delim"} if $i < $#width;
	    } elsif ($style eq "tabbed") {
		if ($self->{"titles"}->[$i] eq "") {
		    $output .= " ";
		} else {
		    $output .= $self->{"titles"}->[$i];
		}
		$output .= "\t" x (int(($width[$i] - length($self->{"titles"}->[$i])+7)/8)) if $i < $#width;
	    } elsif ($style eq "html") {
		if(defined $self->{'titlescol'}->[$i]){
		    $colstr=qq{ bgcolor=$self->{'titlecol'}->[$i]};
		}
		if(defined $self->{'noBackgroundOnEmpty'} && !$self->{'titles'}->[$i]){
		    $colstr2='';
		}else{
		    $colstr2=$colstr;
		}
		# add href and/or mouseover labels
		my $href=$self->{'titleshref'}->[$i];
		my $label=$self->{'titleslabel'}->[$i];
		my $name=$self->{'titles'}->[$i];
		my $string;
		if($href || $label){
		    $string="<a href=";
		    if($href){
			$string.="\"$href\"";
		    }else{
			$string.="\"#\"";
		    }
		    if($label){
			$string.=" onMouseOver=\"window.status=\'$label\'; return true\"";
			$string.=" onMouseOut=\"window.status=\'\'; return true\"";
			$string.=" name=\"$label\"";
		    }
		    $string.=">$name<\/a>";
		}else{
		    $string=$name;
		}
		$output .= "<TH$colstr2>$string</TH>";
	    } else {
		$output .= sprintf "%-$width[$i]s ",$self->{"titles"}->[$i];
	    }
	}
	$output .=  "|" if $style eq "gaudy";
	$output .=  "</TR>" if $style eq "html";
	$output .=  "\n";
	$output .=  $sep . "\n";
    }

    # Print out table
    for($i=0;$i<=$#{$self->{"table"}};$i++) {
# set named to label each line of table according to value in a column
# any html wrapping is stripped off first
	if($style eq "html"){
	    unless($fhtmltit){
		$output .= $htmltit;
		$fhtmltit=1;
	    }
	    $output .=  "<TR align=left>" if $style eq "html";
	}
	for ($j=0;$j<=$#width;$j++) {
	    $output .=  "| " if $style eq "gaudy";
	    if ($style eq "delimited") {
		$output .= $self->{"table"}->[$i]->[$j];
		$output .= $self->{"delim"} if $j < $#width;
	    } elsif ($style eq "tabbed") {
		if ($self->{"table"}->[$i]->[$j] eq "") {
		    $output .= " ";
		} else {
		    $output .= $self->{"table"}->[$i]->[$j];
		}
		$output .= "\t" x (int(($width[$j] - length($self->{"table"}->[$i]->[$j])+7)/8)) if $j < $#width;
	    } elsif ($style eq "html") {
# change colour if defined
		if(defined $self->{'tablecol'}->[$i]->[$j]){
		    $colstr=qq{ bgcolor=$self->{'tablecol'}->[$i]->[$j]};
		}
# don't use background if empty cell and turned off
		if(defined $self->{'noBackgroundOnEmpty'} && !$self->{'table'}->[$i]->[$j]){
		    $colstr2='';
		}else{
		    $colstr2=$colstr;
		}
		$output .= "<TD$colstr2>$self->{'table'}->[$i]->[$j]";
# add name tag
		if($j==0 && $style eq "html" && defined $self->{'named'}){
		    my $named=$self->{'table'}->[$i]->[$self->{'named'}];
		    $named=~s/\<[^\>]+\>//g;
		    $output.="<a name=\"$named\"><\/a>\n";
		}
		$output .= "</TD>";
	    } else {
		$output .= sprintf "%-$width[$j]s ",$self->{"table"}->[$i]->[$j];
	    }
	}
	$output .=  "|" if $style eq "gaudy";
	$output .=  "</TR>" if $style eq "html";
	$output .=  "\n";
    }
    $output .=  $sep . "\n";
    
    # Print out totals (sum)
    if ($#{$self->{"sum"}}>-1) {
	$output .=  "<TR align=right>" if $style eq "html";
	for ($i=0;$i<=$#width;$i++) {
	    $output .=  "| " if $style eq "gaudy";
	    if ($style eq "delimited") {
		$output .= $sum[$i];
		$output .= $self->{"delim"} if $i < $#width;
	    } elsif ($style eq "tabbed") {
		if ($sum[$i] eq "") {
		    $output .= " ";
		} else {
		    $output .= $sum[$i];
		}
		$output .= "\t" x (int(($width[$i] - length($sum[$i])+7)/8)) if $i < $#width;
	    } elsif ($style eq "html") {
		$output .= "<TH>$sum[$i]</TH>";
	    } else {
		$output .= sprintf "%-$width[$i]s ",$sum[$i];
	    }
	}
	$output .=  "|" if $style eq "gaudy";
	$output .=  "</TR>" if $style eq "html";
	$output .=  "\n";
	$output .=  $sep . "\n";
    }

    # Print out counts
    if ($#{$self->{"count"}}>-1) {
	$output .=  "<TR align=right>" if $style eq "html";
	for ($i=0;$i<=$#width;$i++) {
	    $output .=  "| " if $style eq "gaudy";
	    if ($style eq "delimited") {
		$output .= $count[$i];
		$output .= $self->{"delim"} if $i < $#width;
	    } elsif ($style eq "tabbed") {
		if ($count[$i] eq "") {
		    $output .= " ";
		} else {
		    $output .= $count[$i];
		}
		$output .= "\t" x (int(($width[$i] - length($count[$i])+7)/8)) if $i < $#width;
	    } elsif ($style eq "html") {
		$output .= "<TH>$count[$i]</TH>";
	    } else {
		$output .= sprintf "%-$width[$i]s ",$count[$i];
	    }
	}
	$output .=  "|" if $style eq "gaudy";
	$output .=  "</TR>" if $style eq "html";
	$output .=  "\n";
	$output .=  $sep . "\n";
    }

    $output .=  "</TABLE>\n" if $style eq "html";

    $output;
}




############################################################
# Create a new table, selecting rows and columns
############################################################
sub select {
    my($self) = shift;
    my(%args) = @_;

    $newtable = SDTable->new;
    my $i;

    # Set titles
    if (defined $args{'select'}) {
	my @titles = ();
	foreach $i (@{$args{'select'}}) {
	    push(@titles,$self->{'titles'}->[$i]);
	}
	$newtable->set("titles"=>[@titles]);
    } else {
	$newtable->set("titles"=>[@{$self->{'titles'}}]);
    }

    $select = 1;
    for($i=0;$i<=$#{$self->{"table"}};$i++) {

	if (defined $args{"func"}) {
	    $select = &{$args{"func"}}(@{$self->{"table"}->[$i]});
	}
    	if ( $select ) {
	    if (defined $args{'select'}) {
		my @row = ();
		foreach $j (@{$args{'select'}}) {
		    push(@row,$self->{'table'}->[$i]->[$j]);
		}
		$newtable->add(@row);
	    } else {
		$newtable->add(@{$self->{"table"}->[$i]});
	    }
	}
    }

    $newtable;

}


sub sort {
    my($self) = shift;
    my(%args) = @_;

    if (defined $args{'func'}) {
	local($func) = $args{'func'};
	$self->{'table'} = [ sort custom_sort @{$self->{'table'}} ];
    } else {
	$self->{'table'} = [ sort default_sort @{$self->{'table'}} ];
    }

    $self;
}


sub default_sort {
    join(" ",@{$a}) cmp join(" ",@{$b});
}
sub custom_sort {
    &$func();
}




1;  # says use was ok
__END__

=head1 NAME

SDTable.pm - Perl Object for formating tabular data.

=head1 DESCRIPTION

B<SDTable.pm> is a storage class for tabular data, with methods for
producing formatted layout in various styles. There is some very
simple tablutaion functionality.

=head2 AUTHOR

B<Simon Dear> Email sd@sanger.ac.uk

=head1 Class Methods

=over 5

=item C<new>

C<SDTable::new>

Creates a new SDTable. The table is initially empty, with no
titles, in a plain style. Instance variables can be initialised by
specifying them as arguments to this method.

=item C<set>

Sets the value of SDTable instance variables.

=item C<get>

Returns the value of SDTable instance variables.

=item C<add>

Adds a tuple (row) to the table.

=item C<addcol>

For html only, adds a tuple (row) of background colors for the table.
An empty cell results in no background.  A single value for a row
results in entire row taking that colour (note effect of
noBackgroundOnEmpty instance variable which allows empty cells to be
uncoloured, which is most effective when used with border=0).

=item C<format>

Generates an ascii string for the formatted table.

=item C<rows>

Returns the number of rows in the table.

=back

=head1 Instance variables

=over 5

=item C<format>

The format style used in layout. The value is one 

    plain       columns are separated by one or more spaces.
    tabbed      columns are separated by one or more tabs.
    delimited   places a single character delimiter between columns.
    gaudy       table entries are boxed with ascii characters (-,|,+).
    html        produces a HTML <TABLE>...</TABLE>.

=item C<delim>

The delimiter used in the 'delimited' format style.

=item C<title>

An optional title for the table.

=item C<titles>

A list of column titles.

=item C<titlecol>

For html only, an optional background colour for the title of the
table.

=item C<titlescol>

For html only, an optional background colour for each of the list of
column titles.

=item C<named>

For html only, if named is set to column number (numbered from 0) then
the html output includes a <a name="x"><\a> entry in each row, where x is
the value in that column selected.  This allows reference from another
page to particular lines, i.e. y.html#x.

=item C<noBackgroundOnEmpty>

For html only, if defined then when a cell is empty no background
color will be used.

=item C<border>

For html only, set border qualifier of html table (default=2)

=item C<table>

The anonymous array of arrays used to store the data in the table.

=item C<sum>

A list of columns (numbered from 0) whose sum will be placed at the
foot of the table.

=item C<count>

A list of columns (numbered from 0) whose count will be placed at the
foot of the table

=back


=head1 Example

Here is an example of SDTable use:


        #! /usr/local/bin/perl

        unshift(@INC,"$ENV{BADGER}/bin");

        require SDTable;

        # create a new table instance
        my $table = SDTable->new("format"=>"gaudy");

        # Set titles
        $table->set("title" => "My favourite Fruit", "titles" => ["Fruit","Colour","Number"]);

        # Sum the third column

        $table->set("sum" => [2]);

        # Add rows to table
        $table->add("Peach", "Orange","10");
        $table->add("Blueberry", "Blue","200");
        $table->add("Kiwifuit", "Brown","5");
        $table->add("Grapefruit", "Yellow","2");

        print $table->format;

This program generates the following output:

        +------------------------------+
        |      My favourite Fruit      |
        +------------+--------+--------+
        | Fruit      | Colour | Number |
        +------------+--------+--------+
        | Peach      | Orange | 10     |
        | Blueberry  | Blue   | 200    |
        | Kiwifuit   | Brown  | 5      |
        | Grapefruit | Yellow | 2      |
        +------------+--------+--------+
        |            |        | 217    |
        +------------+--------+--------+
