__END__
=head1 NAME

SPEM::Spem

=head1 DESCRIPTION

Overview of the SPEM:: modules, their intent and their uses.

=head1 AUTHOR

B<Matthew Pocock> Email mrp@sanger.ac.uk

=head1 Overview of SPEM

The Spem modules encapsulate the core functionality required to parse and represent EMBL and SWISSPROT formatted files. Three base classes are provided - L<SPEM::Parser>, L<SPEM::Set> and L<SPEM::Entry> which encapsulate some of the behavure and content of a stream parser, a set of entries and an individual entry respectively. A fourth file named after the database the parser parses (in the case of EMBL, this was named Embl.pm) can be included at a low level in the module hierachy.

What follows is a brief overview of how the files are treated in memory and during parsing, and how each of the base classes help you to write code to deal with this.

=head1 Parsing Moddle

The parser object is solely responsible for consuming a portion of a stream and creating SPEM::Entry and possibly SPEM::Set objects to represent that contense. The parser firstly creates SPEM::Set and SPEM::Entry derived objects to contain the stream, and then reads in a single line of text from it. The first two characters on the line are used to chose an apropreate action. You associate a module, object or function with each line type. The control of the stream is then handed over to the regitree which consumes as much of the stream as it can, before handing control back to the parser. The registree will create an object out of what has been consumed which will be added to the current SPEM::Entry derived object.

=head1 Entries and sets of entries

An entry is taken to be a collection of line groups.  A line group represents one logical block of the entry, such as the AC lines, a single reference (complete with author, title etc.), or the sequence lines. A set represents an unordered collection of entries. A set knows how to write it's self to a stream - it asks each of its entries to write themselves. An entry writes its self to stream by asking each of its lines to write themselves, and then terminating the record with a // line.

=head1 Data querying

Set derived objects have basic functionality for finding, adding and removing entries. Entries have basic fuctionality for finding, adding and removing lines. The lines may have functionality for getting and setting the variables they contain.

=head1 Guidelines for deriving your own SPEM parser

The best advice is to see what I have done in EMBL::* - This is a real working example. Also, read the documentation for each of the SPEM::* classes. You should name the packages as Database::packageName - so the SPEM::Parser derived object for Embl is EMBL::Parser. Lastly, so that the user can use a single line to use the system, you should write a module like Embl.pl that can be used to create a default parser, hide the intricacies of creating entries, and contain simple documentation. Other than that, do it any way that works.

If what you are doing doesn't seem to behave, then e-mail me. It could well be a bug rather than a misunderstanding.

Happy parsing.
