package Spangle::Analysis::Dir;

use Hum::Conf qw(
	       LINK_ANALYSIS_ROOT
	       ANALYSIS_ROOT
               HUMPUB_ROOT
	      );

use humpub qw( datedir unglob_user );

use strict;  
use vars   qw(@ISA);

# Object preamble - inherits from Bio::SeqFeature::Generic

use Bio::SeqFeature::Generic;

@ISA = qw(Bio::SeqFeature::Generic);

sub new {
    my( $pkg, @args ) = @_;

    my $self = bless {}, $pkg;
    $self->throw("Unexpected arguments [@args]") if @args;
    return $self;
}

sub make {
  my ($self,$name) = @_;

  my $chr = $self->chr || $self->throw("Chromosome isn't set for $name\n");

  # Clones named Em:SOMECLONE are stored in files named
  # SOMECLONE to avoid colons in filenames
  
  my ($fileName) = $name =~ /^(?:\w{2,3}:)?(.+)/
    or die "Weird clone name: '$name'";
  
  if ($fileName =~ /LINK_\S{2,3}:.*/) {
    $fileName =~ s/LINK_\S{2,3}:(.*)/LINK_$1/;
  }

  # New date directory for today
  my $datedir = datedir();

  # Make analysis directory name
  $self->root  ($ANALYSIS_ROOT);
  $self->chr   ($chr);
  $self->date  ($datedir);
  $self->filename($fileName);

  if ($self->isLink) {
    $self->root($LINK_ANALYSIS_ROOT);
  }

  my $analysisDir = $self->root . "/" . $self->chr . "/" . $self->filename . "/" . $self->date;

  $self->{_analysisDir} = "$analysisDir";
  $self->{_aceTag} = 'Analysis_directory ' . $self->{_analysisDir};
  
}


sub fetch {
  my ($self,$seq) = @_;

  return unless defined($seq);

  my $analysisDir = $seq->at('Analysis_details.Analysis_directory[1]');

  my $root;
  my $date;
  my $chr;
  my $filename;

  return unless $analysisDir;

  if ($analysisDir =~ /(.*?)\/(.*)\/(.*)\/(.*)/) {
    $root     = $1;
    $chr      = $2;
    $filename = $3;
    $date     = $4;
  } else {
    $self->throw("Analysis directory is in wrong format : $analysisDir\n");
  }

  $self->{_root}       = $root;
  $self->{_chr}        = $chr;
  $self->{_date}       = $date;
  $self->{_filename}   = $filename;
  $self->{_analysisDir} = $analysisDir;

  return $analysisDir;

}


sub update {
  my ($self,$name) = @_;
  
  if (defined($self->{_analysisDir})) {
    
    my $date   = datedir();
    my $newdir = $self->root. "/" . $self->chr . "/" . $self->filename . "/" . $date;
    
    $self->{_date} = $date;
    $self->{_analysisDir} = $newdir;
  } else {
    $self->make($name);
  }

}
  

sub construct {
  my ($self) = @_;

  my $newdir = $self->{_analysisDir};
  
  if (-e $newdir) {
    $self->warn("Directory $newdir already exists\n");
    return 1;
  } else {
    
    if (! -d $self->root) {
      my $res = mkdir($self->root,0755);
      $self->throw("Can't make root directory " . $self->root) unless $res;
    }
      
    if (! -d $self->root . "/" . $self->chr) {
      my $res = mkdir($self->root . "/" . $self->chr,0755);
      $self->throw("Can't make chromosome  directory " . $self->root . "/" . $self->chr) unless $res;
    }

    if (! -d $self->root . "/" . $self->chr . "/" . $self->filename) {
      my $res = mkdir($self->root . "/" . $self->chr . "/" . $self->filename,0755);
      $self->throw("Can't make chromosome  directory " . $self->root . "/" . $self->chr . "/" . $self->filename) unless $res;
    }

    if (! -d $self->analysisDir) {
      my $res = mkdir($self->analysisDir,0755);
      $self->throw("Can't make analysis directory " . $self->analysisDir) unless $res;
    }
  }

  if (! $self->analysisDir . "/rawdata") {
      my $res = mkdir ($self->analysisDir . "/rawdata",0755);
      $self->throw("Can't make rawdata directory " . $self->analysisDir . "/rawdata") unless $res;
  }
}

  
sub aceTag {
  my ($self) = @_;

  defined($self->{_analysisDir}) || $self->throw("No analysis directory found\n");
  return 'Analysis_directory ' .  $self->{_analysisDir};
}


sub isLink {
  my ($self,$arg) = @_;

  if (defined($arg)) {
    $self->{_isLink} = $arg;
  }

  return $self->{_isLink};

}


sub date {
  my ($self,$arg) = @_;

  if (defined($arg)) {
    $self->{_date} = $arg;
  }

  return $self->{_date};

}

sub root {
  my ($self,$arg) = @_;

  if (defined($arg)) {
    $self->{_root} = $arg;
  }

  return $self->{_root};

}

sub chr {
  my ($self,$arg) = @_;

  if (defined($arg)) {
    $self->{_chr} = $arg;
  }

  return $self->{_chr};

}

sub filename {
  my ($self,$arg)  = @_;

  if (defined($arg)) {
    $self->{_filename} = $arg;
  } 
  return $self->{_filename};
}

sub analysisDir {
  my ($self) = @_;

  return $self->{_analysisDir};

}
