package Spangle::Analysis::MUMmer;


use GD;

=pod

  =head1 NAME
  
  Spangle::Analysis::MUMmer - genomic sequence aligner

  =head1 SYNOPSIS
  
  $cl = new Spangle::Analysis::MUMmer
  
=head1 DESCRIPTION

A module to run, parse and display the output from
the program MUMmer

=head1 CONTACT

Describe contact details here

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut

use strict;  
use vars   qw(@ISA);

# Object preamble - inherits from Bio::SeqFeature::Generic

use Bio::Root::RootI;

@ISA = qw(Bio::Root::RootI);

sub new {
  my( $pkg, @args ) = @_;

  my $self = bless {}, $pkg;

  # Initialize the variables

  $self->{_start1}   = [];
  $self->{_start2}   = [];
  $self->{_length}   = [];
  $self->{_score}    = [];

  return $self; # success - we hope!
}

sub add_match {
  my ($self,$start1,$start2,$length,$score) = @_;

  if (!defined($start1) || !defined($start2) ||!defined($length)) {
    $self->throw("Must enter start1 start2 length for a match. Input was \"$start1,$start2,$length\"");
  }

  push(@{$self->{_start1}},$start1);
  push(@{$self->{_start2}},$start2);
  push(@{$self->{_length}},$length);
  push(@{$self->{_score}} ,$score);

}
sub print_matches {
  my ($self,$fh) = @_;

  my $count = 0;

  while ($count < $self->count) {
    print($fh $self->{_start1}[$count] . " " . $self->{_start2}[$count] . " " .   
$self->{_length}[$count] . " " . $self->{_score}[$count] . "\n"); 
    $count++;
  }
}
    


    
sub length1 {
  my ($self,$length1) = @_;

  if (defined($length1)) {
    $self->{_length1} = $length1;
  }
  return $self->{_length1};
}

sub length2 {
  my ($self,$length2) = @_;

  if (defined($length2)) {
    $self->{_length2} = $length2;
  }
  return $self->{_length2};
}
   
sub count {
  my ($self) = @_;

  my $count1 = $#{$self->{_start1}};
  my $count2 = $#{$self->{_start2}};
  my $count3 = $#{$self->{_length}};

  if ($count1 != $count2  || $count2 != $count3) {
    $self->throw("ERROR: Number of start1 start2 and lengths disagree");
  } else {
    return $count1;
  }
}

sub draw_box_and_scale {
  my ($self,$im,$width,$height,$col,$start1,$end1,$start2,$end2) = @_;


  my $xoffset   = $width * 0.2;
  my $yoffset   = $width * 0.2;

  my $newim = new GD::Image($width + 2 * $xoffset, $height + 2 * $yoffset);

  my $white  = $newim->colorAllocate(255,255,255);
  my $black  = $newim->colorAllocate(0,0,0);

  $newim->rectangle($xoffset,$yoffset,$width+$xoffset+1,$height+$yoffset+1,$black);

  my $tmp = int(log($end1-$start1)/log(10));

  print("Tmp is $tmp\n");

  my $tick = 10**($tmp-1);

  print("Tick is $tick\n");

  my $start = int($start1);  
  my $end =   int($end1);
  

  print("start is $start\n");
  print("end is $end\n");

  my $ticklen = 10;

  my $i = $start;
  my $tickcount = 0;
  while ($i <= $end) {


    $newim->line($xoffset + ($i-$start1)*$width/($end1-$start1),$yoffset-$ticklen,
		 $xoffset + ($i-$start1)*$width/($end1-$start1),$yoffset,$black);
    

    my $val = int($i/1000);

    if ($tickcount % 5 == 0) {
      $newim->stringUp(gdSmallFont,$xoffset + ($i-$start1)*$width/($end1-$start1) - 5,$yoffset - 4*$ticklen,$val,$black);

      $newim->line($xoffset + ($i-$start1)*$width/($end1-$start1),$yoffset-2* $ticklen,
		   $xoffset + ($i-$start1)*$width/($end1-$start1),$yoffset,$black);

      $tickcount = 0;
    }
    $tickcount++;
    $i += $tick;
  }

  $start = int($start2);
  $end =   int($end2);

  print("start is $start\n");
  print("end is $end\n");

  $i = $start;
  $tickcount = 0;

  while ($i <= $end) {
    my $val = int($i/1000);
    $newim->line($xoffset-$ticklen, $yoffset + ($i-$start2)*$height/($end2-$start2),
		 $xoffset    ,$yoffset + ($i-$start2)*$height/($end2-$start2),$black);
    
    

    if ($tickcount % 5 == 0) {
      $newim->string(gdSmallFont,$xoffset - 5*$ticklen, $yoffset + ($i-$start2)*$height/($end2-$start2) - 5,$val,$black);

      $newim->line($xoffset-2*$ticklen, $yoffset + ($i-$start2)*$height/($end2-$start2),
		   $xoffset    ,$yoffset + ($i-$start2)*$height/($end2-$start2),$black);

      $tickcount = 0;
    }
    $tickcount++;

    $i += $tick;
  }
  
  $newim->copy($im,$xoffset+1,$yoffset+1,0,0,$width,$height);
  return $newim;
}

sub dotplot  {
  my ($self,$size,$fh,$start1,$end1,$start2,$end2) = @_;

  $fh   = \*STDOUT unless $fh;
  $size = 500      unless $size;

  my $len1   = $self->{_length1} || $self->throw("ERROR: Length of sequence 1 not defined");
  my $len2   = $self->{_length2} || $self->throw("ERROR: Length of sequence 2 not defined");

  $start1 = 1     unless $start1;
  $start2 = 1     unless $start2;
  
  $end1   = $len1 unless $end1;
  $end2   = $len2 unless $end2;

  $len1 = ($end1 -  $start1 + 1 );
  $len2 = ($end2 -  $start2 + 1 );

  if ($len2 > $len1) {
      $size = $size* $len1/$len2;
  }

  my $height = $size;
  my $width  = $size * $len2/$len1;

  print("Width height = $width $height\n");

  my $im     = new GD::Image($width,$height);

  my $white  = $im->colorAllocate(255,255,255);
  my $black  = $im->colorAllocate(0,0,0);       
  my $red    = $im->colorAllocate(255,0,0);
  my $green  = $im->colorAllocate(0,255,0);
  my $blue   = $im->colorAllocate(0,0,255);



  my $count = 0;

  POINT: while ($count < $self->count) {
      my $s1coord = $self->{_start1}->[$count];
      my $s2coord = $self->{_start2}->[$count];
      
    $count++;

    next POINT if ($s1coord < $start1 || $s2coord < $start2 || $s1coord > $end1 || $s2coord > $end2);


    my $s1     = ($s1coord - $start1)* $height/$len1;
    my $s2     = ($s2coord - $start2)* $width /$len2;

    my $len   = $self->{_length}->[$count];
    my $score = $self->{_score}->[$count];

    if ($len > 0 ) {

      $im->line($s1,$s2,$s1+$len*$height/$len1,$s2+$len*$width/$len2,$black);

    } else {

    #  $im->line($width - $s1,$s2,$width - $s1+$len*$height/$len1,$s2-$len*$width/$len2,$black);

      $im->line($s1,$s2,$s1+$len*$height/$len1,$s2-$len*$width/$len2,$black);

    }
    $count++;
  }

  my $newim = $self->draw_box_and_scale($im,$width,$height,$black,$start1,$end1,$start2,$end2);

  print $fh $newim->gif;
}

sub print_binary {
  my ($self,$fh,$start1,$end1,$start2,$end2) = @_;
  
  $fh   = \*STDOUT unless $fh;

  my $len1   = $self->{_length1} || $self->throw("ERROR: Length of sequence 1 not defined");
  my $len2   = $self->{_length2} || $self->throw("ERROR: Length of sequence 2 not defined");
  
  $start1 = 1     unless $start1;
  $start2 = 1     unless $start2;
  
  $end1   = $len1 unless $end1;
  $end2   = $len2 unless $end2;
  
  $len1 = ($end1 -  $start1 + 1 );
  $len2 = ($end2 -  $start2 + 1 );
  
  my $count = 0;

 POINT: while ($count < $self->count) {
    my $s1coord = $self->{_start1}->[$count];
    my $s2coord = $self->{_start2}->[$count];
    my $len     = $self->{_length}->[$count]; 
    
    $count++;
    
    next POINT if ($s1coord < $start1 || $s2coord < $start2 || $s1coord > $end1 || $s2coord > $end2);
    
    if ($len > 0 ) {
      
      #	$im->line($s1,$s2,$s1+$len*$height/$len1,$s2+$len*$width/$len2,$black);
      
    } else {
      
      #	$im->line($s1,$s2,$s1+$len*$height/$len1,$s2-$len*$width/$len2,$black);
      
    }
    $count++;
  }

}
  
sub parse_match {
  my ($self,$fh) = @_;
  my $line;
  my $count = 0;

  while ($line = <$fh>) {
    chomp($line);
    my ($start,$end,$length,$score) = split(' ',$line);
#    print("$count $start $end $length\n");
    $self->add_match($start,$end,$length,$score);
    $count++;
  }
}

sub parse {
  my ($self,$fh,$dir) = @_;
  my $line;

  while(($line = <$fh>) && $line !~ /Start1\ +Start2\ +Length/) { }

  print("Starting parse\n");

  while (($line = <$fh>) && $line) {
    my @f = split(' ',$line);
    if ($#f == 2) {
      print("start1/2/len $f[0] $f[1] $f[2]\n");
      if ($dir == 1) {
	$self->add_match($f[0],$f[1],$f[2]);
      } else {
	$self->add_match($f[0],$f[1],-1 * $f[2]);
      }
    }
  }

  print("Done parsing MUMmer output\n");
}
sub parseMSP {
  my ($self,$fh) = @_;
  my $line;
  my $clones;

  $self->throw("ERROR: No length set for sequence 1") unless $self->length1;
  $self->throw("ERROR: No length set for sequence 2") unless $self->length2;

  print("Starting parse\n");

  while ($line = <$fh>) {
      my ($score,$pid,$qstart,$qend,$id1,$hstart,$hend,$id,$rest) = split(' ',$line);
      if ($id1 =~ /(.*)\/(.*)-(.*)/) {
	my $clstart1 = $2;
	my $clend1   = $3;
        $qstart     += $clstart1;
        $qend       += $clstart1;

      }
      if ($id=~ /(.*)\/(.*)-(.*)/) {
	my $clstart2 = $2;
	my $clend2   = $3;
        $hstart     += $clstart2;
        $hend       += $clstart2;
      }
      $clones->{$id}++;
      
      print("Adding $qstart $hstart " . (($qend - $qstart) + 1) . "\n");
      if ($qstart > $qend) {
	  $self->add_match($qend,$hstart,($qend - $qstart + 1),$score);
      } else {
	  $self->add_match($qstart,$hstart,($qend - $qstart + 1),$score);
      }
  }
      
  print("Done parsing MSP output\n");
}

1;
