#
# Object for storing sequence analysis details
#
# Cared for by Michele Clamp  <michele@sanger.ac.uk>
#
# Copyright Michele Clamp
#
# You may distribute this module under the same terms as perl itself
#
# POD documentation - main docs before the code

=pod 

=head1 NAME

Spangle::Analysis::Match - small object storing ungapped match data

=head1 SYNOPSIS

    my $obj    = new Spangle::Analysis::Match(-start1  => $start1,
					      -start2  => $start2
					      -length  => $length,
					      -score   => $score);

=head1 DESCRIPTION

Stores an ungapped sequence match.

=head1 CONTACT

Descri

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut

# Let the code begin...

package Spangle::Analysis::Match;

use vars qw(@ISA);
use strict;

use Bio::Root::RootI;

@ISA = qw(Bio::Root::RootI);


sub new {
  my($pkg,@args) = @_;
  
  my $self = bless {}, $pkg;

  my ($start1,$start2,$length,$score)  =
      $self->_rearrange([qw(START1
			    START2
			    LENGTH
			    SCORE
			    )],@args);

  defined($start1)  || $self->throw("Start not defined in match input");
  defined($start2) || $self->throw("End not defined in match input");
  defined($length) || $self->throw("Length not defined in match input");
  defined($score)  || $self->throw("Score not defined in match input");
  
  $self->start1($start1);
  $self->start2($start2);
  $self->length($length);
  $self->score($score);

  return $self;
}


=head2 start1

  Title   : start1
  Usage   : $self->start1($start1);
  Function: Get/set method for the start coordinate
            in the first sequence
  Returns : int
  Args    : int

=cut

sub start1 {
    my ($self,$arg) = @_;

    if (defined($arg)) {
	$self->{_start1} = $arg;
    }

    return $self->{_start1};
}

=head2 start2

  Title   : start2
  Usage   : $self->start2($start2);
  Function: Get/set method for the start coordinate
            in the second sequence
  Returns : int
  Args    : int

=cut

sub start2 {
    my ($self,$arg) = @_;

    if (defined($arg)) {
	$self->{_start2} = $arg;
    }

    return $self->{_start2};
}

=head2 length

  Title   : length
  Usage   : $self->length($length)
  Function: Get/set method for the length of the match
  Returns : int
  Args    : int

=cut

sub length {
    my ($self,$arg) = @_;

    if (defined($arg)) {
	$self->{_length} = $arg;
    }

    return $self->{_length};
}

=head2 score

  Title   : score
  Usage   : $self->score
  Function: Get/set method for the score of the match
  Returns : int
  Args    : int

=cut

sub score  {
    my ($self,$arg) = @_;

    if (defined($arg)) {
	$self->{_score } = $arg;
    }

    return $self->{_score };
}


1;
