#
# BioPerl module for Exon
#
# Cared for by Michele Clamp <michele@sanger.ac.uk>
#
# Copyright  Michele Clamp
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=pod

=head1 NAME

Spangle::Exon - Confirmed Exon with splice site info and supporting features

=head1 SYNOPSIS

    $ex = new Spangle::Exon;

    $ex->start(10);
    $ex->end(100);

Examples of creating an exon

    $ex = new Spangle::Exon(1208,1506,1) # start = 1208, end = 1506, forward strand


=head1 DESCRIPTION
Exon object.

=head1 CONTACT

Describe contact details here

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Spangle::Exon;
use vars qw(@ISA);
use strict;

# Object preamble - inherits from Bio::EnsEMBL::Exon

use Bio::EnsEMBL::Exon;

@ISA = qw(Bio::EnsEMBL::Exon);

sub new {
    my( $pkg, @args ) = @_;

    my $self = bless {}, $pkg;
    $self->throw("Unexpected arguments [@args]") if @args;
    return $self;
}

sub add_supporting_feature {
  my ($self,$f) = @_;

  if (!defined($self->{_supporting_feature})) {
    $self->{_supporting_feature} = [];
  }
  push(@{$self->{_supporting_feature}},$f);
}

sub each_supporting_feature {
  my $self = shift;

  if (!defined($self->{_supporting_feature})) {
    $self->{_supporting_feature} = [];
  }
  return @{$self->{_supporting_feature}};
}

sub donor {
  my ($self,$arg) = @_;

  if (defined($arg)) {
    $self->{_donor} = $arg;
  }
  return $self->{_donor};
}


sub acceptor {
  my ($self,$arg) = @_;

  if (defined($arg)) {
    $self->{_acceptor} = $arg;
  }
  return $self->{_acceptor};
}

sub seq {
  my ($self,$arg) = @_;

  if (defined($arg)) {
    $self->{_seq} = $arg;
  }
  return $self->{_seq};
}
