package Spangle::GFF;

use GFF::GeneFeatureSet;
use GFF::GeneFeature;
use GFF::GifGFF;

use strict;

sub clone2gff {
  my ($clone,$url) = @_;

  my $filestub = $url;
     $filestub =~ s/\.html$//;
     $filestub =~ s/.*\/(.*)/$1/;

  print ("FILESTUB = $filestub\n");
  print ("URL      = $url\n");

  my $gff = GFF::GeneFeatureSet->new;

  my $seq = $clone->dna_seq->seq;
  my $cf  = make_gff_contig($clone,length($seq));

  $gff->addGeneFeature($cf);
  
  foreach my $g ($clone->eachSubsequence,$clone->each_new_span) {
    $g->sort;
    
    my $count = 0;
    my $prev;
    my $type;
    
    if ($g->primary_tag eq "new") {
      $type = 'new';
    } elsif ($g->primary_tag eq "otherspan") {
      $type = 'otherspan';
    } elsif ($g->primary_tag eq "merged") {
      $type = 'merged';
    } elsif ($g->primary_tag eq "extended") {
      $type = 'extended';
    } elsif ($g->source_tag =~ /SPAN/) {
      $type = 'span';
    } elsif ($g->primary_tag =~ /redicted/) {
      $type = 'predicted';
    } else {
      $type = 'gene';
    }
    
    foreach my $ex ($g->each_Exon) {
      
	my $gf = make_gff_exon($ex,$filestub,$g->source_tag,$type);
      
      $gff->addGeneFeature($gf);
      
      if ($count > 0) {
	my $in = make_gff_intron($prev,$ex,$filestub,$g->source_tag,$type);
	
	$gff->addGeneFeature($in);
      }
      $prev = $ex;
      $count++;
      
    }
  }
  return $gff;
}


sub make_gif {
  my ($clone,$gff,$fh,$giffile)  = @_;

  my $id = $clone->id;
  my $file2 =  $giffile;
  print("Writing gif to $giffile\n");
  open   (GIF,">$giffile");
  binmode(GIF);
  chmod 0666,$giffile;


  
  my $group_by_name=sub{
    my $self=shift;
    return $self->seqname;
  };
  
  my $group_features=sub{
    my $self=shift;
    return $self->feature;
  };
  
  my @layout=(
  
              # order  height   label        color1       color2      fbox  layout_filter
 
	       [  -10,      6, 'merged',    'darkgreen', 'F',         1,    $group_by_name],
	       [  -11,      6, 'otherspan', 'green',     'F',         1,    $group_by_name],
	       [   -9,      6, 'extended',  'purple',    'F',         1,    $group_by_name],
	       [   -8,      6, 'new',       'cyan',      'F',         1,    $group_by_name],
	       [   -6,      6, 'span',      'yellow',    'F',         1,    $group_by_name],
	       [   -4,      6, 'predicted', 'blue',      'F',         1,    $group_by_name],
	       [   -2,      6, 'gene',      'red',       'F',         1,    $group_by_name],
	       [   -1,     15,                                                            ],
	       [    0,     10, 'contig',    'green',     'darkgreen', 1,    $group_by_name],
	       [    1,     15,                                                            ],
	       [    2,      6, 'gene',      'red',       'R',         1,    $group_by_name],
	       [    4,      6, 'predicted', 'blue',      'R',         1,    $group_by_name],
	       [    6,      6, 'span',      'yellow',    'R',         1,    $group_by_name],
	       [    8,      6, 'new',       'cyan',      'R',         1,    $group_by_name],
	       [    9,      6, 'extended',  'purple',    'R',         1,    $group_by_name],
	       [   10,      6, 'otherspan', 'green',     'R',         1,    $group_by_name],
	       [   11,      6, 'merged',    'darkgreen', 'R',         1,    $group_by_name],
		);
  
  print($fh "<html><head><title></title></head>\n");
  print($fh "<body bgcolor=\"#FFFFFF\"><map name=\"gifmap\">\n");

  my $giffh = \*GIF;

  print("GIFFY " . $giffh . "\n");

  my ($x,$y) = GFF::GifGFF::gff2gif( gff         => $gff,
				     io          => $giffh,
				     mio         => $fh,
				     layout      => \@layout,
				     filter      => $group_features,
				     intronstyle => 'angle',
				     width       => 700,
				     scale       => 50,
         #                            yseparator  => 2,
				   );
  print STDERR "Done gif\n"  ;
  close(GIF);



  print($fh " </map>\n");
  print($fh " <h2>Image map of clone $id</h2>\n");
  print($fh "<img src=\"$file2\" usemap=\"#gifmap\" width=$x height=$y border=0>\n");
  print($fh "<table border=0><tr><td>red/dark red</td><td> Annotated genes</td></tr>\n");
  print($fh "<tr><td>blue/dark blue</td><td>Predicted genes</td</tr>\n");
  print($fh "<tr><td>yellow</td><td>Homology spans overlapping annotated genes</td></tr>\n");
  print($fh "<tr><td>cyan</td><td>Homology spans not overlapping annotated genes (potential new genes)</td></tr>\n");
  print($fh "<tr><td>green</td><td>Other homology spans not overlapping annotated genes</td></tr>\n");
  print($fh "<tr><td>purple</td><td>Possible extended genes</td></tr>\n");
  print($fh "<tr><td>dark green</td><td>Possible merged genes</td></tr></table>\n");
  print($fh "</body></html>\n");

}

sub make_gff_exon {
  my ($exon,$id,$name,$type) = @_;

  my $gf = new GFF::GeneFeature;
  
  my $start = $exon->start;
  my $end   = $exon->end;

  if ($start > $end) {
      my $tmp = $start;
      $start = $end;
      $end   = $tmp;
  }

  $gf->feature($type);
  $gf->start  ($start);
  $gf->end    ($end);
  $gf->seqname($name);
  
  if ($exon->strand == -1) {
    $gf->strand('-');
  } else {
    $gf->strand('+');
  }

  $gf->addMember("Exon $name:" . $gf->start . "-" . $gf->end,'label');
  $gf->addMember("$id.$name.html",'href');
  return $gf;
}

sub make_gff_intron {
  my ($exon1, $exon2, $id,$name,$type) = @_;

  my $gf = new GFF::GeneFeature;
  
  $gf->feature($type);
  $gf->seqname($name);
  
  if ($exon1->strand == -1) {
    $gf->start  ($exon2->end);
    $gf->end    ($exon1->start);
    $gf->strand('-');
  } else {
    $gf->strand('+');
    $gf->start  ($exon1->end);
    $gf->end    ($exon2->start);
  }
  $gf->addMember(1,'intron');
  $gf->addMember("Exon $name:" . $gf->start . "-" . $gf->end,'label');
  $gf->addMember("$id.$name.html",'href');
  return $gf;
  
}

sub make_gff_contig {
  my ($clone,$length) = @_;

  my $gf = new GFF::GeneFeature;

  $gf->start  (1);
  $gf->end    ($length);
  $gf->seqname($clone->id);
  $gf->feature('contig');
  $gf->strand ('+');
  
  return $gf;
}


1;
