package Spangle::Trembl_DB;

use strict;
use Spangle::DB_FileI;
use Hum::Tracking qw( ref_from_query );

use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

$VERSION = 1.00;

@ISA = qw(Spangle::DB_FileI);

my $file = "/nfs/croc/michele/pogdir/dbmfiles/trembl.dbm";

sub new {

  my($class,$myfile) = @_;
  my($self);

  $self = {};

  bless $self,$class;

  $myfile = $file unless $myfile;

  if (! -e $myfile) {
    print("WARNING: sequence file doesn't exist\n");
  }

  $self->dbfile($myfile);

  if ( -e $myfile) {
      $self->read_DB();
  } else {
      $self->generate_DB();
  }

  return $self;
}

sub get_accession {
    my ($self,$cloneid) = @_;

    my $string        = $self->_tied_hash->{$cloneid};
    my ($acc,@trembl) = split(' ',$string);
    return $acc;

}

sub set_accession {
  my ($self,$cloneid,$accession) = @_;

  my $hash = $self->_tied_hash->{$cloneid} = $accession;

}

	
sub each_tremblid {
  my ($self,$cloneid) = @_;
  
  my $string        = $self->_tied_hash->{$cloneid};
  print("String is $string:\n");
  my ($acc,@trembl) = split(' ',$string);

  return @trembl;
}

sub add_tremblid {
  my ($self,$cloneid,$tremblid) = @_;

  my $acc     = $self->get_accession($cloneid);
  my @trembls = $self->each_tremblid($cloneid);

  push(@trembls,$tremblid);
  print("ACC $acc:@trembls\n");
  my $string = $acc . " " . (join(" ",@trembls));

  $self->_tied_hash->{$cloneid} = $string;

}

sub generate_DB {
  my ($self) = @_;

  print("Generating hash\n");
  $self->SUPER::generate_DB;
  print("done\n");
  my %clones;

  my $species = ucfirst lc "human";
    
  my $query = qq(
		 select c.clonename, f.suffix, f.accession
		 from   project             p, 
                        finished_submission f,
		        clone_project       cp, 
		        clone               c
		 where
            c.clonename    = cp.clonename and
            cp.projectname = p.projectname and
            p.projectname  = f.projectname and
            c.speciesname  = '$species' and

            p.projectname not in(
                select projectname
                from project_status
                where status = 24 )
		);
    
  my $ans = ref_from_query( $query );
    
  foreach my $row (@$ans) {
    my( $clone, $suffix, $accession ) = @$row;
    my $seq_name = $suffix ? $clone.$suffix : $clone;
    print("Seq is $seq_name\n");
    $clones{$seq_name} = $accession;
  }

  foreach my $clone (keys %clones) {
    open (SEQ,"efetch EM:" . $clones{$clone} . " |");

    my $found = 0;
    my $accession;
    my $refline;

    my @trembls;
    my @others;
    
    while (defined((my $line = <SEQ>)) && ($found == 0)) {
      
      if ($line =~ /^SQ /) {
	$found = 1;
	close(SEQ);
      } elsif ($line =~ /^AC +(\S+)/) {
	$accession = $1;
	$accession =~ s/;//g;
      } elsif ($line =~ /^RL +(.*)\n$/) {
	$refline .= $1;
      } elsif ($line =~ /^DR +SPTREMBL\; +(\S+)\; +/) {
	my $tmp = $1;
	$tmp =~ s/\;//g;
	push(@trembls,split(' ',$tmp));
      } elsif ($line =~ /^DR +.*\n$/) {
	push(@others,$1);
      }
    }
    
    if ($found == 1) {
      print(ref($self) . " " . $clone . " " . $accession . "\n");
      $self->set_accession($clone,$accession);

      my $newacc = $self->get_accession($clone);
      print("Entry is $clone : $accession @trembls\n");
      
      foreach my $tr (@trembls) {
	$self->add_tremblid($clone,$tr);
      }
    }
  
  }

    
}

=head2 _tied_hash

 Title   : _tied_hash
 Usage   : $obj->_tied_hash($newval)
 Function: 
 Example : 
 Returns : value of _tied_hash
 Args    : newvalue (optional)


=cut


=head2 dbfile

 Title   : dbfile
 Usage   : $obj->dbfile($newval)
 Function: 
 Example : 
 Returns : value of dbfile
 Args    : newvalue (optional)


=cut

	


1;
