
### Hum::Fox

package Hum::Fox;

use strict;
use Carp;
use DBI;
use Exporter;
use vars qw{ @ISA @EXPORT_OK };
use Hum::Submission qw { get_user };
use Hum::FoxConf qw { localisation };

@ISA = qw{ Exporter };
@EXPORT_OK = qw{
    current_uname
    fox_prepare_statement
    fox_disconnect
    };

{
    my( $uname );

    sub current_uname {
        unless ($uname) {
            $uname = get_user();
        }
        return $uname;
    }
}


# fox_db returns the database handle
{
    my( $db );

    sub fox_db () {
        unless ($db) {
            my $user = current_uname();
	    my ($host, $port)=Hum::FoxConf::localisation();
            # Make the database connection
            $db = DBI->connect("DBI:mysql:host=$host;port=$port;database=fox",
                               $user, undef, {RaiseError => 1})
                or die "Can't connect to fox database as '$user' ",
                DBI::errstr();
        }
        return $db;
    }
    sub fox_prepare_statement {
        my( $text ) = @_;
        
        my $sth = fox_db()->prepare($text);
        return $sth;
    }
    
    sub fox_disconnect {
        $db->disconnect if $db;
        $db = undef;
    }
    
    END {
        fox_disconnect();
    }
}


1;

__END__

=head1 NAME - Hum::Fox

=head1 AUTHOR

James Gilbert B<email> jgrg@sanger.ac.uk

