
### Bio::EnsEMBL::Ace::Filter

package Bio::EnsEMBL::Ace::Filter;

use strict;
use Carp;
use Bio::SeqIO;

sub new {
    my ($class) = @_;

    my $self = bless {}, $class;

    return $self;
}

sub ace_data {
    my( $self ) = @_;
    
    my $pkg = ref($self);
    confess("Filter module '$pkg' does not implement the 'ace_data' method");
}

sub analysis_object {
    my ( $self, $ana_obj ) = @_;

    if ($ana_obj) {
        my $expected = 'Bio::EnsEMBL::Analysis';
        confess("Not a '$expected' : '$ana_obj'")
            unless $ana_obj->isa($expected);
        $self->{'_analysis_object'} = $ana_obj;
    }
    return $self->{'_analysis_object'};
}

sub contig_internal_id {
    my( $self, $contig ) = @_;
    
    my @mapc = $contig->_vmap->each_MapContig;
    confess("Got ", scalar(@mapc), " MapContigs (only expecting 1)") unless @mapc == 1;
    my $raw_contig_id = $mapc[0]->contig->internal_id;
    return $raw_contig_id;
}


1;

__END__

=head1 NAME - Bio::EnsEMBL::Ace::Filter

=head1 AUTHOR

Stephen Keenan B<email> keenan@sanger.ac.uk

