
### .::Bio::EnsEMBL::Ace::Filter::SimpleFeature

package Bio::EnsEMBL::Ace::Filter::CpG;

use strict;
use Carp;
use strict;
use Bio::SeqIO;
use Bio::EnsEMBL::Ace::Filter;
use vars '@ISA';
@ISA = ('Bio::EnsEMBL::Ace::Filter');

sub new {
    my ($class) = @_;

    my $feat = bless {}, $class;
    return $feat;
}

sub ace_data {
    my ( $self, $contig ) = @_;
    
    my $seqname = "\nSequence : ".$contig->id."\n"; 
   
    my $ana_obj = $self->analysis_object
        or confess "analysis_object not set";
    my $ana_id = $ana_obj->dbID;
    my $raw_contig_id = $self->contig_internal_id($contig);
    my $db_obj = $contig->dbobj;
    my $sth = $db_obj->prepare(qq{
        SELECT seq_start
          , seq_end
          , score
          , evalue
          , perc_id
        FROM feature
        WHERE contig = $raw_contig_id
          AND analysis = $ana_id
        ORDER BY hid
        });
    $sth->execute;
    my ( $start,  $end,  $score,  $e_value,  $percent_id );
    $sth->bind_columns(
        \$start, \$end, \$score, \$e_value, \$percent_id );

    my $ace_data = '';
    while ($sth->fetch) {
        $ace_data .= sprintf(
            qq{Feature "Predicted_CpG_island" %d %d %f "Predicted_CpG Island: %.0f%% GC, o/e=%.2f, #CpGs=%.0f"\n},
            $start, $end, $e_value, $percent_id, $e_value, $score
            );
    }
    
    if ($ace_data){
        return $seqname . $ace_data;
    } else {
        #warn "\nNo CpG data for $seqname\n";
        return '';
    }
}

1;

__END__

=head1 NAME - Bio::EnsEMBL::Ace::Filter::SimpleFeature

=head1 AUTHOR

Stephen Keenan<th> B<email> keenan@sanger.ac.uk

