
### Bio::EnsEMBL::Ace::Filter::DNA

package Bio::EnsEMBL::Ace::Filter::DNA;

use strict;
use Carp;
use Bio::SeqIO;
use Bio::EnsEMBL::Ace::Filter;
use vars '@ISA';
@ISA = ('Bio::EnsEMBL::Ace::Filter');

sub new {
    my ($class) = @_;

    my $self = bless {}, $class;
    return $self;
}

sub ace_data {
    my ( $self, $contig ) = @_;

    #warn "Making DNA\n";
    
    my $id = $contig->id;

    my $raw_contig_id = $self->contig_internal_id($contig);
    my $db_obj = $contig->dbobj;
    my $sth = $db_obj->prepare(qq{
        SELECT d.sequence
        FROM contig g
          , dna d
        WHERE g.dna = d.id
          AND g.internal_id = $raw_contig_id
        });
    my( $dna_str );
    $sth->execute;
    $sth->bind_columns(\$dna_str);
    $sth->fetch or confess "Can't get dna for '$id' (contig.internal_id = $raw_contig_id)";;

    my $ace = qq{\nSequence : "$id"\n}
        . qq{Genomic_canonical\n}
        . qq{\nDNA "$id"\n};
    while ($dna_str =~ /(.{1,60})/g) {
        $ace .= "$1\n";
    }
    return $ace . $self->non_golden_features($contig);
}

sub non_golden_features {
    my( $self, $contig ) = @_;
    
    my $id = $contig->id;
    my @mapc = $contig->_vmap->each_MapContig;
    if (@mapc != 1) {
        confess "Got ", scalar(@mapc), " MapContigs";
    }
    
    my $mc = $mapc[0];
    my $raw_start  = $mc->rawcontig_start;
    my $raw_end    = $mc->rawcontig_end;
    my $raw_length = $mc->contig->length;
    
    #warn "$raw_start (1) to $raw_end ($raw_length)";
    my $feats = '';
    if ($raw_start != 1) {
        $raw_start--;
        $feats .= qq{Feature "NonGolden" 1 $raw_start 100 "Region of $id not on golden path"\n};
    }
    if ($raw_end != $raw_length) {
        $raw_end++;
        $feats .= qq{Feature "NonGolden" $raw_end $raw_length 100 "Region of $id not on golden path"\n};
    }
    
    if ($feats) {
        return qq{\nSequence : "$id"\n} . $feats;
    } else {
        return "";
    }
}

1;

__END__

=head1 NAME - Bio::EnsEMBL::Ace::Filter::DNA

=head1 AUTHOR

Stephen Keenan B<email> keenan@sanger.ac.uk

