
### Bio::EnsEMBL::Ace::Filter::Repeatmasker

package Bio::EnsEMBL::Ace::Filter::Repeatmasker;

use strict;
use Carp;
use strict;
use Bio::SeqIO;

use Bio::EnsEMBL::Ace::Filter;
use vars '@ISA';
@ISA = ('Bio::EnsEMBL::Ace::Filter');

sub new {
    my ($class) = @_;

    my $self = bless {}, $class;
    return $self;
}

sub ace_data {
    my ( $self, $contig ) = @_;

    my $contig_id = $contig->id;
    my $ana_obj = $self->analysis_object
        or confess "analysis_object not set";
    my $repeat_class = $self->get_repeat_class_hash;

    my $ana_id = $ana_obj->dbID;
    my $raw_contig_id = $self->contig_internal_id($contig);
    my $db_obj = $contig->dbobj;
    my $sth = $db_obj->prepare(qq{
        SELECT seq_start
          , seq_end
          , score
          , strand
          , hstart
          , hend
          , hid
        FROM repeat_feature
        WHERE contig = $raw_contig_id
          AND analysis = $ana_id
        ORDER BY hid
        });
    $sth->execute;
    my ( $start,  $end,  $score,  $strand,  $hstart,  $hend,  $hid );
    $sth->bind_columns(
        \$start, \$end, \$score, \$strand, \$hstart, \$hend, \$hid );
    
    my $repeat_ace = qq{\nSequence "$contig_id"\n};
    my $count = 0;
    while ($sth->fetch) {
        $count++;
        my $feature_name = 'RepeatMasker';
        if (my $class = $repeat_class->{$hid}) {
            if ($class eq 'LINE') {
                $feature_name = 'RepeatMasker_LINE';
            }
            elsif ($class eq 'SINE') {
                $feature_name = 'RepeatMasker_SINE';
            }
        }
        if ($strand == 1) {
            $repeat_ace .=
                qq{Motif_homol "$hid" "$feature_name" $score $start $end $hstart $hend\n};
        }
        else {
            $repeat_ace .=
                qq{Motif_homol "$hid" "$feature_name" $score $end $start $hstart $hend\n};
        }
    }
    
    if ($count) {
        return $repeat_ace;
    } else {
        return '';
    }
}

sub repeatmasker_home {
    my( $self, $home ) = @_;
    
    if ($home) {
        confess "Not a directory: '$home'"
            unless -d $home;
        $self->{'_repeatmasker_home'} = $home;
    }
    return $self->{'_repeatmasker_home'};
}

{
    my( $repeats );

    sub get_repeat_class_hash {
        my( $self ) = @_;

        unless ($repeats) {
            $repeats = {};

	    # potential alternative source, if LACE_LOCAL set
	    if($ENV{"LACE_LOCAL"}){
		my $rep_header_file=$ENV{"LACE_LOCAL"}."/repclass.lis";
		if(-e $rep_header_file){
		    local(*REPCLASS);
		    open(REPCLASS, $rep_header_file) or confess "Cannot open $rep_header_file: $!";
		    while (<REPCLASS>) {
			my($rep,$class)=split(/\t/);
			$repeats->{$rep} = $class;
		    }
		    close(REPCLASS);
		    print "INFO: used FILE $rep_header_file (".scalar(keys %$repeats).")\n";
		    return $repeats;
		}
	    }

            my $rep_home = $self->repeatmasker_home
                or confess "repeatmasker_home not set";
            my $rep_lib_dir = "$rep_home/Libraries";
            local(*REPLIB, *REPFILE);
            opendir REPLIB, $rep_lib_dir or confess "Can't opendir '$rep_lib_dir' : $!";
            

            foreach my $rep_file (map "$rep_lib_dir/$_", sort grep /\.lib$/, readdir REPLIB) {

                open REPFILE, $rep_file
                    or confess "Can't open '$rep_file' : $!";
                while (<REPFILE>) {
                    if (/^>(.*)#(\w+)/) {
                        my( $exist_class );
                        if ($exist_class = $repeats->{$1} and $exist_class ne $2) {
                             #warn qq{Repeat '$1' already exists with class '$exist_class'\n  but has class '$2' in '$rep_file'\n} ;
                        } else {
                            $repeats->{$1} = $2;
                        }
                    }
                }
                close REPFFILE;
            }
            closedir REPLIB;
	    print "INFO: used DIR $rep_lib_dir (".scalar(keys %$repeats).")\n";
        }
        return $repeats;
    }
}

1;

__END__

=head1 NAME - Bio::EnsEMBL::Ace::Filter::Repeatmasker

=head1 AUTHOR

Stephen Keenan B<email> keenan@sanger.ac.uk

