
### Bio::EnsEMBL::Ace::Filter::TRF

package Bio::EnsEMBL::Ace::Filter::TRF;

use strict;
use Carp;
use Bio::EnsEMBL::Ace::Filter;
use vars '@ISA';
@ISA = ('Bio::EnsEMBL::Ace::Filter');


sub ace_data {
    my ( $self, $contig ) = @_;

    my $contig_id = $contig->id;
    my $ana_obj = $self->analysis_object
        or confess "analysis_object not set";

    my $ana_id = $ana_obj->dbID;
    my $raw_contig_id = $self->contig_internal_id($contig);
    my $db_obj = $contig->dbobj;
    my $sth = $db_obj->prepare(qq{
        SELECT seq_start
          , seq_end
          , score
          , hid
        FROM repeat_feature
        WHERE contig = $raw_contig_id
          AND analysis = $ana_id
        ORDER BY hid
        });
    $sth->execute;
    my ( $start,  $end,  $score,  $hid );
    $sth->bind_columns(
        \$start, \$end, \$score, \$hid );
    
    my $repeat_ace = qq{\nSequence "$contig_id"\n};
    my $count = 0;
    my $feature_name = $ana_obj->logic_name;
    while ($sth->fetch) {
        $count++;
        my $mer_length = length($hid);
        my $rep_length = $end - $start + 1;
        
        $repeat_ace .= sprintf(
            qq{Feature "%s" %d %d %f "%.1f copies %d mer %s"\n},
            $feature_name, $start, $end, $score, ($rep_length / $mer_length), $mer_length, $hid
            );
    }
    
    if ($count) {
        return $repeat_ace;
    } else {
        return '';
    }
}

1;

__END__

=head1 NAME - Bio::EnsEMBL::Ace::Filter::TRF

=head1 AUTHOR

James Gilbert B<email> jgrg@sanger.ac.uk

