
### Bio::EnsEMBL::Ace::Filter::Gene::Halfwise

package Bio::EnsEMBL::Ace::Filter::Gene::Halfwise;

use strict;
use Carp;

use Bio::EnsEMBL::Ace::Filter::Gene;
use vars '@ISA';
@ISA = ('Bio::EnsEMBL::Ace::Filter::Gene');

sub new {
    my ($class) = @_;

    my $self = bless {}, $class;

    $self->db_connect();
    
    return $self;
}

sub ace_data {
    my ( $self, $contig ) = @_;

    my @transcripts = $self->get_transcripts($contig);
    my $contig_id = $contig->id;

    my $ace_data = '';
    foreach my $transcript (@transcripts) {

        my ($start_not_found, $end_not_found, @exons) = $self->transcript_exons_in_contig( $contig, $transcript )
          or confess "no exons in contig";
        my $pfam_id = $self->pfam_id( $exons[0] );
        my $count   = $self->next_gene_int;
        my $stable_id = "$contig_id.$pfam_id.$count";
        $transcript->stable_id($stable_id);
        $transcript->gene_name($pfam_id);
        
        my $description = $self->fetch_pfam_description($pfam_id);
        $transcript->description($description);

        # Hack to get round exons where start == end!
        # (Bad GeneWise!)
        foreach my $exon (@exons) {
            my $end = $exon->end;
            if ($exon->start == $end) {
                warn "Resetting end of exon in '$stable_id' where start = end = $end\n";
                $exon->end($end + 3);
            }
        }
                                
    }

    $ace_data .= $self->ace_data_from_transcripts( $contig, \@transcripts );

    return $ace_data;

}

sub fetch_pfam_description {
    my( $self, $pfam_id ) = @_;
    
    my $query = "SELECT description FROM pfamA WHERE pfamA_acc = '$pfam_id'";
    my $sth   = $self->db_connect->prepare($query);
    $sth->execute();
    my ($description) = $sth->fetchrow;
    return $description;
}

sub invariant_ace_data {
    my ( $self, $contig, $transcript ) = @_;

    ### Ick!  Shouldn't have to parse PFAM id out again when we just put it in!
    my ($pfam_id) = $transcript->stable_id =~ /(\w+).\w+$/;
    
    my $query = "SELECT description FROM pfamA WHERE pfamA_acc = '$pfam_id'";
    my $sth   = $self->db_connect->prepare($query);
    $sth->execute();
    my ($description) = $sth->fetchrow;
    return qq{Method HALFWISE\nDatabase PFAM $pfam_id\nWeb_Location "Pfam-Sanger"\nRemark "$description"\nCDS\nStart_not_found\nEnd_not_found\nCDS_predicted_by genewise\n};

}

sub pfam_id {
    my ( $self, $exon ) = @_;
    
    
    if ($exon) {
        my $expected = 'Bio::EnsEMBL::Exon';
        confess("Not a '$expected' : '$exon'") unless $exon->isa($expected);

        $exon->adaptor->fetch_evidence_by_Exon($exon);
        my @supp_features = $exon->each_Supporting_Feature
          or confess "No supporting features for exon";

        my $pfid = $supp_features[0]->hseqname
          or confess "No hseqname in supporting feature";
        return $pfid;
    }
    else {
        confess "No Exon given";
    }
}

sub db_connect {
    my ($self) = @_;

    my ($dbh);
    unless ( $dbh = $self->{'_db_connection'} ) {
        eval {
	    # change for vegadb
            my $user     = 'ensro';
            my $password = '';
            my $host     = 'vegadb.sanger.ac.uk';
            my $dbname   = 'pfamlive';
            my $port     = 3306;

            $dbh =
              DBI->connect( "DBI:mysql:host=$host;port=$port;database=$dbname",
                $user, $password, { RaiseError => 1 } );

            $self->{'_db_connection'} = $dbh;
        };
        if ($@) { die $@; }
    }
    
    return $dbh;
}


1;

__END__

=head1 NAME - Bio::EnsEMBL::Ace::Filter::Gene::Halfwise

=head1 AUTHOR

James Gilbert B<email> jgrg@sanger.ac.uk

